/*						    */
/*             SPECTRUM ANALYSER		    */
/*             -----------------		    */
/*
 
 
   People who have contributed to this code :
  
	Chris Plack		- chrisp@epunix.susx.ac.uk


*/

#include    <X11/Intrinsic.h>
#include    <Xm/Xm.h>
#include    <Xm/FileSB.h>

#include <stdio.h>
#include <gl.h>
#include <device.h>
#include <fmclient.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <sys/types.h>
#include <sys/prctl.h>
#include <sys/dir.h>
#include <string.h>
#include <audio.h>
#include <audiofile.h>



typedef short 	SAMPLE;	


/*	fft code				*/

extern 	float 	spectrum ( SAMPLE *, int , float *, int);

/*	subroutines				*/

static	void	clear_display (void);
static	void	process_buffer (void);
static	void	get_spectrum (void);
static	void	play_selection (void);
static	void	write_trigger_level (void);
static	void	write_input_atten (void);
static	void	set_attenuators (void);
static	void	set_sample_rate (void);
static	void	get_spectrum_nrt (void);
static	void	set_time_annint (void);
static	void	set_freq_annint (void);
void		get_file ();
void		get_file_callback ();
void		cancel_callback ();
static	void	open_audio_output (void);
static	void	close_audio_output (void);

/*	constants				*/

#define		ROOT_TWO		1.414213562
#define		PI			3.14159265358979

/*	audio parameters			*/

#define		Q_SIZE			10000
#define		APPROX_BUFFER_SIZE	(800*input_channels)	/* input buffer from Q */
#define		MIC_CALIBRATION		19.8	/* +ve values for increased internal level */
#define		LINE_CALIBRATION	-8.2
#define		FILE_CALIBRATION	0	/* 6.3 */
#define		SR_CORRECTION		1.00558	/* sample rate error correction */	
#define		MAX_VOLTAGE_IN		8.4	/* peak to peak, for line-level input */
#define		POST_TRIGGER_PROPORTION 0.75	/* of screen size */
#define		PRE_EMPHASIS		6.0	/* dB per octave for speech spectrogram */				

/*	windows					*/

#define		RECTANGULAR		1
#define		HAMMING			2
#define		HANNING			3
#define		BARTLETT		4
#define		BLACKMAN		5
#define		BLACKMAN_HARRIS		6

/*	input DC correction			*/
/*	for DC_DEFAULT				*/

#define		DC_MIC_0		-2713
#define		DC_MIC_10		-651 
#define		DC_MIC_20		84  
#define		DC_LINE_0		-5876
#define		DC_LINE_10		-1787 
#define		DC_LINE_20		-197 

/*	DC correction options			*/

#define		DC_DEFAULT		0
#define		DC_AUTO			1
#define		DC_KEEP			2
#define		DC_OFF			3

/*	display constraints			*/

#define		MAX_DISPLAY_SECONDS		4
#define		MAX_DISPLAY_KHZ			20
#define		APPROX_PIXELS_PER_RMS_POINT	2

/*	axes lengths and offsets		*/

#define		DEFAULT_XOL	70
#define		DEFAULT_XL	800
#define		DEFAULT_XOR	50
#define		DEFAULT_YOB	60
#define		DEFAULT_YL	300
#define		DEFAULT_YOT	50
#define		WATER_OFF	300

/*	screen dimensions			*/

#define		TIME_X_SCREEN_SIZE	(tdis.x_offset_left + tdis.x_length + tdis.x_offset_right)
#define		TIME_Y_SCREEN_SIZE	(tdis.y_offset_bottom + tdis.y_length + tdis.y_offset_top)
#define		FREQ_X_SCREEN_SIZE	(fdis.x_offset_left + fdis.x_length + fdis.x_offset_right)
#define		FREQ_Y_SCREEN_SIZE	(fdis.y_offset_bottom + fdis.y_length + fdis.y_offset_top)
#define		SPEC_X_SCREEN_SIZE	(sdis.x_offset_left + sdis.x_length + sdis.x_offset_right)
#define		SPEC_Y_SCREEN_SIZE	(sdis.y_offset_bottom + sdis.y_length + sdis.y_offset_top)
#define		WATER_X_SCREEN_SIZE	(wdis.x_offset_left + wdis.x_length + wdis.x_offset_right)
#define		WATER_Y_SCREEN_SIZE	(wdis.y_offset_bottom + wdis.y_length + wdis.y_offset_top)

/*	line widths				*/

#define		AXIS_WIDTH		1
#define		GRID_WIDTH		1
#define		DATA_WIDTH		1

/*	directions				*/

#define		UP			1
#define		DOWN			0
#define		LEFT			2
#define		RIGHT			3

/*	button dimensions			*/

#define		BUTTON_WIDTH		12
#define		TAXIS_INC_BUTTON_X	18
#define		TAXIS_DEC_BUTTON_X	18
#define		BAXIS_INC_BUTTON_X	18
#define		BAXIS_DEC_BUTTON_X	18
#define		TAXIS_INC_BUTTON_Y	360
#define		TAXIS_DEC_BUTTON_Y	347
#define		BAXIS_INC_BUTTON_Y	60
#define		BAXIS_DEC_BUTTON_Y	47
#define		RAXIS_INC_BUTTON_X	870
#define		RAXIS_DEC_BUTTON_X	857
#define		RAXIS_INC_BUTTON_Y	18
#define		RAXIS_DEC_BUTTON_Y	18

#define		TRIGGER_INC_BUTTON_X	1020
#define		TRIGGER_DEC_BUTTON_X	1020
#define		TRIGGER_INC_BUTTON_Y	300
#define		TRIGGER_DEC_BUTTON_Y	287
#define		ATTEN_INC_BUTTON_X	1020
#define		ATTEN_DEC_BUTTON_X	1020
#define		ATTEN_INC_BUTTON_Y	260
#define		ATTEN_DEC_BUTTON_Y	247

/*	font sizes				*/

#define		FONT_SIZE_SMALL		8.0
#define		FONT_SIZE_MEDIUM	10.0

/*	color definitions			*/

#define		BACKGROUND_COLOR	0x0f0000
#define		DISPLAY_COLOR		0x000000
#define		WATER_COLOR		0x000000
#define		WHITE1			0xffffff
#define		AXIS_COLOR		0x00ff00
#define		GRID_COLOR		0x00007f
#define		BUTTON_COLOR		0x7f007f
#define		CURSOR_COLOR		0x00ffff
#define		DATA_COLOR		0xffffff
#define		SELECTED_COLOR		0x3f0000
#define		SELECTED_COLOR2		0xff0000

/*	useful shorthands			*/

#define		PIXELS_PER_SECOND	((float)tdis.x_length / (time_axis_end - time_axis_start))
#define		PIXELS_PER_TDB		((float)tdis.y_length / (tlevel_axis_end - tlevel_axis_start))	
#define		PIXELS_PER_KHZ		((float)fdis.x_length / (freq_axis_end - freq_axis_start))
#define		PIXELS_PER_FDB		((float)fdis.y_length / (flevel_axis_end - flevel_axis_start))
#define		SPIXELS_PER_KHZ		((float)sdis.y_length / (spec_axis_end - spec_axis_start))
#define		SAMPLES_PER_PIXEL	((float)input_sample_rate / PIXELS_PER_SECOND)

/* 	switches				*/

short	begin_new_input = TRUE;
short	time_selected = FALSE;	/* selection on time axis */
short	freq_selected = FALSE;	/* selection on freq axis */
short	new_fft_selection = FALSE;
short	new_play_selection = FALSE;
short	pause = FALSE;		/* pauses audio input */
short	time_cursor_on = FALSE;
short	freq_cursor_on = FALSE;
short	spectrum_level = FALSE;
short	time_on = TRUE;
short	spectrum_on = FALSE;
short	spectrogram_on = FALSE;
short	waterfall_on = FALSE;
short	waterfall_nrt = FALSE;
short	spec_average_on = FALSE;
short	excitation_pattern_on = FALSE;
short	smooth_spec = FALSE;
short	average_first_time = TRUE;
short	trigger_on = FALSE, triggered = FALSE;
short	rms_on = TRUE;
short	envelope_on = TRUE;
short	line_input = FALSE;
short	dc_correction = DC_AUTO;
short	file_data = FALSE;
short	enhance_on = FALSE;	/* 3-dB per octave spectrogram enhancement */
short	color_on = FALSE;	/* spectrogram color mode */
short	invert_colors = FALSE;  /* invert spectrogram colors */
short	scrolling_on = FALSE;	/* scroll vs. roll spectrogram */
short	reset_spectro = TRUE;	/* reinitialises spectrogram scrolling */
short	single_scroll = FALSE;

/*	data selection				*/

int	tpixels_selected_start = 0, tpixels_selected_end = 0;
int	fpixels_selected_start = 0, fpixels_selected_end = 0;
float	time_selected_start = 0.0, time_selected_end = 0.0;
float	freq_selected_start = 0.0, freq_selected_end = 0.0;

/*	axes ranges				*/

float	time_axis_start = 0.0, time_axis_end = 4.0;		/* seconds */
float	old_time_axis_start[200], old_time_axis_end[200];
float	last_input_end_time;
float	tlevel_axis_start = 40.0, tlevel_axis_end = 120.0;	/* dB */
float	tlevel_axis_start_old = 30.0;
float	freq_axis_start = 0.0, freq_axis_end = 10.0;		/* kHz */
float	old_freq_axis_start[200], old_freq_axis_end[200];
float	flevel_axis_start = 0.0, flevel_axis_end = 100.0;	/* dB */
float	spec_axis_start = 0.0, spec_axis_end = 10.0;		/* kHz */	

/*	intervals between annotations		*/

float	time_ann_int = 0.5;					/* seconds */
float	tlevel_ann_int = 20.0;					/* dB */
float	freq_ann_int = 2.0;					/* kHz */
float	flevel_ann_int = 20.0;					/* dB */
float	spec_ann_int = 1.0;					/* kHz */

/*	zoom stuff				*/

short	tz_count = 0, fz_count = 0; 

/*	audio parameters			*/

float		input_sample_rate = 16000.0, sample_rate_old = 16000.0, output_sample_rate;
long		input_channels = AL_STEREO;
char		input_channel = 'R';
long		input_buffer_size, time_points_per_buffer,  samples_per_time_point;
long		input_sum = 0;
long		freq_components, spec_components;
long		spec_fft_points = 512;
long		spec_overlap = 1;
long		ffts_per_buffer;
float		approx_buffer_size;
float		seconds_per_fft, pixels_per_fft;
float		input_atten = 0.0, level_scale = 0.0, amp_scale = 1.0, dc_shift = 0.0;
float		dc = 0.0;
float		trigger_level = 30.0;
float		spec_boost = 0.2 * 32768.0 / MAX_VOLTAGE_IN;
ALport 		input_port, output_port;
ALconfig 	input_config, output_config;

/*	windowing				*/

short		window_type = RECTANGULAR;
short		window_type_old = 0;
int		window_size_old = 0;
float		*window_array;

/*	data arrays				*/

SAMPLE		input_buffer[Q_SIZE*2];
SAMPLE		main_time_buffer[MAX_DISPLAY_SECONDS*50000], *main_time_ptr;
float		time_display_buffer[DEFAULT_XL], *time_display_ptr;
float		time_max_buffer[DEFAULT_XL], *time_max_ptr;
float		time_min_buffer[DEFAULT_XL], *time_min_ptr;
float		freq_data[MAX_DISPLAY_SECONDS*50000];
float		spec_data[50][5000];
float		power_spectrum[5000];
float		enhance_array[MAX_DISPLAY_SECONDS*50000];

/*	pop-up menus				*/

long		time_menu = NULL, time_submenu1, time_submenu2, time_submenu3;
long		time_submenu4, time_submenu5, time_submenu6; 
long		freq_menu, freq_submenu1, freq_submenu2, freq_submenu3, freq_submenu4;
long		spec_menu, spec_submenu1, spec_submenu2, spec_submenu3;

/*	window and display parameters		*/

long		time_points_per_display, samples_per_display;
float		seconds_per_time_point, pixels_per_time_point;
long		components_per_display, freq_points_per_display;
long		scomponents_per_display;
float		kHz_per_component, pixels_per_component;
float		skHz_per_component, spixels_per_component;
float		kHz_per_freq_point, pixels_per_freq_point;
float		kHz_per_spec_point,  pixels_per_spec_point;

struct 	window_info {
		long	gid;
	} twin, fwin, swin, wwin;

struct	display_info {
		long 	x_offset_left;
		long	x_length;
		long   	x_offset_right;
		long	y_offset_bottom;
		long	y_length;
		long	y_offset_top;
	} tdis, fdis, sdis, wdis;

/***************************************************************************/

static	void	set_tdisplay_parameters (void)

{
	int	i;
	
	samples_per_display = (long) ((float)input_sample_rate 
					* (time_axis_end - time_axis_start));
	time_points_per_display = (long) ((float)tdis.x_length 
					/ (float) APPROX_PIXELS_PER_RMS_POINT);
	samples_per_time_point = (long) ((float)samples_per_display 
					/ (float)time_points_per_display);
	if (samples_per_time_point < 1) samples_per_time_point = 1;
	time_points_per_display = (long) ((float)samples_per_display
					/ (float)samples_per_time_point); 
	for (i = 1; (input_buffer_size = samples_per_time_point * i)
					< approx_buffer_size; i++);
	input_buffer_size = input_buffer_size * input_channels;
	time_points_per_buffer = i;
	seconds_per_time_point = (time_axis_end - time_axis_start) 
					/ (float)time_points_per_display;
	pixels_per_time_point = seconds_per_time_point * PIXELS_PER_SECOND;

	if ((time_axis_end - time_axis_start) > 4.0) time_ann_int = 1.0;
	else if ((time_axis_end - time_axis_start) > 2.0) time_ann_int = 0.5;
	else if ((time_axis_end - time_axis_start) > 1.0) time_ann_int = 0.2;
	else if ((time_axis_end - time_axis_start) > 0.5) time_ann_int = 0.1;
	else if ((time_axis_end - time_axis_start) > 0.2) time_ann_int = 0.05;
	else if ((time_axis_end - time_axis_start) > 0.1) time_ann_int = 0.02;
	else if ((time_axis_end - time_axis_start) > 0.05) time_ann_int = 0.01;
	else if ((time_axis_end - time_axis_start) > 0.02) time_ann_int = 0.005;
	else if ((time_axis_end - time_axis_start) > 0.01) time_ann_int = 0.002;
	else if ((time_axis_end - time_axis_start) > 0.005) time_ann_int = 0.001;
	else if ((time_axis_end - time_axis_start) > 0.002) time_ann_int = 0.0005;
	else time_ann_int = 0.0002;
}

/***************************************************************************/

static	void	set_fdisplay_parameters (void)

{
	int	i;
	
	kHz_per_component = ((float)input_sample_rate / 2000.0) / (float)freq_components;
	components_per_display = (long)((freq_axis_end - freq_axis_start) 
			/ kHz_per_component); 
	pixels_per_component = kHz_per_component * PIXELS_PER_KHZ;

	if ((freq_axis_end - freq_axis_start) > 20.0) freq_ann_int = 4.0;
	else if ((freq_axis_end - freq_axis_start) > 10.0) freq_ann_int = 2.0;
	else if ((freq_axis_end - freq_axis_start) > 5.0) freq_ann_int = 1.0;
	else if ((freq_axis_end - freq_axis_start) > 2.0) freq_ann_int = 0.5;
	else if ((freq_axis_end - freq_axis_start) > 1.0) freq_ann_int = 0.2;
	else if ((freq_axis_end - freq_axis_start) > 0.5) freq_ann_int = 0.1;
	else if ((freq_axis_end - freq_axis_start) > 0.2) freq_ann_int = 0.05;
	else if ((freq_axis_end - freq_axis_start) > 0.1) freq_ann_int = 0.02;
	else if ((freq_axis_end - freq_axis_start) > 0.05) freq_ann_int = 0.01;
	else if ((freq_axis_end - freq_axis_start) > 0.02) freq_ann_int = 0.005;
	else if ((freq_axis_end - freq_axis_start) > 0.01) freq_ann_int = 0.002;
	else if ((freq_axis_end - freq_axis_start) > 0.005) freq_ann_int = 0.001;
	else if ((freq_axis_end - freq_axis_start) > 0.002) freq_ann_int = 0.0005;
	else freq_ann_int = 0.0002;
}

/***************************************************************************/

static	void	set_sdisplay_parameters (void)

{
	int	i;
	
	skHz_per_component = ((float)input_sample_rate / 2000.0) / (float)spec_components;
	seconds_per_fft = (float)spec_fft_points / (float)input_sample_rate; 
	pixels_per_fft = seconds_per_fft * PIXELS_PER_SECOND;
	scomponents_per_display = (long)((spec_axis_end - spec_axis_start) 
			/ skHz_per_component); 
	spixels_per_component = skHz_per_component * SPIXELS_PER_KHZ;
}

/***************************************************************************/

static 	void	initialise (void)

{
	set_sample_rate ();
	approx_buffer_size = APPROX_BUFFER_SIZE;

/*	set axes positions and lengths		*/


	tdis.x_offset_left = DEFAULT_XOL; 
	tdis.x_length = DEFAULT_XL; 
	tdis.x_offset_right = DEFAULT_XOR + 130; 
	tdis.y_offset_bottom = DEFAULT_YOB;
	tdis.y_length = DEFAULT_YL;
	tdis.y_offset_top = DEFAULT_YOT;
	fdis.x_offset_left = DEFAULT_XOL; 
	fdis.x_length = DEFAULT_XL; 
	fdis.x_offset_right = DEFAULT_XOR; 
	fdis.y_offset_bottom = DEFAULT_YOB;
	fdis.y_length = DEFAULT_YL;
	fdis.y_offset_top = DEFAULT_YOT;
	sdis.x_offset_left = DEFAULT_XOL; 
	sdis.x_length = DEFAULT_XL; 
	sdis.x_offset_right = DEFAULT_XOR; 
	sdis.y_offset_bottom = DEFAULT_YOB;
	sdis.y_length = DEFAULT_YL;
	sdis.y_offset_top = DEFAULT_YOT;
	wdis.x_offset_left = DEFAULT_XOL; 
	wdis.x_length = DEFAULT_XL + WATER_OFF; 
	wdis.x_offset_right = DEFAULT_XOR; 
	wdis.y_offset_bottom = DEFAULT_XOL;
	wdis.y_length = DEFAULT_XL;
	wdis.y_offset_top = DEFAULT_XOR;
	

/*	set parameters (buffer size etc.)	*/ 
 
 	set_tdisplay_parameters ();
	set_time_annint ();
	set_freq_annint ();
 
/*	set up input devices			*/ 
 
    	qdevice (ESCKEY);
    	qdevice (WINQUIT);
    	qdevice (WINSHUT);
    	qdevice (WINFREEZE);
    	qdevice (WINTHAW);
/*	qdevice (MOUSEX);
	qdevice (MOUSEY); */
    	qdevice (LEFTMOUSE);
    	qdevice (MIDDLEMOUSE);
    	qdevice (RIGHTMOUSE);
	qdevice (SPACEKEY); 
}

/***************************************************************************/

static	void	clear_screen (void)

{	
	cpack (BACKGROUND_COLOR);
	if (winget () == twin.gid)	
		rectfi (0, 0, TIME_X_SCREEN_SIZE, TIME_Y_SCREEN_SIZE);
	else if (winget () == fwin.gid)	
		rectfi (0, 0, FREQ_X_SCREEN_SIZE, FREQ_Y_SCREEN_SIZE);
	else if (winget () == swin.gid)	
		rectfi (0, 0, SPEC_X_SCREEN_SIZE, SPEC_Y_SCREEN_SIZE);
	else if (winget () == wwin.gid) {	
		cpack (WATER_COLOR);
		rectfi (0, 0, WATER_X_SCREEN_SIZE, WATER_Y_SCREEN_SIZE);
	}
	clear_display ();
}

/***************************************************************************/

static	void	clear_display (void)

{	
	cpack (DISPLAY_COLOR);
	if (winget () == twin.gid){
 		rectfi (tdis.x_offset_left + AXIS_WIDTH, 
				tdis.y_offset_bottom + AXIS_WIDTH, 
				tdis.x_offset_left + tdis.x_length - 1,
				tdis.y_offset_bottom + tdis.y_length - 1);
		if (time_cursor_on) {
			cpack (BACKGROUND_COLOR);
			rectfi (tdis.x_offset_left,
				tdis.y_offset_bottom + tdis.y_length + 10,
				300 + tdis.x_offset_left,
				30 + tdis.y_offset_bottom + tdis.y_length + 10);
			time_cursor_on = FALSE;
		}
	}	

	else if (winget () == fwin.gid) {
		rectfi (fdis.x_offset_left + AXIS_WIDTH, 
				fdis.y_offset_bottom + AXIS_WIDTH, 
				fdis.x_offset_left + fdis.x_length - 1,
				fdis.y_offset_bottom + fdis.y_length - 1);
		if (freq_cursor_on) {
			cpack (BACKGROUND_COLOR);
			rectfi (fdis.x_offset_left,
				fdis.y_offset_bottom + fdis.y_length + 10,
				300 + fdis.x_offset_left,
				30 + fdis.y_offset_bottom + fdis.y_length + 10);
			freq_cursor_on = FALSE;
		}
	}
	else if (winget () == swin.gid) {
	    if (invert_colors) cpack (WHITE1);
	    rectfi (fdis.x_offset_left + AXIS_WIDTH, 
				fdis.y_offset_bottom + AXIS_WIDTH, 
				fdis.x_offset_left + fdis.x_length - 1,
				fdis.y_offset_bottom + fdis.y_length - 1);
	}
	else if (winget () == wwin.gid) {
	    cpack (WATER_COLOR);
	    rectfi (wdis.x_offset_left + AXIS_WIDTH, 
				wdis.y_offset_bottom + AXIS_WIDTH, 
				wdis.x_offset_left + wdis.x_length - 1,
				wdis.y_offset_bottom + wdis.y_length - 1);
	}	
}

/***************************************************************************/

static	void	draw_button (long xpos, long ypos, long button_width, short direction) 

{
	long		vert[2];
	
	cpack (0x3f3f3f); 
	rectfi (xpos, ypos, xpos + button_width, ypos + button_width);
	cpack (0x000000);
	bgnline ();
	    vert[0] = xpos + button_width;
	    vert[1] = ypos + button_width;
	    v2i (vert);
	    vert[1] = ypos;
	    v2i (vert);
	    vert[0] = xpos;
	    v2i (vert);
	endline ();
	cpack (0xffffff);
	bgnline ();
	    vert[0] = xpos + button_width;
	    vert[1] = ypos + button_width;
	    v2i (vert);
	    vert[0] = xpos;
	    v2i (vert);
	    vert[1] = ypos;
	    v2i (vert);
	endline ();
	
	cpack (BUTTON_COLOR);
	if (direction == UP) {
	    bgnpolygon ();
		vert[0] = xpos + 1;
		vert[1] = ypos + 1;
		v2i (vert);
		vert[0] = xpos + button_width - 1;
		v2i (vert);
		vert[0] = xpos + button_width / 2;
		vert[1] = ypos + button_width - 1;
		v2i (vert);
	    endpolygon (); 
	}
	else if (direction == DOWN){
	    bgnpolygon ();
		vert[0] = xpos + 1;
		vert[1] = ypos + button_width - 1;
		v2i (vert);
		vert[0] = xpos + button_width - 1;
		v2i (vert);
		vert[0] = xpos + button_width / 2;
		vert[1] = ypos + 1;
		v2i (vert);
	    endpolygon ();
	}
	else if (direction == LEFT){
	    bgnpolygon ();
		vert[0] = xpos + button_width - 1;
		vert[1] = ypos + button_width - 1;
		v2i (vert);
		vert[1] = ypos + 1;
		v2i (vert);
		vert[0] = xpos + 1;
		vert[1] = ypos + button_width / 2;
		v2i (vert);
	    endpolygon ();
	}
	else if (direction == RIGHT){
	    bgnpolygon ();
		vert[0] = xpos + 1;
		vert[1] = ypos + button_width - 1;
		v2i (vert);
		vert[1] = ypos + 1;
		v2i (vert);
		vert[0] = xpos + button_width - 1;
		vert[1] = ypos + button_width / 2;
		v2i (vert);
	    endpolygon ();
	}
}

/***************************************************************************/

static	void	draw_axis_buttons (void)

{
	if (winget () == wwin.gid) return;

	draw_button (TAXIS_INC_BUTTON_X, TAXIS_INC_BUTTON_Y, BUTTON_WIDTH, UP);
	draw_button (TAXIS_DEC_BUTTON_X, TAXIS_DEC_BUTTON_Y, BUTTON_WIDTH, DOWN);
	draw_button (BAXIS_INC_BUTTON_X, BAXIS_INC_BUTTON_Y, BUTTON_WIDTH, UP);
	draw_button (BAXIS_DEC_BUTTON_X, BAXIS_DEC_BUTTON_Y, BUTTON_WIDTH, DOWN);
	draw_button (RAXIS_INC_BUTTON_X, RAXIS_INC_BUTTON_Y, BUTTON_WIDTH, RIGHT);
	draw_button (RAXIS_DEC_BUTTON_X, RAXIS_DEC_BUTTON_Y, BUTTON_WIDTH, LEFT);			
}

/***************************************************************************/

static	void	draw_input_buttons (void)

{
/*	trigger level buttons			*/

	draw_button (TRIGGER_INC_BUTTON_X, TRIGGER_INC_BUTTON_Y, BUTTON_WIDTH, TRUE);
	draw_button (TRIGGER_DEC_BUTTON_X, TRIGGER_DEC_BUTTON_Y, BUTTON_WIDTH, FALSE);
	write_trigger_level ();
			
/*	input attenuation buttons	*/

	draw_button (ATTEN_INC_BUTTON_X, ATTEN_INC_BUTTON_Y, BUTTON_WIDTH, TRUE); 
	draw_button (ATTEN_DEC_BUTTON_X, ATTEN_DEC_BUTTON_Y, BUTTON_WIDTH, FALSE);
	write_input_atten ();
}

/***************************************************************************/

static	void	draw_time_axes (void)

{
	int		xpos, ypos;
	short		plot_mv = FALSE;
	float		ap;
	double		matrix[3][2] = {{0,-FONT_SIZE_MEDIUM},
					{FONT_SIZE_MEDIUM,0},
					{0,0}};
	char		buffer[20];
	fmfonthandle	font1, font_scaled;

	fminit();
	if ((font1 = fmfindfont ("Times-Roman")) == 0) exit (1);

/*	draw axes				*/

	linewidth (AXIS_WIDTH);
	cpack (AXIS_COLOR);
	recti (tdis.x_offset_left, tdis.y_offset_bottom,
		tdis.x_offset_left + tdis.x_length, 
		tdis.y_offset_bottom + tdis.y_length);

/*	draw x axis annotations			*/

	font_scaled = fmscalefont (font1, FONT_SIZE_SMALL);
	fmsetfont (font_scaled);

	ypos = tdis.y_offset_bottom - 20;
	ap = (float)((int)((time_axis_start) / time_ann_int)) * time_ann_int;
	if (ap < time_axis_start) ap += time_ann_int;
	for (; ap <= time_axis_end + 0.00001; ap += time_ann_int){ 
		xpos = tdis.x_offset_left + 
			(int)(PIXELS_PER_SECOND * (ap - time_axis_start));
		cmov2i (xpos, ypos);
		if (time_ann_int < 0.001) sprintf (buffer, "%5.1f", ap * 1000.0);
		else sprintf (buffer, "%5.3f", ap);
		fmprstr (buffer);
	}

/*	draw y axis annotations			*/

	if ((tlevel_axis_end - tlevel_axis_start) <= 0.2) plot_mv = TRUE; 

	xpos = tdis.x_offset_left - 30;
	if ((tlevel_axis_end >= 1000.0) && (!rms_on)) xpos -= 10;
	ap = (float)((int)((tlevel_axis_start) / tlevel_ann_int)) * tlevel_ann_int;
	if (ap < tlevel_axis_start) ap += tlevel_ann_int;
	for (; ap <= tlevel_axis_end; ap += tlevel_ann_int){
		ypos = tdis.y_offset_bottom + 
			(int)(PIXELS_PER_TDB * (ap - tlevel_axis_start));
		
		cmov2i (xpos, ypos);
		if (rms_on) {
			if ((tlevel_axis_end - tlevel_axis_start) >= 10.0) sprintf (buffer, "%d", (int)ap);
			else sprintf (buffer, "%1.1f", ap);
		}
		else {
			if (plot_mv) {
				if (tlevel_ann_int < 0.001) sprintf (buffer, "%5.2f", ap * 1000.0);
				else sprintf (buffer, "%5.0f", ap * 1000.0); 
			}
			else {
			    if (tlevel_axis_end <= 10.0) sprintf (buffer, "%5.1f", ap);
			    else sprintf (buffer, "%d", (int)ap);
			}
		}
			
		fmprstr (buffer);
	}

/*	draw axis labels			*/

	font_scaled = fmscalefont (font1, FONT_SIZE_MEDIUM);
	fmsetfont (font_scaled);
	xpos = tdis.x_offset_left + (int)((float)tdis.x_length / 2.0) - 40;
	ypos = tdis.y_offset_bottom - 50.0;
	cmov2i (xpos, ypos);
	if (time_ann_int < 0.001) fmprstr ("Time (milliseconds)");
	else fmprstr ("Time (seconds)");

	font_scaled = fmmakefont (font1, matrix);
	fmsetfont (font_scaled);
	xpos = tdis.x_offset_left - 40.0;
	ypos = tdis.y_offset_bottom + (int)((float)tdis.y_length / 2.0) - 40.0;
	cmov2i (xpos, ypos);
	if (rms_on) fmprstr ("RMS Level (dB)");
	else{
		if (plot_mv) fmprstr ("Amplitude (mV)");
		else fmprstr ("Amplitude (V)");
	}
}

/***************************************************************************/

static	void	draw_freq_axes (void)

{
	int		xpos, ypos;
	float		ap;
	double		matrix[3][2] = {{0,-FONT_SIZE_MEDIUM},
					{FONT_SIZE_MEDIUM,0},
					{0,0}};
	char		buffer[20];
	fmfonthandle	font1, font_scaled;

	fminit();
	if ((font1 = fmfindfont ("Times-Roman")) == 0) exit (1);

/*	draw axes				*/

	linewidth (AXIS_WIDTH);
	cpack (AXIS_COLOR);
	recti (fdis.x_offset_left, fdis.y_offset_bottom,
		fdis.x_offset_left + fdis.x_length, 
		fdis.y_offset_bottom + fdis.y_length);

/*	draw x axis annotations			*/

	font_scaled = fmscalefont (font1, FONT_SIZE_SMALL);
	fmsetfont (font_scaled);

	ypos = fdis.y_offset_bottom - 20;
	ap = (float)((int)((freq_axis_start) / freq_ann_int)) * freq_ann_int;
	if (ap < freq_axis_start) ap += freq_ann_int;
	for (; ap <= freq_axis_end; ap += freq_ann_int){ 
		xpos = fdis.x_offset_left + 
			(int)(PIXELS_PER_KHZ * (ap - freq_axis_start));
		cmov2i (xpos, ypos);
		sprintf (buffer, "%0.1f", ap*1000.0); 
		fmprstr (buffer);
	}

/*	draw y axis annotations			*/

	xpos = fdis.x_offset_left - 30;
	ap = (float)((int)((flevel_axis_start)
		/ flevel_ann_int)) * flevel_ann_int;
	if (ap < flevel_axis_start) ap += flevel_ann_int;
	for (; ap <= flevel_axis_end; ap += flevel_ann_int){
		ypos = fdis.y_offset_bottom + 
			(int)(PIXELS_PER_FDB * (ap - flevel_axis_start));
		cmov2i (xpos, ypos);
		if ((flevel_axis_end - flevel_axis_start) >= 10.0) sprintf (buffer, "%d", (int)ap);
		else sprintf (buffer, "%1.1f", ap); 
		fmprstr (buffer);
	}

/*	draw axis labels			*/

	font_scaled = fmscalefont (font1, FONT_SIZE_MEDIUM);
	fmsetfont (font_scaled);
	xpos = fdis.x_offset_left + (int)((float)fdis.x_length / 2.0) - 40;
	ypos = fdis.y_offset_bottom - 50.0;
	cmov2i (xpos, ypos);
	fmprstr ("Frequency (Hz)");

	font_scaled = fmmakefont (font1, matrix);
	fmsetfont (font_scaled);
	xpos = fdis.x_offset_left - 40.0;
	ypos = fdis.y_offset_bottom + (int)((float)fdis.y_length / 2.0) - 40;
	cmov2i (xpos, ypos);
	fmprstr ("RMS Level (dB)");	
}

/***************************************************************************/

static	void	draw_spec_axes (void)

{
	int		xpos, ypos;
	short		plot_mv = FALSE;
	float		ap;
	double		matrix[3][2] = {{0,-FONT_SIZE_MEDIUM},
					{FONT_SIZE_MEDIUM,0},
					{0,0}};
	char		buffer[20];
	fmfonthandle	font1, font_scaled;

	fminit();
	if ((font1 = fmfindfont ("Times-Roman")) == 0) exit (1);

/*	draw axes				*/

	linewidth (AXIS_WIDTH);
	cpack (AXIS_COLOR);
	recti (sdis.x_offset_left, sdis.y_offset_bottom,
		sdis.x_offset_left + sdis.x_length, 
		sdis.y_offset_bottom + sdis.y_length);

/*	draw x axis annotations			*/

	font_scaled = fmscalefont (font1, FONT_SIZE_SMALL);
	fmsetfont (font_scaled);

	ypos = sdis.y_offset_bottom - 20;
	ap = (float)((int)((time_axis_start) / time_ann_int)) * time_ann_int;
	if (ap < time_axis_start) ap += time_ann_int;
	for (; ap <= time_axis_end + 0.00001; ap += time_ann_int) { 
		xpos = sdis.x_offset_left + 
			(int)(PIXELS_PER_SECOND * (ap - time_axis_start));
		cmov2i (xpos, ypos);
		if (time_ann_int < 0.001) sprintf (buffer, "%5.1f", ap * 1000.0);
		else sprintf (buffer, "%5.3f", ap); 
		fmprstr (buffer);
	}

/*	draw y axis annotations			*/

	if ((spec_axis_end - spec_axis_start) <= 0.2) plot_mv = TRUE; 

	xpos = sdis.x_offset_left - 30;
	ap = (float)((int)((spec_axis_start) / spec_ann_int)) * spec_ann_int;
	if (ap < spec_axis_start) ap += spec_ann_int;
	for (; ap <= spec_axis_end; ap += spec_ann_int) {
		ypos = sdis.y_offset_bottom + 
		    (int)(SPIXELS_PER_KHZ * (ap - spec_axis_start));
		cmov2i (xpos, ypos);
		sprintf (buffer, "%2.1f", ap);	
		fmprstr (buffer);
	}

/*	draw axis labels			*/

	font_scaled = fmscalefont (font1, FONT_SIZE_MEDIUM);
	fmsetfont (font_scaled);
	xpos = sdis.x_offset_left + (int)((float)sdis.x_length / 2.0) - 40;
	ypos = sdis.y_offset_bottom - 50.0;
	cmov2i (xpos, ypos);
	if (time_ann_int < 0.001) fmprstr ("Time (milliseconds)");
	else fmprstr ("Time (seconds)");

	font_scaled = fmmakefont (font1, matrix);
	fmsetfont (font_scaled);
	xpos = sdis.x_offset_left - 40.0;
	ypos = sdis.y_offset_bottom + (int)((float)sdis.y_length / 2.0) - 40;
	cmov2i (xpos, ypos);
	fmprstr ("Frequency (kHz)");
	
}

/***************************************************************************/

static	void	draw_water_axes (void)

{
	int		xpos, ypos;
	float		ap;
	short		vert[2];
	double		matrix[3][2] = {{0,-FONT_SIZE_MEDIUM},
					{FONT_SIZE_MEDIUM,0},
					{0,0}};
	char		buffer[20];
	fmfonthandle	font1, font_scaled;

	fminit();
	if ((font1 = fmfindfont ("Times-Roman")) == 0) exit (1);

/*	draw axes				*/

	linewidth (AXIS_WIDTH);
	cpack (AXIS_COLOR);
/*	bgnline ();
	vert[0] = wdis.x_offset_left + WATER_OFF;
	vert[1] = wdis.y_offset_bottom;
	v2s (vert);
	vert[0] += fdis.x_length;
	v2s (vert);
	endline ();   */

/*	draw x axis annotations			*/

	font_scaled = fmscalefont (font1, FONT_SIZE_SMALL);
	fmsetfont (font_scaled);

	ypos = wdis.y_offset_bottom - 20;
	ap = (float)((int)((freq_axis_start) / freq_ann_int)) * freq_ann_int;
	if (ap < freq_axis_start) ap += freq_ann_int;
	for (; ap <= freq_axis_end; ap += freq_ann_int){ 
		xpos = wdis.x_offset_left + WATER_OFF +
			(int)(PIXELS_PER_KHZ * (ap - freq_axis_start));
		cmov2i (xpos, ypos);
		sprintf (buffer, "%0.1f", ap*1000.0); 
		fmprstr (buffer);
	}

/*	draw y axis annotations			*/
/*
	xpos = fdis.x_offset_left - 30;
	ap = (float)((int)((flevel_axis_start)
		/ flevel_ann_int)) * flevel_ann_int;
	if (ap < flevel_axis_start) ap += flevel_ann_int;
	for (; ap <= flevel_axis_end; ap += flevel_ann_int){
		ypos = fdis.y_offset_bottom + 
			(int)(PIXELS_PER_FDB * (ap - flevel_axis_start));
		cmov2i (xpos, ypos);
		if ((flevel_axis_end - flevel_axis_start) >= 10.0) sprintf (buffer, "%d", (int)ap);
		else sprintf (buffer, "%1.1f", ap); 
		fmprstr (buffer);
	}
*/
/*	draw axis labels			*/

	font_scaled = fmscalefont (font1, FONT_SIZE_MEDIUM);
	fmsetfont (font_scaled);
	xpos = WATER_OFF + fdis.x_offset_left + (int)((float)fdis.x_length / 2.0) - 40;
	ypos = wdis.y_offset_bottom - 50.0;
	cmov2i (xpos, ypos);
	fmprstr ("Frequency (Hz)");

/*	font_scaled = fmmakefont (font1, matrix);
	fmsetfont (font_scaled);
	xpos = fdis.x_offset_left - 40.0;
	ypos = fdis.y_offset_bottom + (int)((float)fdis.y_length / 2.0) - 40;
	cmov2i (xpos, ypos);
	fmprstr ("RMS Level (dB)");	*/
}

/***************************************************************************/

static	void	draw_time_grid (void)

{
	long		vert1[2], vert2[2];
	float		ap;
	int		end;
		
	cpack (GRID_COLOR);
	
/*	draw vertical grid			*/

	end = tdis.x_offset_left + tdis.x_length - 1;
	vert1[1] = tdis.y_offset_bottom + AXIS_WIDTH;
	vert2[1] = tdis.y_offset_bottom + tdis.y_length - 1;
	ap = time_ann_int + (float)((int)((time_axis_start+0.000001)
		/ time_ann_int)) * time_ann_int;
	for (; ap < time_axis_end; ap += time_ann_int){
		vert1[0] = tdis.x_offset_left + 
			(int)(PIXELS_PER_SECOND * (ap - time_axis_start));
		if (vert1[0] < end) {
		    vert2[0] = vert1[0];
		    bgnline ();
			v2i (vert1);
			v2i (vert2);
		    endline ();
		}   
	}

/*	draw horizontal grid			*/

	vert1[0] = tdis.x_offset_left + AXIS_WIDTH;
	vert2[0] = tdis.x_offset_left + tdis.x_length - 1;
	ap = (float)((int)((tlevel_axis_start)
		/ tlevel_ann_int)) * tlevel_ann_int;
/*	if (ap <= tlevel_axis_start) ap += tlevel_ann_int;  */
	while ((int)(PIXELS_PER_TDB * (ap - tlevel_axis_start)) < AXIS_WIDTH) ap += tlevel_ann_int;
	for (; ap < tlevel_axis_end; ap += tlevel_ann_int){
		vert1[1] = tdis.y_offset_bottom + 
			(int)(PIXELS_PER_TDB * (ap - tlevel_axis_start));
		vert2[1] = vert1[1];
		bgnline ();
		    v2i (vert1);
		    v2i (vert2);
		endline ();
	}

}

/***************************************************************************/

static	void	draw_freq_grid (void)

{
	long		vert1[2], vert2[2];
	float		ap;
		
	cpack (GRID_COLOR);
	
/*	draw vertical grid			*/

	vert1[1] = fdis.y_offset_bottom + AXIS_WIDTH;
	vert2[1] = fdis.y_offset_bottom + fdis.y_length - 1;
	ap = freq_ann_int + (float)((int)((freq_axis_start + 0.0001)
		/ freq_ann_int)) * freq_ann_int;
	for (; ap < freq_axis_end; ap += freq_ann_int){
		vert1[0] = fdis.x_offset_left + 
			(int)(PIXELS_PER_KHZ * (ap - freq_axis_start));
		vert2[0] = vert1[0];
		bgnline ();
		    v2i (vert1);
		    v2i (vert2);
		endline ();   
	}

/*	draw horizontal grid			*/

	vert1[0] = fdis.x_offset_left + AXIS_WIDTH;
	vert2[0] = fdis.x_offset_left + fdis.x_length - 1;
	ap = (float)((int)((flevel_axis_start)
		/ flevel_ann_int)) * flevel_ann_int;
/*	if (ap <= flevel_axis_start) ap += flevel_ann_int; */
	while ((int)(PIXELS_PER_FDB * (ap - flevel_axis_start)) < AXIS_WIDTH) ap += flevel_ann_int;
	for (; ap < flevel_axis_end; ap += flevel_ann_int){
		vert1[1] = fdis.y_offset_bottom + 
			(int)(PIXELS_PER_FDB * (ap - flevel_axis_start));
		vert2[1] = vert1[1];
		bgnline ();
		    v2i (vert1);
		    v2i (vert2);
		endline ();
	}

}

/***************************************************************************/

static	void	write_selected_time ()

{
	char			buffer[50];
	long			xp, yp;
	static	fmfonthandle	font1, font_scaled;
	static	short		first_time = TRUE;

	if (first_time) {
		fminit();
		if ((font1 = fmfindfont ("Times-Roman")) == 0) exit (1);
		font_scaled = fmscalefont (font1, FONT_SIZE_SMALL);
		first_time = FALSE;
	}
	cpack (AXIS_COLOR);
	fmsetfont (font_scaled);
	xp = tdis.x_offset_left + tdis.x_length / 2;
	yp = tdis.y_offset_bottom + tdis.y_length + 20;
	cmov2i (xp, yp);
	sprintf (buffer, "Selected Time: %5.3f -> %5.3f sec"
				, time_selected_start, time_selected_end);
	fmprstr (buffer);
}

/***************************************************************************/

static	void	write_selected_freq (void)

{
	char			buffer[50];
	long			xp, yp;
	static	fmfonthandle	font1, font_scaled;
	static	short		first_time = TRUE;

	if (first_time) {
		fminit();
		if ((font1 = fmfindfont ("Times-Roman")) == 0) exit (1);
		font_scaled = fmscalefont (font1, FONT_SIZE_SMALL);
		first_time = FALSE;
	}
	cpack (AXIS_COLOR);
	fmsetfont (font_scaled);
	xp = fdis.x_offset_left + fdis.x_length / 2;
	yp = fdis.y_offset_bottom + fdis.y_length + 20;
	cmov2i (xp, yp);
	sprintf (buffer, "Selected Freq: %5.2f -> %5.2f Hz"
				, freq_selected_start * 1000.0, freq_selected_end * 1000.0);
	fmprstr (buffer);
}

/***************************************************************************/

static	void	write_trigger_level (void)

{
	char			buffer[50];
	long			xp, yp;
	static	fmfonthandle	font1, font_scaled;
	static	short		first_time = TRUE;

	if (first_time) {
		fminit();
		if ((font1 = fmfindfont ("Times-Roman")) == 0) exit (1);
		font_scaled = fmscalefont (font1, FONT_SIZE_SMALL);
		first_time = FALSE;
	}
	cpack (AXIS_COLOR);
	fmsetfont (font_scaled);
	xp = TRIGGER_INC_BUTTON_X - 120.0;
	yp = TRIGGER_INC_BUTTON_Y - 5;
	cmov2i (xp, yp);
	sprintf (buffer, "Trigger Level: %5.0f dB", trigger_level);
	fmprstr (buffer);
}

/***************************************************************************/

static	void	write_input_atten (void)

{
	char			buffer[50];
	long			xp, yp;
	static	fmfonthandle	font1, font_scaled;
	static	short		first_time = TRUE;

	if (first_time) {
		fminit();
		if ((font1 = fmfindfont ("Times-Roman")) == 0) exit (1);
		font_scaled = fmscalefont (font1, FONT_SIZE_SMALL);
		first_time = FALSE;
	}
	cpack (AXIS_COLOR);
	fmsetfont (font_scaled);
	xp = ATTEN_INC_BUTTON_X - 120.0;
	yp = ATTEN_INC_BUTTON_Y - 5;
	cmov2i (xp, yp);
	sprintf (buffer, "Input Atten: %5.0f dB", -input_atten);
	fmprstr (buffer);
}

/***************************************************************************/

static 	void	draw_display (void)

{
	clear_screen ();
	
	if (winget () == twin.gid) {
		draw_time_axes ();
		draw_input_buttons ();
		draw_time_grid ();
		if (time_selected) write_selected_time ();
	}
	else if (winget () == fwin.gid) {
		draw_freq_axes ();
		draw_freq_grid ();
		if (freq_selected) write_selected_freq ();		
	}
	else if (winget () == swin.gid) {
		clear_display ();
		draw_spec_axes ();		
	}
	else if (winget () == wwin.gid) {
		clear_display ();
		draw_water_axes ();		
	}
	draw_axis_buttons ();
}

/***************************************************************************/

static	void	plot_time_data (void)

{
	int	i, selected_start, selected_end;
	int	bottom, top, left, right;
	int	samples_per_display;
	int	vert_old = 0;
	long	idum, main_time_points;
	short	not_current_window = FALSE;
	long	current_gid;
	float	pixels_per_db;
	float	vert[2];
	float	*td_ptr, *end_of_td_array;
	SAMPLE	*mt_ptr, *end_of_mt_array;
	int	max = -100000, min = 100000;
	float	pixels_per_sample;
	

	if ((current_gid = winget ()) != twin.gid) {
		not_current_window = TRUE;
		winset (twin.gid);
	}
	if (time_selected) {
		cpack (SELECTED_COLOR);
		if (tpixels_selected_start == tpixels_selected_end)
				cpack (SELECTED_COLOR2);
		if (tpixels_selected_end > tdis.x_offset_left + tdis.x_length - 1)
		    selected_end = tdis.x_offset_left + tdis.x_length - 1;
		else selected_end = tpixels_selected_end;
		if (tpixels_selected_start < tdis.x_offset_left + AXIS_WIDTH)
		    selected_start = tdis.x_offset_left + AXIS_WIDTH;
		else selected_start = tpixels_selected_start;
			
		rectfi (selected_start, tdis.y_offset_bottom + AXIS_WIDTH,
				selected_end, tdis.y_offset_bottom + tdis.y_length - 1);
		cpack (DISPLAY_COLOR);
		if (selected_start > selected_end) {
			idum = selected_start;
			selected_start = selected_end;
			selected_end = idum + 1;
		}
		else {
			selected_start = selected_start;
			selected_end = selected_end + 1;
		}
		if (selected_start > tdis.x_offset_left + AXIS_WIDTH) {
		    rectfi (tdis.x_offset_left + AXIS_WIDTH, 
				tdis.y_offset_bottom + AXIS_WIDTH,
				selected_start - 1,
			 	tdis.y_offset_bottom + tdis.y_length - 1);
		}
		if (selected_end < tdis.x_offset_left + tdis.x_length - 1) {
		    rectfi (selected_end, tdis.y_offset_bottom + AXIS_WIDTH,
				tdis.x_offset_left + tdis.x_length - 1,
			 	tdis.y_offset_bottom + tdis.y_length - 1);
		}
		if (time_cursor_on) {
			cpack (BACKGROUND_COLOR);
			rectfi (tdis.x_offset_left,
				tdis.y_offset_bottom + tdis.y_length + 20,
				300 + tdis.x_offset_left,
				20 + tdis.y_offset_bottom + tdis.y_length + 20);
			time_cursor_on = FALSE;
		}
	}
	else clear_display ();

	draw_time_grid ();

	
	vert[0] = 1.0 + (float)tdis.x_offset_left + (float)tdis.x_length 
			- ((float)time_points_per_display * pixels_per_time_point);
	cpack (DATA_COLOR);
	
	pixels_per_db = PIXELS_PER_TDB;
	
	bottom = (tdis.y_offset_bottom + AXIS_WIDTH);
	top = (tdis.y_offset_bottom + tdis.y_length - 1);
	right = (tdis.x_offset_left + tdis.x_length - 1);
	left = (tdis.x_offset_left + AXIS_WIDTH);
	
	if ((!rms_on) && (!envelope_on)) {
	    mt_ptr = main_time_ptr;
	    main_time_points = (long) ((float)input_sample_rate * MAX_DISPLAY_SECONDS);
	    end_of_mt_array = &main_time_buffer[main_time_points-1];
	    mt_ptr -= (long)((last_input_end_time - time_axis_start) * (float)input_sample_rate);
	    if (mt_ptr < &main_time_buffer[0]) {
		idum = (long)end_of_mt_array + sizeof (SAMPLE) - (long)&main_time_buffer[0]; 
		mt_ptr += idum/sizeof (SAMPLE);
	    }
	    samples_per_display = time_points_per_display * samples_per_time_point;
	    pixels_per_sample = (float)tdis.x_length / (float)samples_per_display;
	    bgnline ();
	    if (samples_per_display > tdis.x_length * 2) {
		for (i = 0; i < samples_per_display + 2; i++) {
		
		    if (*mt_ptr > max) max = *mt_ptr;
		    if (*mt_ptr < min) min = *mt_ptr;
		
		    if ((int)vert[0] > vert_old) {   
			vert[1] = (((dc_shift + ((float)max * amp_scale)) - tlevel_axis_start) 
			    * pixels_per_db) + tdis.y_offset_bottom;
			if ((int)vert[1] < bottom) vert[1] = bottom;
			else if ((int)vert[1] >= top) vert[1] = top;
			v2f (vert);
			vert[1] = (((dc_shift + ((float)min * amp_scale)) - tlevel_axis_start) 
			    * pixels_per_db) + tdis.y_offset_bottom;
			if ((int)vert[1] < bottom) vert[1] = bottom;
			else if ((int)vert[1] >= top) vert[1] = top;
			v2f (vert);
			vert_old = (int) vert[0];
			max = -100000;
			min = 100000;
		    }
		    if (++mt_ptr > end_of_mt_array) mt_ptr = &main_time_buffer[0];
		    vert[0] += pixels_per_sample;
		    if (vert[0] > right) break;	
		}
	    }
	    else { 
		for (i = 0; i < samples_per_display + 2; i++) {
		    vert[1] = (((dc_shift + ((float)*mt_ptr * amp_scale)) - tlevel_axis_start) 
			    * pixels_per_db) + tdis.y_offset_bottom;
		    if ((int)vert[1] < bottom) vert[1] = bottom;
		    else if ((int)vert[1] >= top) vert[1] = top;
		    if (++mt_ptr > end_of_mt_array) mt_ptr = &main_time_buffer[0];
		    v2f (vert);
		    vert[0] += pixels_per_sample;
		    if (vert[0] > right) break;	
		}
	    }
	    endline ();
	}	   
	else {    
	
	    if (rms_on) {
		bgnline ();
		end_of_td_array = &time_display_buffer[time_points_per_display-1];
		td_ptr = time_display_ptr;
		if (vert[0] < left) {
		    if (++td_ptr > end_of_td_array) 
				td_ptr = &time_display_buffer[0];
		    vert[0] += pixels_per_time_point;
		}
		for (i = 0; i < time_points_per_display; i++){
		    vert[1] = (((*td_ptr) - tlevel_axis_start) * pixels_per_db) 
				+ tdis.y_offset_bottom;
		
		    if ((int)vert[1] < bottom) vert[1] = bottom;
		    else if ((int)vert[1] >= top) vert[1] = top;
				
		    if (++td_ptr > end_of_td_array) 
				td_ptr = &time_display_buffer[0];
		    v2f (vert);	
		    vert[0] += pixels_per_time_point;
		    if (vert[0] > right) break;
		}
		endline ();
	    }
	    else {
		vert[0] = 1.0 + (float)tdis.x_offset_left + (float)tdis.x_length 
			- ((float)time_points_per_display * pixels_per_time_point);
		bgnline ();
		end_of_td_array = &time_max_buffer[time_points_per_display-1];
		td_ptr = time_max_ptr;
		if (vert[0] < left) {
		    if (++td_ptr > end_of_td_array) 
				td_ptr = &time_display_buffer[0];
		    vert[0] += pixels_per_time_point;
		}
		for (i = 0; i < time_points_per_display; i++){
		    
		    vert[1] = (((*td_ptr) - tlevel_axis_start) * pixels_per_db) 
				+ tdis.y_offset_bottom;
		    if ((int)vert[1] < bottom) vert[1] = bottom;
		    else if ((int)vert[1] >= top) vert[1] = top;
				
		    if (++td_ptr > end_of_td_array) 
				td_ptr = &time_max_buffer[0];
		    v2f (vert);	
		    vert[0] += pixels_per_time_point;
		    if (vert[0] > right) break;
		}
		endline ();
		
		if (samples_per_time_point > 1) {
		
		    vert[0] = 1.0 + (float)tdis.x_offset_left + (float)tdis.x_length 
			- ((float)time_points_per_display * pixels_per_time_point);
		    bgnline ();
		    end_of_td_array = &time_min_buffer[time_points_per_display-1];
		    td_ptr = time_min_ptr;
		
		    if (vert[0] < left) {
			if (++td_ptr > end_of_td_array) 
				td_ptr = &time_display_buffer[0];
			vert[0] += pixels_per_time_point;
		    }
		    for (i = 0; i < time_points_per_display; i++){
			vert[1] = (((*td_ptr) - tlevel_axis_start) * pixels_per_db) 
				+ tdis.y_offset_bottom;
			if ((int)vert[1] < bottom) vert[1] = bottom;
			else if ((int)vert[1] >= top) vert[1] = top;
				
			if (++td_ptr > end_of_td_array) 
				td_ptr = &time_min_buffer[0];
			v2f (vert);	
			vert[0] += pixels_per_time_point;
			if (vert[0] > right) break;
		    }
		    endline ();
		}
	    }
	}
	if (not_current_window) winset (current_gid); 	
}

/***************************************************************************/

static	void	calc_freq_data (void)

{
	int	i, j, k, selected_start, selected_end, end, end2, start;
	int	components_per_point, half_comps;
	short	not_current_window = FALSE;
	long	idum, idum2;
	static	int	count=0;
	float	pixels_per_db, sum;
	float	*fd_ptr, *max_ptr, *pow_ptr, power_spectrum2[5000];
	float	f, erb, p;
	float	nyquist;

	
	nyquist = (float)input_sample_rate / 2.0;

	fd_ptr = &freq_data[0];
	
	pixels_per_db = PIXELS_PER_FDB;
	
	
	idum = (long)(freq_axis_start / kHz_per_component); 
	if (!excitation_pattern_on) fd_ptr += idum;
	else {
	    idum2 = (long)(freq_axis_start * 0.8 / kHz_per_component); 
	    fd_ptr += idum2;
	    
	}
	
	pow_ptr = &power_spectrum[0];	

	components_per_point = (int)((float)components_per_display * 2.0 / ((float)fdis.x_length));
	if (components_per_point < 1) components_per_point = 1;
	pixels_per_freq_point = pixels_per_component * (float)components_per_point;
	kHz_per_freq_point = pixels_per_freq_point / PIXELS_PER_KHZ;

	end = components_per_display /* - components_per_point */;
	if (end > freq_components - (idum + components_per_point*2))
	    end = freq_components - (idum + components_per_point*2); 
	if (excitation_pattern_on) {
	    end2 = (idum - idum2) + end * 1.2;
	    if (end2 > freq_components - (idum + components_per_point*2))
	    end2 = freq_components - (idum + components_per_point*2); 
	}  

	j = 0;
	if (!excitation_pattern_on) {
	    for (i = 0; i < end; pow_ptr++){
		
		sum = 0.0;
		
		if (spectrum_level) {
		    for (j = 0; j < components_per_point; j++, i++, fd_ptr++) 		
			    sum += (*fd_ptr); 
		    if (sum <= 0.0000000001) sum = 0.0000000001;		
		    *pow_ptr = level_scale + (10.0 * log10 (sum / ((float) components_per_point 
					* kHz_per_component * 1000.0)));
		}
		else {
		    if (components_per_point == 1) {
			sum = *fd_ptr++; i++;
		    }
		    else {
			for (j = 0; j < components_per_point; j++, i++, fd_ptr++) 		
			    if (*fd_ptr > sum) sum = (*fd_ptr);
	/*		if (max_ptr[-1] > max_ptr[1]) sum += max_ptr[-1], j = 0;
			else {
			    sum += max_ptr[1];
			    if (max_ptr = fd_ptr) j = 1, fd_ptr++, i++;
			    else j = 0;
			}    */
		    }  
		    if (sum <= 0.0000000001) sum = 0.0000000001;
		    *pow_ptr = level_scale + (10.0 * log10 (sum));
		}	
	    }   
	    freq_points_per_display = 1 + (int)(end / components_per_point);
	}
	else {
	    pow_ptr = &power_spectrum2[0];
	    for (i = 0; i < end2; pow_ptr++){
		sum = 0.0;
		
		for (j = 0; j < components_per_point; j++, i++, fd_ptr++) sum += (*fd_ptr); 
		if (spectrum_level) {		
		    *pow_ptr = sum / ((float) components_per_point * kHz_per_component * 1000.0);
		}
		else *pow_ptr = sum / ((float) components_per_point);
			
	    }
	    end2 /= components_per_point;
	    start = (idum - idum2) / components_per_point;
	    end /= components_per_point;
	    
	    for (i = 0; i < end; i++) {
		sum = 0.0;
		f = (i * kHz_per_freq_point) + freq_axis_start;
		erb = 24.7 * ((4.37 * f) + 1.0);
		erb /= (kHz_per_freq_point * 1000.0); 
		p = 4.0 / erb;
		half_comps = (int)(erb * 3);
		if (pause) half_comps *= 3;
		
		if ((i < half_comps - start) || (i > end2 - (half_comps + start))){ 
		    for (j = (i - half_comps) + start, k = -half_comps; j < i + half_comps + start; j++, k++) {
			if ((j >= 0) && (j < end2))  sum += power_spectrum2[j] * (1 + p * abs(k)) * exp (-p * abs(k));
		    }
		} 
		else {
		    for (j = (i - half_comps) + start, k = -half_comps; j < i + half_comps + start; j++, k++) {
			sum += power_spectrum2[j] * (1 + p * abs(k)) * exp (-p * abs(k));
		    }
		}  
		if (sum <= 0.0000000001) sum = 0.0000000001;
		power_spectrum[i] = level_scale + (10.0 * log10 (sum / erb));
	    }
	    freq_points_per_display = 1 + end;
	} 
		
}

/***************************************************************************/

static	void	plot_freq_data (void)

{
	int	i, selected_start, selected_end, end;
	int	components_per_point;
	int	bottom, top, right;
	short	not_current_window = FALSE;
	long	current_gid;
	long	idum;
	static	int	count=0;
	float	pixels_per_db;
	float	vert[2];
	float	*pow_ptr;
	float	nyquist;

	if ((current_gid = winget ()) != fwin.gid) {
		not_current_window = TRUE;
		winset (fwin.gid);
	}
	nyquist = (float)input_sample_rate / 2.0;

	if (freq_selected) {
		cpack (SELECTED_COLOR);
		if (fpixels_selected_start == fpixels_selected_end)
				cpack (SELECTED_COLOR2);
		if (fpixels_selected_end > fdis.x_offset_left + fdis.x_length - 1)
		    selected_end = fdis.x_offset_left + fdis.x_length - 1;
		else selected_end = fpixels_selected_end;
		if (fpixels_selected_start < fdis.x_offset_left + AXIS_WIDTH)
		    selected_start = fdis.x_offset_left + AXIS_WIDTH;
		else selected_start = fpixels_selected_start;
			
		rectfi (selected_start, fdis.y_offset_bottom + AXIS_WIDTH,
				selected_end, fdis.y_offset_bottom + fdis.y_length - 1);
		cpack (DISPLAY_COLOR);
		if (selected_start > selected_end) {
			idum = selected_start;
			selected_start = selected_end;
			selected_end = idum + 1;
		}
		else {
			selected_start = selected_start;
			selected_end = selected_end + 1;
		}
		if (selected_start > fdis.x_offset_left + AXIS_WIDTH) {
		    rectfi (fdis.x_offset_left + AXIS_WIDTH, 
				fdis.y_offset_bottom + AXIS_WIDTH,
				selected_start - 1,
			 	fdis.y_offset_bottom + tdis.y_length - 1);
		}
		if (selected_end < fdis.x_offset_left + fdis.x_length - 1) {
		    rectfi (selected_end, fdis.y_offset_bottom + AXIS_WIDTH,
				fdis.x_offset_left + fdis.x_length - 1,
			 	fdis.y_offset_bottom + fdis.y_length - 1);
		}
		if (freq_cursor_on) {
			cpack (BACKGROUND_COLOR);
			rectfi (fdis.x_offset_left,
				fdis.y_offset_bottom + fdis.y_length + 20,
				300 + fdis.x_offset_left,
				20 + fdis.y_offset_bottom + fdis.y_length + 20);
			freq_cursor_on = FALSE;
		}
	}
	else clear_display ();

	draw_freq_grid ();

	cpack (DATA_COLOR);
	bgnline ();
	pixels_per_db = PIXELS_PER_FDB;
	
	
	idum = (long)(freq_axis_start / kHz_per_component); 
	
	vert[0] = (float)fdis.x_offset_left + (((((float) idum + 1.0) * kHz_per_component)
			- freq_axis_start) * PIXELS_PER_KHZ);
	if ((int)vert[0] < fdis.x_offset_left + AXIS_WIDTH) vert[0] = fdis.x_offset_left + AXIS_WIDTH;

	pow_ptr = &power_spectrum[0];	

	components_per_point = (int)((float)components_per_display * 2.0 / ((float)fdis.x_length));
	if (components_per_point < 1) components_per_point = 1;
	pixels_per_freq_point = pixels_per_component * (float)components_per_point;
	kHz_per_freq_point = pixels_per_freq_point / PIXELS_PER_KHZ;

	end = components_per_display /* - components_per_point */;
	if (end > freq_components - (idum + components_per_point*2))
	    end = freq_components - (idum + components_per_point*2); 
	 

	bottom = (fdis.y_offset_bottom + AXIS_WIDTH);
	top = (fdis.y_offset_bottom + fdis.y_length - 1);
	right = (fdis.x_offset_left + fdis.x_length - 1);
	
	end /= components_per_point;
	
	for (i = 0; i < end; i++, pow_ptr++){
		
	    vert[1] = ((*pow_ptr - flevel_axis_start) * pixels_per_db) 
				+ fdis.y_offset_bottom;
	    if ((int)vert[1] < bottom) vert[1] = bottom;
	    else if ((int)vert[1] >= top) vert[1] = top;

	    v2f (vert);
	    vert[0] += pixels_per_freq_point;
	    if (vert[0] > (float)right) vert[0] = right;	
	}  
	endline ();	
	if (not_current_window) winset (current_gid);
}

/***************************************************************************/

static	void	plot_spec_data (void)	/* rolling display */ 

{
	int		i, j, k, m, n, start, end, end2, half_comps;
	int		components_per_point, axis_end;
	int		brightness;
	long		vert1[2], vert2[2];
	short	        not_current_window = FALSE;
	short		color_vec[3];
	static	short	color_vec_previous[500][3];		
	long		current_gid, idum, idum2;
	Screencoord	x1, y1, x2, y2, newx; 
	static	int 	count=0;
	float		pixels_per_buffer, sum, fdum;
	float		vert1_new;
	float		*sd_ptr, pow, boost;
	float		erb, f, p;
	float		*pow_ptr, power_spectrum1[5000], power_spectrum2[5000];
	static	float	error = 0.0;

	if ((current_gid = winget ()) != swin.gid) {
		not_current_window = TRUE;
		winset (swin.gid);
	}
	axis_end = sdis.y_length + sdis.y_offset_bottom;
	boost = spec_boost * amp_scale;

	pixels_per_buffer = (pixels_per_fft * (float)ffts_per_buffer / (float)spec_overlap) + error;
	if (pixels_per_buffer >= sdis.x_length - 2) {
		clear_display ();
		if (not_current_window) winset (current_gid);
		return;
	} 
	newx = sdis.x_offset_left + AXIS_WIDTH;
	error = pixels_per_buffer - (float)(int)pixels_per_buffer;
	x1 = newx + (int)(pixels_per_buffer);
	y1 = sdis.y_offset_bottom + AXIS_WIDTH;
	x2 = sdis.x_offset_left + sdis.x_length - 1;
	y2 = sdis.y_offset_bottom + sdis.y_length - 1;
	
	rectcopy (x1, y1, x2, y2, newx, y1);

	fdum = (float)x2 - (float)(int)pixels_per_buffer;

	idum = (long)(spec_axis_start / skHz_per_component);
	if (excitation_pattern_on) {
	    idum2 = (long)(spec_axis_start * 0.8 / skHz_per_component); 
	}
	components_per_point = (int)((float)scomponents_per_display * 2.0 / ((float)sdis.y_length));
	if (components_per_point < 1) components_per_point = 1;
	pixels_per_spec_point = spixels_per_component * (float)components_per_point;

	
	kHz_per_spec_point = skHz_per_component * components_per_point;

	if (scomponents_per_display > spec_components - (idum + components_per_point*2))
	    end = spec_components - (idum + components_per_point*2);
	else end = scomponents_per_display;
	if (excitation_pattern_on) {
	    end2 = (idum - idum2) + end * 1.2;
	    if (end2 > spec_components - (idum + components_per_point*2))
	    end2 = spec_components - (idum + components_per_point*2); 
	} 
	if (smooth_spec) shademodel(GOURAUD);
	
	for (m = 0; m < ffts_per_buffer; m++) {
	
	    sd_ptr = &spec_data[m][0]; 
	    
	    if (!excitation_pattern_on) sd_ptr += idum;
	    else sd_ptr += idum2;
	    
	    vert1[0] = (int)fdum;
	    vert1_new = (float)sdis.y_offset_bottom + (((((float) idum + 1.0) * skHz_per_component)
			- spec_axis_start) * SPIXELS_PER_KHZ);
	    vert1[1] = (long)vert1_new;
	/* if ((int)vert1[1] < sdis.y_offset_bottom + AXIS_WIDTH) */
	    vert1[1] = sdis.y_offset_bottom + AXIS_WIDTH;
	    vert2[0] = (int)(fdum + (pixels_per_fft / (float)spec_overlap));
	    vert2[1] = vert1[1];
	    
	    if (vert2[0] - vert1[0] < 1) {
		vert1[0] -= 1;
	    } 	
	    
	    fdum += (pixels_per_fft / (float)spec_overlap);
	
	    if (smooth_spec) {
		vert1[0] -= 1; vert2[0] += 1; 
		bgnqstrip ();
	    }
	    if (vert1[0] < sdis.x_offset_left + AXIS_WIDTH) 
		vert1[0] = sdis.x_offset_left + AXIS_WIDTH;
	    if (vert2[0] >= sdis.x_offset_left + AXIS_WIDTH + sdis.x_length)
		vert2[0] = sdis.x_offset_left + AXIS_WIDTH + sdis.x_length - 1;
	    
	    if (!excitation_pattern_on) {
		for (i = 0; i < end;) {
		    if (components_per_point > 1) {
			sum = 0.0;
			for (j = 0; j < components_per_point; j++, i++, sd_ptr++) 
			    sum += (*sd_ptr);
			power_spectrum1[i] = sum;
		    }
		    else power_spectrum1[i] = (*sd_ptr++), i++;
		}
	    }
	    
	    else {
		pow_ptr = &power_spectrum2[0];
		for (i = 0; i < end2; pow_ptr++){
		    sum = 0.0;
		
		    for (j = 0; j < components_per_point; j++, i++, sd_ptr++) {
			sum += (*sd_ptr);
		    }     
		    *pow_ptr = sum / components_per_point;	
		}
		end2 /= components_per_point;
		start = (idum - idum2) / components_per_point;
		end /= components_per_point;
		for (i = 0; i < end; i++) {

		    sum = 0.0;
		    f = (i * kHz_per_spec_point) + spec_axis_start;
		    erb = 24.7 * ((4.37 * f) + 1.0);
		    erb /= (kHz_per_spec_point * 1000.0); 
		    p = 4.0 / erb;
		    half_comps = (int)(erb * 3);
		
		    if ((i < half_comps - start) || (i > end2 - (half_comps + start))) { 
			for (j = (i - half_comps) + start, k = -half_comps; j < i + half_comps + start; j++, k++) {
			    if ((j >= 0) && (j < end2))  sum += power_spectrum2[j] * (1 + p * abs(k)) * exp (-p * abs(k));
			}
		    } 
		    else {
			for (j = (i - half_comps) + start, k = -half_comps; j < i + half_comps + start; j++, k++) {
			    sum += power_spectrum2[j] * (1 + p * abs(k)) * exp (-p * abs(k));
			}
		    }
		    power_spectrum1[i] = sum / erb;
		}  
	    }
	    
	    for (i = 0, n = 0; i < end; n++, i++) {
		
		if (invert_colors) {
    		    brightness = 255 - (int)(fsqrt(power_spectrum1[i]) * boost);
		    if (brightness < 0) brightness = 0;
		}
		else {
		    brightness = (int)(fsqrt(power_spectrum1[i]) * boost);
		    if (brightness > 255) brightness = 255;
		}
		if (color_on) {
		    if (brightness > 240) {
			color_vec[0] = 255;
			color_vec[1] = 255;
			color_vec[2] = brightness;
		    }
		    else if (brightness > 127) {
			color_vec[0] = brightness + 15;
			color_vec[1] = brightness + 15;
			color_vec[2] = 0;
		    }
		    else if (brightness > 63) {
			color_vec[0] = brightness * 2;
			color_vec[1] = 0;
			color_vec[2] = 0;
		    }
		    else {
			color_vec[0] = 0;
			color_vec[1] = 0;
			color_vec[2] = brightness;
		    }   
		}
		else {
		    color_vec[0] = brightness;
		    color_vec[1] = brightness;
		    color_vec[2] = brightness;
		}
		
		vert1_new += pixels_per_spec_point;
		if ((int)vert1_new > axis_end) vert1_new = (float)axis_end,  i += 1000;
		
		if (!smooth_spec) {
		
		    c3s (color_vec);
		
/*		    linewidth ((int)vert1_new - vert1[1]);
*/
		    for(; vert1[1] < (int)vert1_new; vert1[1]++, vert2[1]++){
			bgnline ();		    
			v2i (vert1);
			v2i (vert2);  
			endline ();   
		    }	
/*		    vert1[1] = (int)vert1_new;
		    vert2[1] = vert1[1];	*/
		}
		else {
		    
		    c3s (&color_vec_previous[n][0]);
		    v2i (vert1);
		    c3s (color_vec);
		    v2i (vert2);
			
		    for (j = 0; j < 3; j++) {
			color_vec_previous[n][j] = color_vec[j];
		    }
		    vert1[1] = (int)vert1_new;
		    vert2[1] = vert1[1];
		}
	    }
	    if (smooth_spec) {
		if (invert_colors) for (j = 0; j < 3; j++) color_vec[j] = 255;
		else for (j = 0; j < 3; j++) color_vec[j] = 0;   
		c3s (color_vec);
		v2i (vert1);
		c3s (color_vec);
		v2i (vert2);
	    }
	    if (vert1[1] != axis_end) {
		if (invert_colors) cpack (WHITE1);
		else cpack (DISPLAY_COLOR);
		if (smooth_spec) {vert1[0] += 1; vert2[0] -= 1;}
		rectfi (vert1[0], vert1[1], vert2[0], sdis.y_offset_bottom + sdis.y_length - 1);
	    }
	    if (smooth_spec) endqstrip ();
	}
	
	shademodel (FLAT);
	if (smooth_spec) {
	    linewidth (AXIS_WIDTH);
	    cpack (AXIS_COLOR);
	    recti (tdis.x_offset_left, tdis.y_offset_bottom,
		tdis.x_offset_left + tdis.x_length, 
		tdis.y_offset_bottom + tdis.y_length);
	}	
	if (not_current_window) winset (current_gid);
}

/***************************************************************************/

static	void	plot_spec_data2 (void)	/* scrolling display */ 

{
	int		i, j, k, m, n, start, end, end2, half_comps;
	int		components_per_point;
	int		brightness;
	int		axis_end;
	long		vert1[2], vert2[2];
	short	        not_current_window = FALSE;
	short		color_vec[3];
	static	short	color_vec_previous[500][3];
	long		current_gid, idum, idum2; 
	float		erb, f, p;
	float		*pow_ptr, power_spectrum1[5000], power_spectrum2[5000];
	static	int 	count=0;
	float		pixels_per_buffer, sum;
	static	float	fdum = 0.0;
	float		vert1_new;
	float		*sd_ptr, pow,  boost;
	static	short	first_time = TRUE;

	if (first_time || reset_spectro) {
	    fdum = (float)(sdis.x_offset_left + AXIS_WIDTH) + 0.1;
	    first_time = FALSE;
	    reset_spectro = FALSE;
	}   

	if ((current_gid = winget ()) != swin.gid) {
		not_current_window = TRUE;
		winset (swin.gid);
	}
	axis_end = sdis.y_length + sdis.y_offset_bottom;
	boost = spec_boost * amp_scale;

	pixels_per_buffer = (pixels_per_fft * (float)ffts_per_buffer / (float)spec_overlap);

	if ((int)(fdum + pixels_per_buffer) > sdis.x_length + sdis.x_offset_left - 1) {
	    if (invert_colors) cpack (WHITE1);
	    else cpack (DISPLAY_COLOR); 
	    rectfi ((int)fdum, sdis.y_offset_bottom + AXIS_WIDTH, 
		    sdis.x_offset_left + sdis.x_length - 1, sdis.y_offset_bottom + sdis.y_length - 1);   
	    if (single_scroll) {
		if (not_current_window) winset (current_gid);
		return;
	    }
	    else fdum = (float)(sdis.x_offset_left + AXIS_WIDTH) + 0.1;
	}
	idum = (long)(spec_axis_start / skHz_per_component);
	if (excitation_pattern_on) {
	    idum2 = (long)(spec_axis_start * 0.8 / skHz_per_component); 
	}
	components_per_point = (int)((float)scomponents_per_display * 2.0 / ((float)sdis.y_length));
	if (components_per_point < 1) components_per_point = 1;
	pixels_per_spec_point = spixels_per_component * (float)components_per_point;

	if (scomponents_per_display > spec_components - (idum + components_per_point*2))
	    end = spec_components - (idum + components_per_point*2);
	else end = scomponents_per_display;
	if (excitation_pattern_on) {
	    end2 = (idum - idum2) + end * 1.2;
	    if (end2 > spec_components - (idum + components_per_point*2))
	    end2 = spec_components - (idum + components_per_point*2); 
	}  
	
	if (smooth_spec) shademodel(GOURAUD);
	
	for (m = 0; m < ffts_per_buffer; m++) {
	
	    sd_ptr = &spec_data[m][0]; 
	    sd_ptr += idum;
	    
	    vert1[0] = (int)fdum + (int)((float)pixels_per_fft * (float)m / (float)spec_overlap);
	    vert1_new = (float)sdis.y_offset_bottom + (((((float) idum + 1.0) * skHz_per_component)
			- spec_axis_start) * SPIXELS_PER_KHZ);
	    vert1[1] = (long)vert1_new;
	/* if ((int)vert1[1] < sdis.y_offset_bottom + AXIS_WIDTH) */
	    vert1[1] = sdis.y_offset_bottom + AXIS_WIDTH;
	    vert2[0] = (int)fdum + (int)((float)pixels_per_fft * (float)(m + 1) / (float)spec_overlap);
	    vert2[1] = vert1[1];
	    
	    kHz_per_spec_point = skHz_per_component * components_per_point;
	    
	    if (vert2[0] - vert1[0] < 1) {
		if (vert1[0] > sdis.x_offset_left + AXIS_WIDTH) 
		    vert1[0] -= 1;
	    }	
	    if (smooth_spec) {
		vert1[0] -= 1; vert2[0] += 1; 
		
		bgnqstrip ();
	    }
	    if (vert1[0] < sdis.x_offset_left + AXIS_WIDTH) 
		vert1[0] = sdis.x_offset_left + AXIS_WIDTH;
	    if (vert2[0] >= sdis.x_offset_left + AXIS_WIDTH + sdis.x_length)
		vert2[0] = sdis.x_offset_left + AXIS_WIDTH + sdis.x_length - 1;
	    
	    
	    if (!excitation_pattern_on) {
		for (i = 0; i < end;) {
		    if (components_per_point > 1) {
			sum = 0.0;
			for (j = 0; j < components_per_point; j++, i++, sd_ptr++) 
			    sum += (*sd_ptr);
			power_spectrum1[i] = sum;
		    }
		    else power_spectrum1[i] = (*sd_ptr++), i++;
		}
	    }
	    
	    else {
		pow_ptr = &power_spectrum2[0];
		for (i = 0; i < end2; pow_ptr++){
		    sum = 0.0;
		
		    for (j = 0; j < components_per_point; j++, i++, sd_ptr++) {
			sum += (*sd_ptr);
		    }     
		    *pow_ptr = sum / components_per_point;	
		}
		end2 /= components_per_point;
		start = (idum - idum2) / components_per_point;
		end /= components_per_point;
		
		for (i = 0; i < end; i++) {

		    sum = 0.0;
		    f = (i * kHz_per_spec_point) + spec_axis_start;
		    erb = 24.7 * ((4.37 * f) + 1.0);
		    erb /= (kHz_per_spec_point * 1000.0); 
		    p = 4.0 / erb;
		    half_comps = (int)(erb * 3);
		
		    if ((i < half_comps - start) || (i > end2 - (half_comps + start))) { 
			for (j = (i - half_comps) + start, k = -half_comps; j < i + half_comps + start; j++, k++) {
			    if ((j >= 0) && (j < end2))  sum += power_spectrum2[j] * (1 + p * abs(k)) * exp (-p * abs(k));
			}
		    } 
		    else {
			for (j = (i - half_comps) + start, k = -half_comps; j < i + half_comps + start; j++, k++) {
			    sum += power_spectrum2[j] * (1 + p * abs(k)) * exp (-p * abs(k));
			}
		    }
		    power_spectrum1[i] = sum / erb;
		}  
	    }
	    
	    for (i = 0, n = 0; i < end; i++, n++) {
		
		if (invert_colors) {
    		    brightness = 255 - (int)(fsqrt(power_spectrum1[i]) * boost);
		    if (brightness < 0) brightness = 0;
		}
		else {
		    brightness = (int)(fsqrt(power_spectrum1[i]) * boost);
		    if (brightness > 255) brightness = 255;
		}
		if (color_on) {
		    if (brightness > 240) {
			color_vec[0] = 255;
			color_vec[1] = 255;
			color_vec[2] = brightness;
		    }
		    else if (brightness > 127) {
			color_vec[0] = brightness + 15;
			color_vec[1] = brightness + 15;
			color_vec[2] = 0;
		    }
		    else if (brightness > 63) {
			color_vec[0] = brightness * 2;
			color_vec[1] = 0;
			color_vec[2] = 0;
		    }
		    else {
			color_vec[0] = 0;
			color_vec[1] = 0;
			color_vec[2] = brightness;
		    }   
		}
		else {
		    color_vec[0] = brightness;
		    color_vec[1] = brightness;
		    color_vec[2] = brightness;
		}
		
		vert1_new += pixels_per_spec_point;
		if ((int)vert1_new > axis_end) vert1_new = (float)axis_end,  i += 1000;
		
		if (!smooth_spec) {
		
		    c3s (color_vec);
		
/*		    linewidth ((int)vert1_new - vert1[1]);
*/
		    for(; vert1[1] < (int)vert1_new; vert1[1]++, vert2[1]++){
			bgnline ();		    
			v2i (vert1);
			v2i (vert2);  
			endline ();   
		    }	
/*		    vert1[1] = (int)vert1_new;
		    vert2[1] = vert1[1];	*/
		}
		else {
		    
		    c3s (&color_vec_previous[n][0]);
		    v2i (vert1);
		    c3s (color_vec);
		    v2i (vert2);
			
		    for (j = 0; j < 3; j++) {
			color_vec_previous[n][j] = color_vec[j];
		    }
		    vert1[1] = (int)vert1_new;
		    vert2[1] = vert1[1];
		}
	    }
	    if (smooth_spec) {
		if (invert_colors) for (j = 0; j < 3; j++) color_vec[j] = 255;
		else for (j = 0; j < 3; j++) color_vec[j] = 0;   
		c3s (color_vec);
		v2i (vert1);
		c3s (color_vec);
		v2i (vert2);
	    }
	    if (vert1[1] != axis_end) {
		if (invert_colors) cpack (WHITE1);
		else cpack (DISPLAY_COLOR);
		if (smooth_spec) {vert1[0] += 1; vert2[0] -= 1;}  
		rectfi (vert1[0], vert1[1], vert2[0], sdis.y_offset_bottom + sdis.y_length - 1);
	    }
	    if (smooth_spec) endqstrip ();    
	}
	shademodel (FLAT);
	
	fdum += pixels_per_buffer;
		
	if (not_current_window) winset (current_gid);
}

/***************************************************************************/

static	void	plot_waterfall (void)

{
	int		i, j, k, selected_start, selected_end, end, end2, start, half_comps;
	int		components_per_point;
	int		bottom, bottom_plus, top, right;
	short		not_current_window = FALSE;
	long		current_gid;
	short		color_vec[5][3], black_vec[3] = {0, 0, 0};
	Screencoord	x1, y1, x2, y2, newx, newy;
	long		idum, idum2;
	float		fdum;
	static		int	count=0;
	float		f, p, erb;
	float		pixels_per_db, pixels_per_buffer, sum;
	static	float	error = 0.0;
	float		vert[2], cutoff[3];
	float		lower_vert[1000][4][2];
	float		power_spectrum2[5000];
	float		*fd_ptr, *max_ptr, *pow_ptr;
	float		nyquist;
	static	float	old_sample_rate = 0.0;
	static	long	old_overlap = 0, old_fft_points = 0, old_s_per_d = 0;


	if ((current_gid = winget ()) != wwin.gid) {
		not_current_window = TRUE;
		winset (wwin.gid);
	}

	if ((spec_fft_points != old_fft_points) || (spec_overlap != old_overlap) 
		    || (old_sample_rate != input_sample_rate) || (samples_per_display != old_s_per_d)) {
	    old_fft_points = spec_fft_points;
	    old_overlap = spec_overlap;
	    old_sample_rate = input_sample_rate;
	    old_s_per_d = samples_per_display;
	    set_sdisplay_parameters ();
	    set_fdisplay_parameters ();
	    clear_screen ();
	    draw_water_axes ();
	}

	
	nyquist = (float)input_sample_rate / 2.0;

	bottom = (wdis.y_offset_bottom + AXIS_WIDTH);
	bottom_plus = bottom + 1;
	top = (wdis.y_offset_bottom + wdis.y_length - 1);
	right = (wdis.x_offset_left + wdis.x_length - 1);



	pixels_per_buffer = (ffts_per_buffer * pixels_per_fft / (float)spec_overlap) + (error * 2.0);
	if (waterfall_nrt) pixels_per_buffer = pixels_per_fft + (error * 2.0);
	if (pixels_per_buffer >= wdis.x_length - 2) {
		clear_display ();
		if (not_current_window) winset (current_gid);
		return;
	} 
	newx = wdis.x_offset_left + AXIS_WIDTH;
	newy = bottom + (int)pixels_per_buffer;
	if (newy > top) newy = top;
	error = (pixels_per_buffer/2.0) - (float)(int)(pixels_per_buffer/2.0);
	x1 = newx + (int)(pixels_per_buffer/2.0);
	if (x1 > right) x1 = right;
	y1 = bottom;
	x2 = right;
	y2 = top - pixels_per_buffer;
	if (y2 < bottom) y2 = bottom;
	
	
	rectcopy (x1, y1, x2, y2, newx, newy);

	/* cpack (DISPLAY_COLOR); */
	cpack (WATER_COLOR);
	
	rectfi (right - (int)(pixels_per_buffer/2.0), bottom, right, top);
	rectfi (newx, bottom, right, newy);

	fd_ptr = &freq_data[0];
	
	cpack (DATA_COLOR);
	
	pixels_per_db = PIXELS_PER_FDB;
	
	
	idum = (long)(freq_axis_start / kHz_per_component); 
	if (!excitation_pattern_on) fd_ptr += idum;
	else {
	    idum2 = (long)(freq_axis_start * 0.8 / kHz_per_component); 
	    fd_ptr += idum2;
	    
	}
	
	vert[0] = WATER_OFF + (float)wdis.x_offset_left + (((((float) idum + 1.0) * kHz_per_component)
			- freq_axis_start) * PIXELS_PER_KHZ);
	if ((int)vert[0] < wdis.x_offset_left + AXIS_WIDTH) vert[0] = wdis.x_offset_left + AXIS_WIDTH;

	pow_ptr = &power_spectrum[0];	

	components_per_point = (int)((float)components_per_display * 2.0 / ((float)fdis.x_length));
	if (components_per_point < 2) components_per_point = 2;
	pixels_per_freq_point = pixels_per_component * (float)components_per_point;
	kHz_per_freq_point = pixels_per_freq_point / PIXELS_PER_KHZ;

	if (components_per_display > freq_components - (idum + components_per_point*2))
	    end = freq_components - (idum + components_per_point*2); 
	else end = components_per_display /* - components_per_point */;
	if (excitation_pattern_on) {
	    end2 = (idum - idum2) + end * 1.2;
	    if (end2 > freq_components - (idum + components_per_point*2))
		end2 = freq_components - (idum + components_per_point*2); 
	}  
	
/*	concave (TRUE);  */
	shademodel (GOURAUD);
	color_vec[0][0] = 0;
	color_vec[0][1] = 0;
	color_vec[0][2] = 0;
	color_vec[1][0] = 63;
	color_vec[1][1] = 0;
	color_vec[1][2] = 0;
	cutoff[0] = wdis.y_offset_bottom + fdis.y_length * 0.25;
	color_vec[2][0] = 255;
	color_vec[2][1] = 0;
	color_vec[2][2] = 0;
	cutoff[1] = wdis.y_offset_bottom + fdis.y_length * 0.5;
	color_vec[3][0] = 255;
	color_vec[3][1] = 255;
	color_vec[3][2] = 0;
	cutoff[2] = wdis.y_offset_bottom + fdis.y_length * 0.75;
	color_vec[4][0] = 255;
	color_vec[4][1] = 255;
	color_vec[4][2] = 255;
	
	bgnqstrip (); 
/*	bgntmesh (); */
	c3s (&color_vec[0][0]);
	vert[1] = bottom;
	v2f (vert);
	
	j = 0;
	
	if (!excitation_pattern_on) {
    	
	    for (i = 0, k = 0; i < end; pow_ptr++, k++){
		
		sum = 0.0;
		
		if (spectrum_level) {
		    for (j = 0; j < components_per_point; j++, i++, fd_ptr++) 		
			    sum += (*fd_ptr); 
		    if (sum <= 0.0000000001) sum = 0.0000000001;		
		    *pow_ptr = level_scale + (10.0 * log10 (sum / ((float) components_per_point 
					* kHz_per_component * 1000.0)));
		}
		else {
		    if (components_per_point == 1) {
			sum = *fd_ptr++; i++;
		    }
		    else {
			for (j = 0; j < components_per_point; j++, i++, fd_ptr++) 		
			    if (*fd_ptr > sum) sum = (*fd_ptr);
	
		    }  
		    if (sum <= 0.0000000001) sum = 0.0000000001;
		    *pow_ptr = level_scale + (10.0 * log10 (sum));
		} 
	    }
	    
	    end /= components_per_point;
	}
	else {
	    pow_ptr = &power_spectrum2[0];
	    for (i = 0; i < end2; pow_ptr++){
		sum = 0.0;
		
		for (j = 0; j < components_per_point; j++, i++, fd_ptr++) sum += (*fd_ptr); 
		if (spectrum_level) {		
		    *pow_ptr = sum / ((float) components_per_point * kHz_per_component * 1000.0);
		}
		else *pow_ptr = sum / components_per_point;
			
	    }
	    end2 /= components_per_point;
	    start = (idum - idum2) / components_per_point;
	    end /= components_per_point;
	    
	    for (i = 0; i < end; i++) {
		sum = 0.0;
		f = (i * kHz_per_freq_point) + freq_axis_start;
		erb = 24.7 * ((4.37 * f) + 1.0);
		erb /= (kHz_per_freq_point * 1000.0); 
		p = 4.0 / erb;
		half_comps = (int)(erb * 3);
		
		if ((i < half_comps - start) || (i > end2 - (half_comps + start))){ 
		    for (j = (i - half_comps) + start, k = -half_comps; j < i + half_comps + start; j++, k++) {
			if ((j >= 0) && (j < end2))  sum += power_spectrum2[j] * (1 + p * abs(k)) * exp (-p * abs(k));
		    }
		} 
		else {
		    for (j = (i - half_comps) + start, k = -half_comps; j < i + half_comps + start; j++, k++) {
			sum += power_spectrum2[j] * (1 + p * abs(k)) * exp (-p * abs(k));
		    }
		}  
		if (sum <= 0.0000000001) sum = 0.0000000001;
		power_spectrum[i] = level_scale + (10.0 * log10 (sum / erb));
	    }
	    freq_points_per_display = 1 + end;
	} 
	pow_ptr = &power_spectrum[0];
	
	for (k = 0; k < end; pow_ptr++, k++) {
		vert[1] = ((*pow_ptr - flevel_axis_start) * pixels_per_db) 
				+ wdis.y_offset_bottom;
		if ((int)vert[1] < bottom) vert[1] = bottom + 1;
		else if ((int)vert[1] >= top) vert[1] = top;
		

		lower_vert[k][0][0] = lower_vert[k][1][0] = lower_vert[k][2][0] = lower_vert[k][3][0] = vert[0];

		if (vert[1] <= cutoff[0]) {
		    c3s (&color_vec[1][0]);
		    v2f (vert);
		    fdum = vert[1] - 20.0;
		    if (fdum < bottom_plus) fdum = bottom_plus; 
		    lower_vert[k][0][1] = lower_vert[k][1][1] = lower_vert[k][2][1] = fdum;
		    c3s (&color_vec[1][0]);
		    v2f (&lower_vert[k][0][0]);
		}   
		else if (vert[1] <= cutoff[1]) { 
		    c3s (&color_vec[2][0]);
		    v2f (vert);
		    lower_vert[k][0][1] = lower_vert[k][1][1] = lower_vert[k][2][1] = vert[1] - 20.0;
		    c3s (&color_vec[1][0]);
		    v2f (&lower_vert[k][0][0]);
		}    
		else if (vert[1] <= cutoff[2]) {
		    c3s (&color_vec[3][0]);
		    v2f (vert);
		    lower_vert[k][0][1] = lower_vert[k][1][1] = vert[1] - 20.0;
		    lower_vert[k][2][1] = vert[1] - 40.0;
		    c3s (&color_vec[2][0]);
		    v2f (&lower_vert[k][0][0]);
		}
		else {
		    c3s (&color_vec[4][0]);
		    v2f (vert);
		    lower_vert[k][0][1] = vert[1] - 20.0;
		    lower_vert[k][1][1] = vert[1] - 40.0;
		    lower_vert[k][2][1] = vert[1] - 60.0;
		    c3s (&color_vec[3][0]);
		    v2f (&lower_vert[k][0][0]);
		}
		
		lower_vert[k][3][1] = bottom_plus;
		
	/*	v2f (vert);
		c3s (black_vec);
		vert[1] = vert[1] - 100;
		if (vert[1] < bottom) vert[1] = bottom;
		v2f (vert);
		vert[1] = bottom;
		v2f (vert);   */
		vert[0] += pixels_per_freq_point;
		if (vert[0] > (float)right) vert[0] = right;	
	}
	endqstrip ();
	/* endtmesh (); */
	
	idum = k;
	bgnqstrip ();
	    for (i = 0; i < idum; i++) {
		if (lower_vert[i][0][1] == lower_vert[i][3][1]) c3s (&color_vec[0][0]);
		else if (lower_vert[i][0][1] == lower_vert[i][2][1]) c3s (&color_vec[1][0]);
		else if (lower_vert[i][0][1] == lower_vert[i][1][1]) c3s (&color_vec[2][0]);
		else c3s (&color_vec[3][0]);
		v2f (&lower_vert[i][0][0]);
		if (lower_vert[i][1][1] == lower_vert[i][3][1]) c3s (&color_vec[0][0]);
		else if (lower_vert[i][1][1] == lower_vert[i][2][1]) c3s (&color_vec[1][0]);
		else c3s (&color_vec[2][0]);
		v2f (&lower_vert[i][1][0]);
	    }
	endqstrip ();
	bgnqstrip ();
	    for (i = 0; i < idum; i++) {
		if (lower_vert[i][1][1] == lower_vert[i][3][1]) c3s (&color_vec[0][0]);
		else if (lower_vert[i][1][1] == lower_vert[i][2][1]) c3s (&color_vec[1][0]);
		else c3s (&color_vec[2][0]);
		v2f (&lower_vert[i][1][0]);
		if (lower_vert[i][2][1] == lower_vert[i][3][1]) c3s (&color_vec[0][0]);
		else c3s (&color_vec[1][0]);
		v2f (&lower_vert[i][2][0]);
	    }
	endqstrip ();
	bgnqstrip ();
	    for (i = 0; i < idum; i++) {
		if (lower_vert[i][2][1] == lower_vert[i][3][1]) c3s (&color_vec[0][0]);
		else c3s (&color_vec[1][0]);
		v2f (&lower_vert[i][2][0]);
		c3s (&color_vec[0][0]);
		v2f (&lower_vert[i][3][0]);
	    }
	endqstrip ();
	
/*	bgnpolygon ();
	if (vert[0] != right) vert[0] -= pixels_per_freq_point;
	c3s (black_vec);
	vert[1] = bottom; 
	v2f (vert);
	vert[1] = top;
	v2f (vert);
	vert[0] = right;
	v2f (vert);
	vert[1] = bottom;
	v2f (vert);
	endpolygon ();  */
	
	freq_points_per_display = 1 + (int)end /*  / (components_per_point)  */;
		
	shademodel (FLAT);
	if (not_current_window) winset (current_gid);
}

/***************************************************************************/

short		mouse_within_display (short xpos, short ypos)

{
	long	x, y;

	getorigin (&x, &y);
	xpos = xpos - x;
	ypos = ypos - y;

	if (winget () == twin.gid) {
		if ((xpos > tdis.x_offset_left + AXIS_WIDTH) 
			&& (ypos > tdis.y_offset_bottom + AXIS_WIDTH) 
			&& (xpos < tdis.x_offset_left + tdis.x_length - 1)
			&& (ypos < tdis.y_offset_bottom + tdis.y_length - 1)) return TRUE;
		else return FALSE;
	}		
	else if (winget () == fwin.gid) {
		if ((xpos > fdis.x_offset_left + AXIS_WIDTH) 
			&& (ypos > fdis.y_offset_bottom + AXIS_WIDTH) 
			&& (xpos < fdis.x_offset_left + fdis.x_length - 1)
			&& (ypos < fdis.y_offset_bottom + fdis.y_length - 1)) return TRUE;
		else return FALSE;
	}
	else if (winget () == swin.gid) {
		if ((xpos > sdis.x_offset_left + AXIS_WIDTH) 
			&& (ypos > sdis.y_offset_bottom + AXIS_WIDTH) 
			&& (xpos < sdis.x_offset_left + sdis.x_length - 1)
			&& (ypos < sdis.y_offset_bottom + sdis.y_length - 1)) return TRUE;
		else return FALSE;
	}
	else if (winget () == wwin.gid) {
	    if ((xpos > wdis.x_offset_left + AXIS_WIDTH) 
			&& (ypos > wdis.y_offset_bottom + AXIS_WIDTH) 
			&& (xpos < wdis.x_offset_left + (wdis.x_length/2) - 1)
			&& (ypos < wdis.y_offset_bottom + (wdis.y_length) - 1)) return 2;
	    if ((xpos > wdis.x_offset_left + (wdis.x_length/2) - 1)
			&& (ypos > wdis.y_offset_bottom + AXIS_WIDTH) 
			&& (xpos < wdis.x_offset_left + (wdis.x_length) - 1)
			&& (ypos < wdis.y_offset_bottom + (wdis.y_length) - 1)) return 3;
	    else return FALSE;
	}
}

/***************************************************************************/

static	void	set_time_annint (void)

{
	if ((tlevel_axis_end - tlevel_axis_start) <= 0.0001) tlevel_ann_int = 0.00002;
	else if ((tlevel_axis_end - tlevel_axis_start) <= 0.0002) tlevel_ann_int = 0.00005;
	else if ((tlevel_axis_end - tlevel_axis_start) <= 0.0005) tlevel_ann_int = 0.0001;
	else if ((tlevel_axis_end - tlevel_axis_start) <= 0.001) tlevel_ann_int = 0.002;
	else if ((tlevel_axis_end - tlevel_axis_start) <= 0.002) tlevel_ann_int = 0.005;
	else if ((tlevel_axis_end - tlevel_axis_start) <= 0.05) tlevel_ann_int = 0.01;
	else if ((tlevel_axis_end - tlevel_axis_start) <= 0.1) tlevel_ann_int = 0.02;
	else if ((tlevel_axis_end - tlevel_axis_start) <= 0.2) tlevel_ann_int = 0.05;
	else if ((tlevel_axis_end - tlevel_axis_start) <= 0.5) tlevel_ann_int = 0.1;
	else if ((tlevel_axis_end - tlevel_axis_start) <= 1) tlevel_ann_int = 0.2;
	else if ((tlevel_axis_end - tlevel_axis_start) <= 2) tlevel_ann_int = 0.5;
	else if ((tlevel_axis_end - tlevel_axis_start) <= 5) tlevel_ann_int = 1.0;
	else if ((tlevel_axis_end - tlevel_axis_start) <= 10) tlevel_ann_int = 2.0;
	else if ((tlevel_axis_end - tlevel_axis_start) <= 20) tlevel_ann_int = 5.0;
	else if ((tlevel_axis_end - tlevel_axis_start) <= 50) tlevel_ann_int = 10.0; 
	else if ((tlevel_axis_end - tlevel_axis_start) <= 100) tlevel_ann_int = 20.0;
	else if ((tlevel_axis_end - tlevel_axis_start) <= 200) tlevel_ann_int = 50.0;
	else if ((tlevel_axis_end - tlevel_axis_start) <= 500) tlevel_ann_int = 100.0;
	else if ((tlevel_axis_end - tlevel_axis_start) <= 1000) tlevel_ann_int = 200.0;
	else if ((tlevel_axis_end - tlevel_axis_start) <= 2000) tlevel_ann_int = 500.0;	
	else tlevel_ann_int = 1000.0;
}

/***************************************************************************/

static	void	set_freq_annint (void)

{
	if ((flevel_axis_end - flevel_axis_start) <= 10) flevel_ann_int = 2.0;
	else if ((flevel_axis_end - flevel_axis_start) <= 20) flevel_ann_int = 5.0;
	else if ((flevel_axis_end - flevel_axis_start) <= 50) flevel_ann_int = 10.0; 
	else if ((flevel_axis_end - flevel_axis_start) <= 100) flevel_ann_int = 20.0;
	else flevel_ann_int = 40.0;
}

/***************************************************************************/

static	void	set_spec_annint (void)

{
	if ((spec_axis_end - spec_axis_start) <= 1) spec_ann_int = 0.2;
	else if ((spec_axis_end - spec_axis_start) <= 2) spec_ann_int = 0.5;
	else if ((spec_axis_end - spec_axis_start) <= 5) spec_ann_int = 1.0;
	else if ((spec_axis_end - spec_axis_start) <= 10) spec_ann_int = 2.0;
	else if ((spec_axis_end - spec_axis_start) <= 20) spec_ann_int = 5.0;
	else if ((spec_axis_end - spec_axis_start) <= 50) spec_ann_int = 10.0; 
	else if ((spec_axis_end - spec_axis_start) <= 100) spec_ann_int = 20.0;
	else spec_ann_int = 40.0;
}

/***************************************************************************/

static	void	taxis_inc (void)

{
	float	    shift;

	if (winget () == twin.gid) {
		if (rms_on) {
		    shift = (tlevel_axis_end - tlevel_axis_start) / 5.0;
		    tlevel_axis_end += shift;
		    tlevel_axis_start -= shift;
	/*	tlevel_axis_end += 6.0;  */
		}
		else {
			tlevel_axis_end *= 2.0;
			tlevel_axis_start = (-tlevel_axis_end);
		}
		set_time_annint ();
		draw_display ();
		plot_time_data ();
	}
	else if (winget () == fwin.gid) {
		shift = (flevel_axis_end - flevel_axis_start) / 5.0;
		flevel_axis_end += shift;
		flevel_axis_start -= shift;
	/*	flevel_axis_end += 6.0;  */
		set_freq_annint ();
		draw_display ();
		calc_freq_data ();
		plot_freq_data ();
	}
	else if (winget () == swin.gid) {
	/*	shift = (spec_axis_end - spec_axis_start) / 5.0;  */
	/*	if ((spec_axis_end + shift) > 20.0) shift = 20.0 - spec_axis_end;
		if (shift > spec_axis_start) shift = spec_axis_start;  */
	/*	spec_axis_end += shift;
		spec_axis_start -= shift;  */ 
		spec_axis_end += 1.0;  
		if (spec_axis_end > 20.0) spec_axis_end = 20.0;
	/*	if (spec_axis_start < 0.0) spec_axis_start = 0.0;  */
		set_sdisplay_parameters ();
		set_spec_annint ();
		draw_display ();
		if (pause && spectrogram_on) get_spectrum_nrt ();
	}
}

/***************************************************************************/

static	void	taxis_dec (void)

{
	float	    shift;

	if (winget () == twin.gid) {
		if (rms_on) {
			shift = (tlevel_axis_end - tlevel_axis_start) / 5.0;
			if (((tlevel_axis_end - tlevel_axis_start) - shift * 2.0) < 5.0) {
			    shift = ((tlevel_axis_end - tlevel_axis_start) - 5.0) /2.0;
			}
			tlevel_axis_end -= shift;
			tlevel_axis_start += shift;
		/*	tlevel_axis_end -= 6.0;
			if ((tlevel_axis_end - tlevel_axis_start) < 6.0) {
				tlevel_axis_end = tlevel_axis_start + 6.0;
			} */
		}
		else {
			tlevel_axis_end /= 2.0;
			tlevel_axis_start = (-tlevel_axis_end);
		}
		set_time_annint ();
		draw_display ();
		plot_time_data ();
	}
	else if (winget () == fwin.gid) {
		shift = (flevel_axis_end - flevel_axis_start) / 5.0;
		if (((flevel_axis_end - flevel_axis_start) - shift * 2.0) < 5.0) {
		    shift = ((flevel_axis_end - flevel_axis_start) - 5.0) /2.0;
		}
		flevel_axis_end -= shift;
		flevel_axis_start += shift;
/*		flevel_axis_end -= 6.0;
		if ((flevel_axis_end - flevel_axis_start) < 6.0) flevel_axis_end += 6.0; */
		set_freq_annint ();
		draw_display ();
		calc_freq_data ();
		plot_freq_data ();
	}
	else if (winget () == swin.gid) {
	/*	shift = (spec_axis_end - spec_axis_start) / 5.0;
		if (((spec_axis_end - spec_axis_start) - shift * 2.0) < 1.0) {
		    shift = ((spec_axis_end - spec_axis_start) - 1.0) /2.0;
		} 
		spec_axis_end -= shift;
		spec_axis_start += shift;   */
		
		spec_axis_end -= 1.0; 
		if ((spec_axis_end - spec_axis_start) < 1.0) spec_axis_end += 1.0; 
		set_sdisplay_parameters ();
		set_spec_annint ();
		draw_display ();
		if (pause && spectrogram_on) get_spectrum_nrt ();
	}
}

/***************************************************************************/

static	void	baxis_inc (void)

{
	float	    shift;
	
	if (winget () == twin.gid) {
		if (rms_on) {
			shift = (tlevel_axis_end - tlevel_axis_start) / 5.0;
			tlevel_axis_end += shift;
			tlevel_axis_start += shift;
		/*	tlevel_axis_start += 6.0;
			if ((tlevel_axis_end - tlevel_axis_start) < 6.0)
				tlevel_axis_start = tlevel_axis_end - 6.0; */
			set_time_annint ();
			draw_display ();
			plot_time_data ();
		}
		else {
			dc_shift += tlevel_axis_end / 10.0;
		/*	printf ("%f\n", dc_shift);  */
			process_buffer ();
			plot_time_data ();
		}
	}
	else if (winget () == fwin.gid) {
		shift = (flevel_axis_end - flevel_axis_start) / 5.0;
		flevel_axis_end += shift;
		flevel_axis_start += shift;
	/*	flevel_axis_start += 6.0;
		if ((flevel_axis_end - flevel_axis_start) < 6.0) flevel_axis_start -= 6.0;  */
		set_freq_annint ();
		draw_display ();
		calc_freq_data ();
		plot_freq_data ();
	}
	else if (winget () == swin.gid) {
	/*	shift = (spec_axis_end - spec_axis_start) / 5.0;
		if (shift > 20.0 - spec_axis_end) shift = 20.0 - spec_axis_end;
		spec_axis_end += shift;
		spec_axis_start += shift;  */
		spec_axis_start += 1.0;
		if ((spec_axis_end - spec_axis_start) < 1.0) spec_axis_start -= 1.0;  
		set_sdisplay_parameters ();
		set_spec_annint ();
		draw_display ();
		if (pause && spectrogram_on) get_spectrum_nrt ();
	}
}

/***************************************************************************/

static	void	baxis_dec (void)

{
	float	shift;
	
	if (winget () == twin.gid) {
		if (rms_on) {
			shift = (tlevel_axis_end - tlevel_axis_start) / 5.0;
			tlevel_axis_end -= shift;
			tlevel_axis_start -= shift;
			
		/*	tlevel_axis_start -= 6.0; */
			set_time_annint ();
			draw_display ();
			plot_time_data ();
		}
		else {
			dc_shift -= tlevel_axis_end / 10.0;
			process_buffer ();
			plot_time_data ();
		}
	}
	else if (winget () == fwin.gid) {
		shift = (flevel_axis_end - flevel_axis_start) / 5.0;
		flevel_axis_end -= shift;
		flevel_axis_start -= shift;
	/*	flevel_axis_start -= 5.0; */
		set_freq_annint ();
		draw_display ();
		calc_freq_data ();
		plot_freq_data ();
	}
	else if (winget () == swin.gid) {
	/*	shift = (spec_axis_end - spec_axis_start) / 5.0;
		if (shift > spec_axis_start) shift = spec_axis_start;
		spec_axis_end -= shift;
		spec_axis_start -= shift; */
		spec_axis_start -= 1.0;   
		if (spec_axis_start < 0.0) spec_axis_start = 0.0;
		set_sdisplay_parameters ();
		set_spec_annint ();
		draw_display ();
		if (pause && spectrogram_on) get_spectrum_nrt ();
	}
}

/***************************************************************************/

static	void	raxis_inc (void)

{
	int	    i;
	float	    shift, selected_shift;

	if ((winget () == twin.gid) || (winget () == swin.gid)) {
		shift = 0.1 * (time_axis_end - time_axis_start);
		if (time_selected) {
		    shift = (time_selected_end - time_selected_start);
		    if (fwin.gid && !new_fft_selection) shift /= spec_overlap;
		}
		selected_shift = shift;
		if (shift > MAX_DISPLAY_SECONDS - time_axis_end) shift = MAX_DISPLAY_SECONDS - time_axis_end;
		time_axis_end += shift;
		time_axis_start += shift;
		if (time_selected) {
		    if (selected_shift > time_axis_end - time_selected_end) 
			selected_shift = time_axis_end - time_selected_end;
		    time_selected_end += selected_shift;
		    time_selected_start += selected_shift;
		    tpixels_selected_start = ((time_selected_start - time_axis_start) * 
				PIXELS_PER_SECOND) + tdis.x_offset_left;
		    tpixels_selected_end = ((time_selected_end - time_axis_start) * 
				PIXELS_PER_SECOND) + tdis.x_offset_left;
		    if (!new_play_selection) play_selection ();
		    if (fwin.gid && !new_fft_selection) {
			get_spectrum ();
			calc_freq_data ();
			plot_freq_data ();
		    }
		}
		if (winget () == twin.gid) {
		    draw_display ();
		    process_buffer ();
		    plot_time_data ();
		    if (swin.gid) {
			winset (swin.gid);
			draw_display ();
			if (spectrogram_on) get_spectrum_nrt ();
			winset (twin.gid);
		    }
		}
		else if (winget () == swin.gid) {
		    draw_display ();
		    if (spectrogram_on) get_spectrum_nrt ();
		    if (twin.gid) {
			winset (twin.gid);
			draw_display ();
			process_buffer ();
			plot_time_data ();
			winset (swin.gid);
		    }
		}
		for (i = 0; i < tz_count; i++) {
		    if (old_time_axis_end[i] + shift > MAX_DISPLAY_SECONDS) {
			old_time_axis_start[i] += MAX_DISPLAY_SECONDS - old_time_axis_end[i];
			old_time_axis_end[i] = MAX_DISPLAY_SECONDS;
		    }	
		    else {
			old_time_axis_start[i] += shift;
			old_time_axis_end[i] += shift;
		    }
		}
	}
	else if (winget () == fwin.gid) {
		shift = 0.1 * (freq_axis_end - freq_axis_start);
		if (freq_selected) shift = (freq_selected_end - freq_selected_start);
		selected_shift = shift;
		if (shift > MAX_DISPLAY_KHZ - freq_axis_end) shift = MAX_DISPLAY_KHZ - freq_axis_end;
		freq_axis_end += shift;
		freq_axis_start += shift;
		if (freq_selected) {
		    if (selected_shift > freq_axis_end - freq_selected_end) 
			selected_shift = freq_axis_end - freq_selected_end;
		    freq_selected_end += selected_shift;
		    freq_selected_start += selected_shift;
		    fpixels_selected_start = ((freq_selected_start - freq_axis_start) * 
				PIXELS_PER_KHZ) + fdis.x_offset_left;
		    fpixels_selected_end = ((freq_selected_end - freq_axis_start) * 
				PIXELS_PER_KHZ) + fdis.x_offset_left;
		}
		draw_display ();
		calc_freq_data ();
		plot_freq_data ();
		for (i = 0; i < fz_count; i++) {
		    if (old_freq_axis_end[i] + shift > MAX_DISPLAY_KHZ) {
			old_freq_axis_start[i] += MAX_DISPLAY_KHZ - old_freq_axis_end[i];
			old_freq_axis_end[i] = MAX_DISPLAY_KHZ;
		    }	
		    else {
			old_freq_axis_start[i] += shift;
			old_freq_axis_end[i] += shift;
		    }
		}
	}
}

/***************************************************************************/

static	void	raxis_dec (void)

{
	int	    i;
	float	    shift, selected_shift;

	if ((winget () == twin.gid) || (winget () == swin.gid)) {
		shift = 0.1 * (time_axis_end - time_axis_start);
		if (time_selected) {
		    shift = (time_selected_end - time_selected_start);
		    if (fwin.gid && !new_fft_selection) shift /= spec_overlap;
		}
		selected_shift = shift;
		if (shift > time_axis_start) shift = time_axis_start;
		time_axis_end -= shift;
		time_axis_start -= shift;
		if (time_selected) {
		    if (selected_shift > time_selected_start - time_axis_start) 
			selected_shift = time_selected_start - time_axis_start;
		    time_selected_end -= selected_shift;
		    time_selected_start -= selected_shift;
		    tpixels_selected_start = ((time_selected_start - time_axis_start) * 
				PIXELS_PER_SECOND) + tdis.x_offset_left;
		    tpixels_selected_end = ((time_selected_end - time_axis_start) * 
				PIXELS_PER_SECOND) + tdis.x_offset_left;
		    if (!new_play_selection) play_selection ();
		    if (fwin.gid && !new_fft_selection) {
			get_spectrum ();
			calc_freq_data ();
			plot_freq_data ();
		    }
		}
		if (winget () == twin.gid) {
		    draw_display ();
		    process_buffer ();
		    plot_time_data ();
		    if (swin.gid) {
			winset (swin.gid);
			draw_display ();
			if (spectrogram_on) get_spectrum_nrt ();
			winset (twin.gid);
		    }
		}
		else if (winget () == swin.gid) {
		    draw_display ();
		    if (spectrogram_on) get_spectrum_nrt ();
		    if (twin.gid) {
			winset (twin.gid);
			draw_display ();
			process_buffer ();
			plot_time_data ();
			winset (swin.gid);
		    }
		}
		for (i = 0; i < tz_count; i++) {
		    if (old_time_axis_start[i] < shift) {	
			old_time_axis_end[i] -= old_time_axis_start[i];
			old_time_axis_start[i] = 0.0;
		    }	
		    else {
			old_time_axis_start[i] -= shift;
			old_time_axis_end[i] -= shift;
		    }
		}
	}
	else if (winget () == fwin.gid) {
		shift = 0.1 * (freq_axis_end - freq_axis_start);
		if (freq_selected) shift = (freq_selected_end - freq_selected_start);
		selected_shift = shift;
		if (shift > freq_axis_start) shift = freq_axis_start;
		freq_axis_end -= shift;
		freq_axis_start -= shift;
		if (freq_selected) {
		    if (selected_shift > freq_selected_start - freq_axis_start) 
			selected_shift = freq_selected_start - freq_axis_start;
		    freq_selected_end -= selected_shift;
		    freq_selected_start -= selected_shift;
		    fpixels_selected_start = ((freq_selected_start - freq_axis_start) * 
				PIXELS_PER_KHZ) + fdis.x_offset_left;
		    fpixels_selected_end = ((freq_selected_end - freq_axis_start) * 
				PIXELS_PER_KHZ) + fdis.x_offset_left;
		}
		draw_display ();
		calc_freq_data ();
		plot_freq_data ();
		for (i = 0; i < fz_count; i++) {
		    if (old_freq_axis_start[i] < shift) {
			old_freq_axis_end[i] -= old_freq_axis_start[i];
			old_freq_axis_start[i] = 0.0;
		    }	
		    else {
			old_freq_axis_start[i] -= shift;
			old_freq_axis_end[i] -= shift;
		    }
		}
	}
}

/***************************************************************************/

static	void	change_axis_scales (void)

{
	if (rms_on) {
		tlevel_axis_end = level_scale + (20.0 * log10 (tlevel_axis_end 
			/ (ROOT_TWO * amp_scale)));
		tlevel_axis_start = tlevel_axis_start_old;
		if (tlevel_axis_start > tlevel_axis_end - 6.0) 
			tlevel_axis_start = tlevel_axis_end - 6.0;
	}
	else {
		tlevel_axis_end = pow (10.0, (tlevel_axis_end - level_scale) / 20.0)
			* (ROOT_TWO * amp_scale);
		tlevel_axis_start_old = tlevel_axis_start;
		tlevel_axis_start = (-tlevel_axis_end);
	}
	set_time_annint ();
}	

/***************************************************************************/

static	void	trigger_inc ()

{
	trigger_level += 6.0;
	draw_display ();
	if (pause) plot_time_data ();
}

/***************************************************************************/

static	void	trigger_dec ()

{
	trigger_level -= 6.0;
	draw_display ();
	if (pause) plot_time_data ();
}

/***************************************************************************/

static	void	atten_inc ()

{
	input_atten -= 10.0;
	if (input_atten < 0.0) input_atten = 0.0;
	if (!pause) set_attenuators ();
	draw_display ();
	if (pause) plot_time_data ();
}

/***************************************************************************/

static	void	atten_dec ()

{
	input_atten += 10.0;
	if (input_atten > 20.0) input_atten = 20.0;
	if (!pause) set_attenuators ();
	draw_display ();
	if (pause) plot_time_data ();
}

/***************************************************************************/

static	short	process_button (short xpos, short ypos)

{
	long	x, y;

	if (mouse_within_display (xpos, ypos)) return 0;
	getorigin (&x, &y);
	xpos = xpos - x;
	ypos = ypos - y;
	if ((xpos > TAXIS_INC_BUTTON_X) && (ypos > TAXIS_INC_BUTTON_Y)
		&& (xpos < TAXIS_INC_BUTTON_X + BUTTON_WIDTH)
		&& (ypos < TAXIS_INC_BUTTON_Y + BUTTON_WIDTH))
			taxis_inc ();
	else if ((xpos > TAXIS_DEC_BUTTON_X) && (ypos > TAXIS_DEC_BUTTON_Y)
		&& (xpos < TAXIS_DEC_BUTTON_X + BUTTON_WIDTH)
		&& (ypos < TAXIS_DEC_BUTTON_Y + BUTTON_WIDTH))
			taxis_dec ();
	else if ((xpos > BAXIS_INC_BUTTON_X) && (ypos > BAXIS_INC_BUTTON_Y)
		&& (xpos < BAXIS_INC_BUTTON_X + BUTTON_WIDTH)
		&& (ypos < BAXIS_INC_BUTTON_Y + BUTTON_WIDTH))
			baxis_inc ();
	else if ((xpos > BAXIS_DEC_BUTTON_X) && (ypos > BAXIS_DEC_BUTTON_Y)
		&& (xpos < BAXIS_DEC_BUTTON_X + BUTTON_WIDTH)
		&& (ypos < BAXIS_DEC_BUTTON_Y + BUTTON_WIDTH))
			baxis_dec ();
	else if ((xpos > TRIGGER_INC_BUTTON_X) && (ypos > TRIGGER_INC_BUTTON_Y)
		&& (xpos < TRIGGER_INC_BUTTON_X + BUTTON_WIDTH)
		&& (ypos < TRIGGER_INC_BUTTON_Y + BUTTON_WIDTH))
			trigger_inc ();
	else if ((xpos > TRIGGER_DEC_BUTTON_X) && (ypos > TRIGGER_DEC_BUTTON_Y)
		&& (xpos < TRIGGER_DEC_BUTTON_X + BUTTON_WIDTH)
		&& (ypos < TRIGGER_DEC_BUTTON_Y + BUTTON_WIDTH))
			trigger_dec ();
	else if ((xpos > ATTEN_INC_BUTTON_X) && (ypos > ATTEN_INC_BUTTON_Y)
		&& (xpos < ATTEN_INC_BUTTON_X + BUTTON_WIDTH)
		&& (ypos < ATTEN_INC_BUTTON_Y + BUTTON_WIDTH))
			atten_inc ();
	else if ((xpos > ATTEN_DEC_BUTTON_X) && (ypos > ATTEN_DEC_BUTTON_Y)
		&& (xpos < ATTEN_DEC_BUTTON_X + BUTTON_WIDTH)
		&& (ypos < ATTEN_DEC_BUTTON_Y + BUTTON_WIDTH))
			atten_dec ();
	else if ((xpos > RAXIS_INC_BUTTON_X) && (ypos > RAXIS_INC_BUTTON_Y)
		&& (xpos < RAXIS_INC_BUTTON_X + BUTTON_WIDTH)
		&& (ypos < RAXIS_INC_BUTTON_Y + BUTTON_WIDTH))
			raxis_inc ();
	else if ((xpos > RAXIS_DEC_BUTTON_X) && (ypos > RAXIS_DEC_BUTTON_Y)
		&& (xpos < RAXIS_DEC_BUTTON_X + BUTTON_WIDTH)
		&& (ypos < RAXIS_DEC_BUTTON_Y + BUTTON_WIDTH))
			raxis_dec ();
	else	return 0;
	return 1;		
}

/***************************************************************************/

static	void	draw_cursor (short xpos, short ypos)

{
	short			plot_mv = FALSE;
	float			vert1[2], vert2[2], start;
	long			x, y, idum;
	float			*td_ptr, *end_of_td_array;
	float			*pow_ptr;
	float			x_value, y_value;
	float			fxpos, fypos;
	char			buffer[50];
	static	fmfonthandle	font1, font_scaled;
	static	short		xpos_old, ypos_old;
	static	short		first_time = TRUE, cursor_on = TRUE;

	if ((xpos == xpos_old) && (ypos == ypos_old)) return;
	xpos_old = xpos;
	ypos_old = ypos;

	if (first_time) {
		fminit();
		if ((font1 = fmfindfont ("Times-Roman")) == 0) exit (1);
		font_scaled = fmscalefont (font1, FONT_SIZE_SMALL);
		first_time = FALSE;
	}

	if (winget () == twin.gid) {
		time_cursor_on = TRUE;
				
		getorigin (&x, &y);
		xpos = xpos - x;
		ypos = ypos - y;

		plot_time_data ();

		start = 1.0 + (float)tdis.x_offset_left
					+ (float)tdis.x_length 
					- ((float)time_points_per_display 
					* pixels_per_time_point);
		idum = (long)(((float)xpos-start) / pixels_per_time_point);
		fxpos = start + pixels_per_time_point * idum;
		if (rms_on) {
		    end_of_td_array = &time_display_buffer[time_points_per_display-1];
		    td_ptr = time_display_ptr;
		}
		else if (ypos > tdis.y_offset_bottom + tdis.y_length/2) {
		    end_of_td_array = &time_max_buffer[time_points_per_display-1];
		    td_ptr = time_max_ptr;
		}
		else {
		    end_of_td_array = &time_min_buffer[time_points_per_display-1];
		    td_ptr = time_min_ptr;
		}
		td_ptr += idum;
		
		if (td_ptr > end_of_td_array) { 
			idum = (long)td_ptr - (sizeof(SAMPLE) + (long)end_of_td_array);
			if (rms_on) td_ptr = &time_display_buffer[0];
			else if (ypos > tdis.y_offset_bottom + tdis.y_length/2) {
			    td_ptr = &time_max_buffer[0];
			}
			else {
			    td_ptr = &time_min_buffer[0];
			}
			td_ptr += idum / sizeof (float);
		}
		if (*td_ptr <= tlevel_axis_start) fypos = tdis.y_offset_bottom 
					+ AXIS_WIDTH;
		else if (*td_ptr >= tlevel_axis_end) fypos = tdis.y_offset_bottom 
					+ tdis.y_length - 1;
		else fypos = ((*td_ptr - tlevel_axis_start) * PIXELS_PER_TDB)
				+ (float)tdis.y_offset_bottom;

		y_value = (*td_ptr);
		x_value = time_axis_start + (fxpos - (float)tdis.x_offset_left)
				/ PIXELS_PER_SECOND;

		cpack (CURSOR_COLOR);

/*	vertical cursor				*/

		vert1[0] = fxpos;
		vert1[1] = tdis.y_offset_bottom + AXIS_WIDTH;
		vert2[0] = fxpos;
		vert2[1] = tdis.y_offset_bottom + tdis.y_length - 1;			

		bgnline ();
		    v2f (vert1);
		    v2f (vert2);
		endline ();

/*	horizontal cursor				*/

		vert1[0] = tdis.x_offset_left + AXIS_WIDTH;
		vert1[1] = fypos;
		vert2[0] = tdis.x_offset_left + tdis.x_length - 1;;
		vert2[1] = fypos;			

		bgnline ();
		    v2f (vert1);
		    v2f (vert2);
		endline ();

/*	cursor position indicators			*/

/*		cpack (BACKGROUND_COLOR);

		rectfi (tdis.x_offset_left,
			tdis.y_offset_bottom + tdis.y_length + 20,
			300 + tdis.x_offset_left,
			20 + tdis.y_offset_bottom + tdis.y_length + 20);
*/
		cpack (AXIS_COLOR);
		
		if ((tlevel_axis_end - tlevel_axis_start) <= 0.2) plot_mv = TRUE;
		
		fmsetfont (font_scaled);
		xpos = tdis.x_offset_left;
		ypos = tdis.y_offset_bottom + tdis.y_length + 20;
		cmov2i (xpos, ypos);
		if (time_axis_end - time_axis_start < 0.5) {
		    if (rms_on) sprintf (buffer, "Time: %5.2f ms     Level: %5.1f dB"
					, x_value * 1000.0, y_value);
		    else { 
			if (plot_mv) sprintf (buffer, "Time: %5.2f ms     Amplitude: %5.2f mV"
					, x_value * 1000.0, y_value * 1000.0);
			else sprintf (buffer, "Time: %5.2f ms     Amplitude: %5.3f V"
					, x_value * 1000.0, y_value);
		    }
		}
		else {
		    if (rms_on) sprintf (buffer, "Time: %5.3f sec     Level: %5.1f dB"
					, x_value, y_value);
		    else {
			if (plot_mv) sprintf (buffer, "Time: %5.3f sec     Amplitude: %5.2f mV"
					, x_value, y_value * 1000.0);
			else sprintf (buffer, "Time: %5.3f sec     Amplitude: %5.3f V"
					, x_value, y_value);
		    }
		}
		fmprstr (buffer);
	
		time_cursor_on = TRUE;
	}

	else if (winget () == fwin.gid) {
			
		getorigin (&x, &y);
		xpos = xpos - x;  

		plot_freq_data ();

		idum = (long)(freq_axis_start / kHz_per_component); 
		
		start = (float)fdis.x_offset_left + (((((float) idum + 1.0) * kHz_per_component)
			- freq_axis_start) * PIXELS_PER_KHZ);

		idum = (int)(((float)xpos - start) / pixels_per_freq_point);
		
		if (idum >= freq_points_per_display) return;
		freq_cursor_on = TRUE;
		
		fxpos = start + ((float)idum * pixels_per_freq_point);	
		
		pow_ptr = &power_spectrum[0];
		pow_ptr += idum; 

		fypos = ((*pow_ptr - flevel_axis_start) * PIXELS_PER_FDB)
				+ fdis.y_offset_bottom;

		if ((int)fypos <= fdis.y_offset_bottom) fypos = (float)(fdis.y_offset_bottom 
					+ AXIS_WIDTH);
		else if ((int)fypos >= fdis.y_offset_bottom + fdis.y_length)
					 fypos = (float)(fdis.y_offset_bottom 
					+ fdis.y_length - 1);

		y_value = (*pow_ptr);
		x_value = freq_axis_start + (fxpos - (float)fdis.x_offset_left)
				/ PIXELS_PER_KHZ;  

		if ((int)fxpos < fdis.x_offset_left + AXIS_WIDTH) 
		    fxpos = (float)(fdis.x_offset_left + AXIS_WIDTH);

		cpack (CURSOR_COLOR);

/*	vertical cursor				*/

		vert1[0] = fxpos;
		vert1[1] = (float)fdis.y_offset_bottom + AXIS_WIDTH;
		vert2[0] = fxpos;
		vert2[1] = (float)fdis.y_offset_bottom + (float)fdis.y_length - 1;			

		bgnline ();
		    v2f (vert1);
		    v2f (vert2);
		endline ();

/*	horizontal cursor				*/

		vert1[0] = fdis.x_offset_left + AXIS_WIDTH;
		vert1[1] = fypos;
		vert2[0] = fdis.x_offset_left + fdis.x_length - 1;;
		vert2[1] = fypos;			

		bgnline ();
		    v2f (vert1);
		    v2f (vert2);
		endline ();

/*	cursor position indicators			*/

/*		cpack (BACKGROUND_COLOR);

		rectfi (fdis.x_offset_left,
			fdis.y_offset_bottom + fdis.y_length + 20,
			300 + fdis.x_offset_left,
			20 + fdis.y_offset_bottom + fdis.y_length + 20);
*/
		cpack (AXIS_COLOR);

		fmsetfont (font_scaled);
		xpos = fdis.x_offset_left;
		ypos = fdis.y_offset_bottom + fdis.y_length + 20;
		cmov2i (xpos, ypos);
		if (spectrum_level) sprintf (buffer, "Freq: %5.2f Hz     Level: %5.1f dB / Hz"
					, x_value * 1000.0, y_value);
		else sprintf (buffer, "Freq: %5.2f Hz     Level: %5.1f dB"
					, x_value * 1000.0, y_value);
		fmprstr (buffer);
	
		time_cursor_on = TRUE;
	}
}

/***************************************************************************/

static	void	deselect (void)

{
	if (winget () == twin.gid) {
		time_selected = FALSE;
		plot_time_data ();
		cpack (BACKGROUND_COLOR);
		rectfi (tdis.x_offset_left + tdis.x_length / 2,
				tdis.y_offset_bottom + tdis.y_length + 20,
				300 + tdis.x_offset_left + tdis.x_length / 2,
				20 + tdis.y_offset_bottom + tdis.y_length + 20);
	}
	else if (winget () == fwin.gid) {
		freq_selected = FALSE;
		plot_freq_data ();
		cpack (BACKGROUND_COLOR);
		rectfi (fdis.x_offset_left + fdis.x_length / 2,
				fdis.y_offset_bottom + fdis.y_length + 5,
				300 + fdis.x_offset_left + fdis.x_length / 2,
				20 + fdis.y_offset_bottom + fdis.y_length + 20);
	}
}

/***************************************************************************/

static	void	highlight_selected (short xpos)

{
	long			x, y;
	static	short		begin_selection = TRUE;
	static  float		start, end;
		
	getorigin (&x, &y);
	xpos -= (short)x;

	if (winget () == twin.gid) {
		if (!time_selected) {
			tpixels_selected_start = xpos; 
			time_selected = TRUE;
			start = time_axis_start + (float)(tpixels_selected_start - 
				tdis.x_offset_left) / PIXELS_PER_SECOND;;
		}
		tpixels_selected_end = xpos;
		end = time_axis_start + (float)(tpixels_selected_end - 
			tdis.x_offset_left) / PIXELS_PER_SECOND;
		if (start > end) {
			time_selected_start = end;
			time_selected_end = start;
		}
		else {
			time_selected_start = start;
			time_selected_end = end;
		}
		plot_time_data ();
		cpack (BACKGROUND_COLOR);

		rectfi (tdis.x_offset_left + tdis.x_length / 2,
				tdis.y_offset_bottom + tdis.y_length + 20,
				300 + tdis.x_offset_left + tdis.x_length / 2,
				20 + tdis.y_offset_bottom + tdis.y_length + 20);

		write_selected_time ();
	}

	else if (winget () == fwin.gid) {
		if (!freq_selected) {
			fpixels_selected_start = xpos;
			freq_selected_start = (float)(fpixels_selected_start - 
				fdis.x_offset_left) / PIXELS_PER_KHZ;
			freq_selected = TRUE;
			start = freq_axis_start + freq_selected_start;
		}
		fpixels_selected_end = xpos;
		end = freq_axis_start + (float)(fpixels_selected_end - 
			fdis.x_offset_left) / PIXELS_PER_KHZ;
		if (start > end) {
			freq_selected_start = end;
			freq_selected_end = start;
		}
		else {
			freq_selected_start = start;
			freq_selected_end = end;
		}
		plot_freq_data ();
		cpack (BACKGROUND_COLOR);

		rectfi (fdis.x_offset_left + fdis.x_length / 2,
				fdis.y_offset_bottom + fdis.y_length + 20,
				300 + fdis.x_offset_left + fdis.x_length / 2,
				20 + fdis.y_offset_bottom + fdis.y_length + 20);

		write_selected_freq ();
	}
}

/***************************************************************************/

static	void	create_time_window (void)

{
	prefsize (TIME_X_SCREEN_SIZE, TIME_Y_SCREEN_SIZE);
	
	twin.gid = winopen ("Time Display");
		
	RGBmode ();
	gconfig ();
	
	winset (twin.gid);
	draw_display ();
}

/***************************************************************************/

static	void	create_freq_window (void)

{
	prefsize (FREQ_X_SCREEN_SIZE, FREQ_Y_SCREEN_SIZE);
	
	fwin.gid = winopen ("Spectrum Display");
		
	RGBmode ();
	gconfig ();
	
	winset (fwin.gid);
	draw_display ();
}

/***************************************************************************/

static	void	create_spec_window (void)

{
	prefsize (SPEC_X_SCREEN_SIZE, SPEC_Y_SCREEN_SIZE);
	swin.gid = winopen ("Spectrogram Display");  
		
	RGBmode ();
	gconfig ();
	
	winset (swin.gid);
	draw_display ();
}

/***************************************************************************/

static	void	create_water_window (void)

{
	prefsize (WATER_X_SCREEN_SIZE, WATER_Y_SCREEN_SIZE);
	wwin.gid = winopen ("Waterfall Display");  
		
	RGBmode ();
	gconfig ();
	
	winset (wwin.gid);
	draw_display ();
}

/***************************************************************************/

static	void	process_input (void)

{
	int		i, j;
	int		dc_sum = 0;
	long		main_time_points, idum;
	SAMPLE		*end_of_mt_array, *mt_ptr;
	SAMPLE		*input_ptr;
	float		*end_of_td_array, *end_of_max_array, *end_of_min_array;
	float		sum, max, min, trigger_amp;
	
	if (trigger_on) trigger_amp = (SAMPLE) pow (10.0, (trigger_level - level_scale) / 20.0);

	input_ptr = &input_buffer[0];
	if ((input_channel == 'R') && (input_channels == 2)) input_ptr++;
	
	main_time_points = (long) ((float)input_sample_rate * MAX_DISPLAY_SECONDS);
	end_of_mt_array = &main_time_buffer[main_time_points-1];
	end_of_td_array = &time_display_buffer[time_points_per_display-1];
	end_of_max_array = &time_max_buffer[time_points_per_display-1];
	end_of_min_array = &time_min_buffer[time_points_per_display-1];
	
	if (begin_new_input) { 	
		main_time_ptr = &main_time_buffer[0];
		time_display_ptr = &time_display_buffer[0];
		time_max_ptr = &time_max_buffer[0];
		time_min_ptr = &time_min_buffer[0];
		begin_new_input = FALSE;
	}
	last_input_end_time = time_axis_end;	
	
	if (dc_correction == DC_DEFAULT) {
	    if (line_input) {
		if ((int)input_atten == 0) dc = DC_LINE_0; 
		if ((int)input_atten == 10) dc = DC_LINE_10;
		if ((int)input_atten == 20) dc = DC_LINE_20;
	    }
	    else {
		if ((int)input_atten == 0) dc = DC_MIC_0; 
		if ((int)input_atten == 10) dc = DC_MIC_10;
		if ((int)input_atten == 20) dc = DC_MIC_20;
	    }
	}
	else if (dc_correction == DC_OFF) dc = 0.0;
		
	for (i = 0; i < time_points_per_buffer; i++){
		sum = 0.0;
		max = -100000.0;
		min = 100000.0;
		for (j = 0; j < samples_per_time_point; j++){
			if (dc_correction == DC_AUTO) dc = ((dc * 1999.0) - (float)*input_ptr) / 2000.0;
			*input_ptr += (int)dc;
			if (trigger_on && !triggered) {
				if (*input_ptr > trigger_amp)
					triggered = TRUE;
			}
			if (rms_on) sum += (float)(*input_ptr) * (float)(*input_ptr);
			else if (envelope_on) {
			    if ((float)*input_ptr > max) {
				max = (float)*input_ptr;
			    }
			    if ((float)*input_ptr < min) {
				min = (float)*input_ptr;
			    }
			}    
			*main_time_ptr = (*input_ptr);
			if (++main_time_ptr > end_of_mt_array)  
					main_time_ptr = &main_time_buffer[0];
			input_ptr += input_channels;	
		}
		if (rms_on) {
		    if (sum <= 0.0) *time_display_ptr = -10000.0;
		    else *time_display_ptr = level_scale + 10.0 * log10 (sum 
				/ (float)samples_per_time_point);
		    if (++time_display_ptr > end_of_td_array) 
				time_display_ptr = &time_display_buffer[0];
		}
		else {
		    *time_max_ptr = dc_shift + (amp_scale * max);
		    *time_min_ptr = dc_shift + (amp_scale * min);
		    if (++time_max_ptr > end_of_max_array) time_max_ptr = &time_max_buffer[0];
		    if (++time_min_ptr > end_of_min_array) time_min_ptr = &time_min_buffer[0]; 
		}
		 
	}
}

/***************************************************************************/

static	void	process_buffer (void)

{
	int		i, j;
	long		main_time_points, idum;
	SAMPLE		*end_of_mt_array, *mt_ptr;
	float		*end_of_td_array, *end_of_max_array, *end_of_min_array;
	float		sum, max, min;
	
/*	if (!rms_on && !envelope_on) return; */
	
	main_time_points = (long) ((float)input_sample_rate * MAX_DISPLAY_SECONDS);
	end_of_mt_array = &main_time_buffer[main_time_points-1];
	end_of_td_array = &time_display_buffer[time_points_per_display-1];
	end_of_max_array = &time_max_buffer[time_points_per_display-1];
	end_of_min_array = &time_min_buffer[time_points_per_display-1];
	
	mt_ptr = main_time_ptr;
	mt_ptr -= (long)((last_input_end_time - time_axis_start) * (float)input_sample_rate);
	if (mt_ptr < &main_time_buffer[0]) {
		idum = (long)end_of_mt_array + sizeof (SAMPLE) - (long)&main_time_buffer[0]; 
		mt_ptr += idum/sizeof (SAMPLE);
	}
	time_display_ptr = &time_display_buffer[0];
	time_min_ptr = &time_min_buffer[0];
	time_max_ptr = &time_max_buffer[0]; 
			
	for (i = 0; i < time_points_per_display; i++){
		sum = 0.0;
		max = -100000.0;
		min = 100000.0;
		for (j = 0; j < samples_per_time_point; j++){
		    if (rms_on) sum += (float)(*mt_ptr) * (float)(*mt_ptr);
		    else { 
			if ((float)*mt_ptr > max) {
			    max = (float)*mt_ptr;
			}
			if ((float)*mt_ptr < min) {
			    min = (float)*mt_ptr;
			}
		    }
		    if (++mt_ptr > end_of_mt_array)  mt_ptr = &main_time_buffer[0];	
		}
		if (rms_on) {
		    if (sum <= 0.0) *time_display_ptr = -10000.0;
		    else *time_display_ptr = level_scale + 10.0 * log10 (sum 
				/ (float)samples_per_time_point);
		    if (++time_display_ptr > end_of_td_array) 
				time_display_ptr = &time_display_buffer[0]; 
		}
		else {
		    *time_max_ptr = dc_shift + (amp_scale * max);
		    *time_min_ptr = dc_shift + (amp_scale * min);
		    *time_display_ptr = *time_max_ptr;
		    if (++time_display_ptr > end_of_td_array) 
				time_display_ptr = &time_display_buffer[0];
		    if (++time_max_ptr > end_of_max_array) 
				time_max_ptr = &time_max_buffer[0];
		    if (++time_min_ptr > end_of_min_array) 
				time_min_ptr = &time_min_buffer[0];
		     
		}

		
	}
}

/***************************************************************************/

int 	process_filedata (char* filename, char channel)

{
    int			fd;
    AFfilehandle	file;			    /* AF file handle */
    long		filefmt;		    /* file format */
    int			i, j;
    long		main_time_points, time_points, count;
    SAMPLE		*end_of_mt_array;
    SAMPLE		input_array [500000], *input_ptr;
    float		*end_of_td_array, *end_of_max_array, *end_of_min_array;
    float		sum, max, min, trigger_amp;
    long    		samples_per_display;
    long		miscid;
    short		channels = 2;
    float		calib_level;	
	
    if (begin_new_input) { 	
	main_time_ptr = &main_time_buffer[0];
	time_display_ptr = &time_display_buffer[0];
	time_max_ptr = &time_max_buffer[0];
	time_min_ptr = &time_min_buffer[0];
	begin_new_input = FALSE;
    }
    last_input_end_time = MAX_DISPLAY_SECONDS; /* time_axis_end; */
/*    
    if (!(fd = open (filename, "r"))) return 0;   
    if (!(file = AFopenfd(fd, "r", AF_NULL_FILESETUP))) return 0;
*/
    if (!(file = AFopenfile (filename, "r", AF_NULL_FILESETUP))) return 0;
    
    count = AFgetframecnt (file, AF_DEFAULT_TRACK);
    input_sample_rate = AFgetrate (file, AF_DEFAULT_TRACK);
    channels = AFgetchannels (file, AF_DEFAULT_TRACK);
    
    if (count > (MAX_DISPLAY_SECONDS * input_sample_rate * 0.8)) 
	count = (MAX_DISPLAY_SECONDS * input_sample_rate * 0.8);
    
    set_tdisplay_parameters ();
    
    main_time_points = (long) ((float)input_sample_rate * MAX_DISPLAY_SECONDS);
    end_of_mt_array = &main_time_buffer[main_time_points-1];
    end_of_td_array = &time_display_buffer[time_points_per_display-1];
    end_of_max_array = &time_max_buffer[time_points_per_display-1];
    end_of_min_array = &time_min_buffer[time_points_per_display-1];

    time_display_ptr = &time_display_buffer[0];
    time_min_ptr = &time_min_buffer[0];
    time_max_ptr = &time_max_buffer[0];
    
    for (i = 0; i < time_points_per_display; i++) {
	time_max_buffer[i] = time_min_buffer[i] = 0;
    }
    if (rms_on) {
	for (i = 0; i < time_points_per_display; i++) {
	    time_display_buffer[i] = -10000.0;
	}
    }
    else {
	for (i = 0; i < time_points_per_display; i++) {
	    time_display_buffer[i] = 0.0;
	}
    }
    
    for (i = 0; i < main_time_points; i++) main_time_buffer[i] = 0;
    for (i = 0; i < 500000; i++) input_array[i] = 0;
    
    AFreadframes (file, AF_DEFAULT_TRACK, input_array, count);
    
    
    input_ptr = &input_array[0];
    if ((channels == 2) && (channel == 'R')) input_ptr++;
/* */
    samples_per_display = (long) ((float)input_sample_rate * MAX_DISPLAY_SECONDS);
				/*	* (time_axis_end - time_axis_start));  */
    
    
    count += (long)((float)samples_per_display / 10.0);
/*    time_points = (int)((float)count / (float)samples_per_time_point);
    count = time_points * samples_per_time_point;
*/ 
    input_sum = 0;
    for (i = 0; i < count; i++) {
	*main_time_ptr = (*input_ptr);
	input_sum += abs(*input_ptr);  
	if (++main_time_ptr > end_of_mt_array)  main_time_ptr = &main_time_buffer[0];
	if (channels == 2) input_ptr += 2;
	else input_ptr++;
    }
    

    if (AFgetmiscids (file, NULL) == 1) {
	AFgetmiscids (file, &miscid);
	if (AFgetmisctype (file, miscid) == AF_MISC_AIFF_APPL) {
	    AFreadmisc (file, miscid, &calib_level, sizeof (float));
	    level_scale = calib_level - 87.0;
	}
	else level_scale = FILE_CALIBRATION;
    }
    else  level_scale = FILE_CALIBRATION; 
    
    amp_scale = (MAX_VOLTAGE_IN / 32768.0);
   
    process_buffer ();
       
    /*   
    time_points = 20 + (int)((float)count / (float)samples_per_time_point);
    
    for (i = 0; i < time_points; i++){
	sum = 0.0;
	max = -100000.0;
	min = 100000.0;
	for (j = 0; j < samples_per_time_point; j++){
	   
	    if (rms_on) sum += (float)(*input_ptr) * (float)(*input_ptr);
	    else if (envelope_on) {
		if ((float)*input_ptr > max) {
		    max = (float)*input_ptr;
		}
		if ((float)*input_ptr < min) {
		    min = (float)*input_ptr;
		}
	    }    
	    *main_time_ptr = (*input_ptr);
	    if (++main_time_ptr > end_of_mt_array)  
		main_time_ptr = &main_time_buffer[0];
	    input_ptr += 2;	
	}
	if (rms_on) {
	    if (sum <= 0.0) sum = 0.000001;
	    *time_display_ptr = level_scale + 10.0 * log10 (sum 
				/ (float)samples_per_time_point);
	    if (++time_display_ptr > end_of_td_array) 
		    time_display_ptr = &time_display_buffer[0];
	}
	else {
	    *time_max_ptr = dc_shift + (amp_scale * max);
	    *time_min_ptr = dc_shift + (amp_scale * min);
	    if (++time_max_ptr > end_of_max_array) time_max_ptr = &time_max_buffer[0];
	    if (++time_min_ptr > end_of_min_array) time_min_ptr = &time_min_buffer[0]; 
	}
		 
    }
    */  
    AFclosefile (file);
    close (fd);
    return 1;   
}

/***************************************************************************/

static	void	make_window (int window_size)

{
    int	    i;
    float   arg;
    
    if ((window_type != window_type_old) || (window_size != window_size_old)) {
    
	if (window_array) free (window_array);
        window_array = (float *) malloc (window_size * sizeof (float)); 
        
        if (window_type == HAMMING) {
    	    arg = 2.0 * PI / (window_size - 1);
	    for (i = 0; i < window_size; i++) window_array[i] = 0.54 - 0.46 * cos (arg * i);  
	}
	if (window_type == HANNING) {
	    arg = 2.0 * PI / (window_size - 1);
	    for (i = 0; i < window_size; i++) window_array[i] = 0.5 - 0.5 * cos (arg * i);  
	}
	if (window_type == BARTLETT) {
	    arg = 2.0 / (window_size - 1);
	    for (i = 0; i <= (window_size - 1) / 2; i++) window_array[i] = i * arg; 
	    for (; i < window_size; i++) window_array[i] = 2.0 - i * arg;
	}
	if (window_type == BLACKMAN) {
	    arg = 2.0 * PI / (window_size - 1);
	    for (i = 0; i < window_size; i++) 
		window_array[i] = 0.42 - 0.5 * cos (arg * i) + 0.08 * cos (2 * arg * i);
	} 
	if (window_type == BLACKMAN_HARRIS) {   
	     arg = 2.0 * PI / (window_size - 1);
	     for (i = 0; i < window_size; i++) 
		window_array[i] = 0.35875 - 0.48829 * cos (arg * i) + 0.14128 * cos (2 * arg * i) 
					- 0.01168 * cos (3 * arg * i);
	}
    }
}

/***************************************************************************/

static	void	get_spectrum ()

{
	int		i, j, end, fft_points;
	int		get_components;
	static	int	get_components_old = 0;
	static	float	kHz_per_component_old = 0;
	long		main_time_points, idum;
	float		*enhance_ptr;
	SAMPLE		selection[MAX_DISPLAY_SECONDS*100000];
	SAMPLE		*end_of_mt_array, *mt_ptr, *sel_ptr;
	float		*fd_ptr, sum = 0.0;
	float		scale;
	float		sum1, sum2;
	
	if (!waterfall_nrt) {
	    if (time_selected == FALSE) return;
	    if (fwin.gid == 0) {
		idum = winget ();
		create_freq_window ();
		winset (idum);
	    }
	}
	
	main_time_points = (long) ((float)input_sample_rate * MAX_DISPLAY_SECONDS);
	end_of_mt_array = &main_time_buffer[main_time_points-1];
	
	mt_ptr = main_time_ptr;
	mt_ptr -= (long)((last_input_end_time - time_selected_start) * (float)input_sample_rate);
	if (mt_ptr < &main_time_buffer[0]) {
		idum = (long)end_of_mt_array + sizeof (SAMPLE) - (long)&main_time_buffer[0]; 
		mt_ptr += idum / sizeof (SAMPLE);
	}
	sel_ptr = &selection[0];
	
	end = (int)((time_selected_end - time_selected_start) * input_sample_rate); 
	if (end == 0) return;
	new_fft_selection = FALSE;


	if (window_type == RECTANGULAR) {
	    for (i = 0; i < end; i++) {
		sel_ptr[i] = (*mt_ptr);	
		if (++mt_ptr > end_of_mt_array) mt_ptr = &main_time_buffer[0];	 
	    }
	}
	else {
	    make_window (end);
	    window_size_old = end;
	    sum1 = 0.0;
	    sum2 = 0.0;
	    for (i = 0; i < end; i++) {
		sum1 += (*mt_ptr) * (*mt_ptr);
		sel_ptr[i] = (*mt_ptr) * window_array[i];
		sum2 += sel_ptr[i] * sel_ptr[i];	
		if (++mt_ptr > end_of_mt_array) mt_ptr = &main_time_buffer[0];	 
	    }
	    if ((sum1 > 0.0) && (sum2 > 0.0)) {
		scale = sqrt (sum1/sum2);
		for (i = 0; i < end; i++) sel_ptr[i] *= scale;
	    }
	} 
	
	window_type_old = window_type;
	
	fft_points = 1;
        while (fft_points < end) fft_points<<=1;
	for (i = end; i < fft_points; i++) sel_ptr[i] = 0;
	freq_components = fft_points>>1;

	get_components = (1.2 * freq_components) + 20;
	
	set_fdisplay_parameters ();
	
	if ((get_components != get_components_old) || (kHz_per_component_old != kHz_per_component)) {
	    get_components_old = get_components;
	    kHz_per_component_old = kHz_per_component;
	    enhance_ptr = &enhance_array[0];
	    for (i = 0; i < get_components; i++) {
		*enhance_ptr++ = pow(10.0, kHz_per_component * i * PRE_EMPHASIS / 10.0);
	    }
	}


/*	the clever fft bit (stolen)		*/

	spectrum (&selection[0], fft_points, &freq_data[0], get_components);

	fd_ptr = &freq_data[0];

	scale = 2.0 * ((float)fft_points / (float)end);

	sum = 0.0;
	if (enhance_on) {
	    enhance_ptr = &enhance_array[0];
	    for (i = 0; i < get_components; i++){ 
		*fd_ptr++ = *fd_ptr * scale * (*enhance_ptr++);
	    }
	}
	else {
	    for (i = 0; i < get_components; i++){ 
		*fd_ptr++ = *fd_ptr * scale;
	    }
	}
	
	if (!waterfall_nrt) {set_fdisplay_parameters (); calc_freq_data (); plot_freq_data ();}
}

/***************************************************************************/

static	void	get_spectrum_rt ()

{
	int		i, j, get_components, first_time1;
	long		idum, ib_size;
	static unsigned long	average_count = 1;
	unsigned long	ac_minus_one = 0;
	float		*sd_ptr, *fd_ptr;
	float		scale, scale2;
	static	SAMPLE	spec_input_buffer[50000], *sib_ptr1, *sib_ptr2, sib_ptr3[20000], *ib_ptr;
	static	int	get_components_old = 0, first_time = TRUE;
	static	float	kHz_per_component_old = 0.0;
	float		*enhance_ptr;
	float		trigger_amp;
	float		sum1, sum2;
	
	if (trigger_on) trigger_amp = (SAMPLE) pow (10.0, (trigger_level - level_scale) / 20.0);
	
	ib_ptr = &input_buffer[0];
	if ((input_channel == 'R') && (input_channels == 2)) ib_ptr++;
	
	if (first_time) {
	    first_time = FALSE;
	    sib_ptr1 = &spec_input_buffer[0];
	}
 	if (!time_on) {
	    if (dc_correction == DC_DEFAULT) {
		if (line_input) {
		    if ((int)input_atten == 0) dc = DC_LINE_0; 
		    if ((int)input_atten == 10) dc = DC_LINE_10;
		    if ((int)input_atten == 20) dc = DC_LINE_20;
		}
		else {
		    if ((int)input_atten == 0) dc = DC_MIC_0; 
		    if ((int)input_atten == 10) dc = DC_MIC_10;
		    if ((int)input_atten == 20) dc = DC_MIC_20;
		}
	    }
	    else if (dc_correction == DC_OFF) dc = 0.0;
	}   
	
	ib_size = input_buffer_size/input_channels;
	if (trigger_on && !triggered) {
		for (i = 0; i < ib_size; i++) {	
		    if (!time_on) {
			if (dc_correction == DC_AUTO) dc = ((dc * 1999.0) - (float)*ib_ptr) / 2000.0;
			*ib_ptr += (int)dc;
		    }   
		    if (*ib_ptr > trigger_amp) triggered = TRUE;
		    *sib_ptr1++ = *ib_ptr;
		    ib_ptr += input_channels;
		}
	}
	else for (i = 0; i < ib_size; i++) {
	    if (!time_on) {
		if (dc_correction == DC_AUTO) dc = ((dc * 1999.0) - (float)*ib_ptr) / 2000.0;
		*ib_ptr += (int)dc;
	    } 
	    *sib_ptr1++ = *ib_ptr; 
	    ib_ptr += input_channels;
	}
	
	sib_ptr2 = &spec_input_buffer[0]; 
	
	spec_components = spec_fft_points / 2;

	if ((spectrum_on || waterfall_on) && !spectrogram_on) get_components = (int)((float)spec_components 
		    * (float)freq_axis_end * 2000.0 / (float)input_sample_rate);
	else if (!(spectrum_on || waterfall_on) && spectrogram_on) get_components = (int)((float)spec_components 
		    * (float)spec_axis_end * 2000.0 / (float)input_sample_rate);
	else if ((spectrum_on || waterfall_on) && spectrogram_on) {
	    if (freq_axis_end > spec_axis_end) get_components = (int)((float)spec_components 
		    * (float)freq_axis_end * 2000.0 / (float)input_sample_rate);
	    else get_components = (int)((float)spec_components 
		    * (float)spec_axis_end * 2000.0 / (float)input_sample_rate);
	}


	get_components *= 1.2; 

	if (get_components >= spec_components) get_components = spec_components;
	
	get_components += 20;
	
	if (spectrogram_on) {
	    set_sdisplay_parameters ();
	}
	if (spectrum_on || waterfall_on) {
	    freq_components = spec_components;
	    set_fdisplay_parameters ();
	    scale = 2.0;
	}
	
	
	if (spectrogram_on) {
	    if ((get_components != get_components_old) || (skHz_per_component != kHz_per_component_old)) {
		get_components_old = get_components;
		kHz_per_component_old = skHz_per_component; 
		enhance_ptr = &enhance_array[0];
		for (i = 0; i < get_components * 2; i++) {
		    *enhance_ptr++ = pow(10.0, skHz_per_component * i * PRE_EMPHASIS / 10.0);
		}
	    }	
	}
	else {
	    if ((get_components != get_components_old) || (kHz_per_component != kHz_per_component_old)) {
		get_components_old = get_components;
		kHz_per_component_old = kHz_per_component; 
		enhance_ptr = &enhance_array[0];
		for (i = 0; i < get_components; i++) {
		    *enhance_ptr++ = pow(10.0, kHz_per_component * i * PRE_EMPHASIS / 10.0);
		}
	    }	
	}
		 

	j = 0;
	first_time1 = TRUE;
	while ((long)sib_ptr2 < (long)sib_ptr1 - (spec_fft_points * sizeof(SAMPLE))) {

	    if (window_type != RECTANGULAR) {
		make_window (spec_fft_points);
		window_size_old = spec_fft_points;
		
		sum1 = 0.0;
		sum2 = 0.0;
		for (i = 0; i < spec_fft_points; i++) {
		    sum1 += sib_ptr2[i] * sib_ptr2[i];
		    sib_ptr3[i] = sib_ptr2[i] * window_array[i];
		    sum2 += sib_ptr3[i] * sib_ptr3[i];		 
		}
		if ((sum1 > 0.0) && (sum2 > 0.0)) {
		    scale2 = sqrt (sum1/sum2);
		    for (i = 0; i < spec_fft_points; i++) sib_ptr3[i] *= scale2;
		}
		spectrum (sib_ptr3, spec_fft_points, &spec_data[j][0], get_components);
	    }
	    else spectrum (sib_ptr2, spec_fft_points, &spec_data[j][0], get_components);
	    
	    window_type_old = window_type;
	    
	    
	    
	    if ((spectrum_on || waterfall_on) && first_time1) {
		first_time1 = FALSE;
		fd_ptr = &freq_data[0];
		if (!spec_average_on || average_first_time) {
		    for (i = 0; i < get_components; i++, fd_ptr++) *fd_ptr = 0.0;
		    average_count = 1;
		}
		if (spec_average_on) average_first_time = FALSE;
	    } 
	    if (spectrum_on || waterfall_on) {
		if (enhance_on) {
		    sd_ptr = &spec_data[j][0];
		    enhance_ptr = &enhance_array[0];
		    for (i = 0; i < get_components; i++, sd_ptr++){
			*sd_ptr *= (*enhance_ptr++);
		    }
		}
		sd_ptr = &spec_data[j][0];
		fd_ptr = &freq_data[0];
		enhance_ptr = &enhance_array[0];
		if (spec_average_on) {
		    ac_minus_one = average_count - 1;
		    for (i = 0; i < get_components; i++, sd_ptr++, fd_ptr++) {     
			*fd_ptr = ((*fd_ptr * ac_minus_one) + (*sd_ptr * scale)) / average_count;
		    }
		    average_count++;
		}
		else {
		    for (i = 0; i < get_components; i++, sd_ptr++, fd_ptr++) {  
			*fd_ptr = *sd_ptr;    
		    }
		}
	    }
	    if (spectrogram_on) {
		if (enhance_on) {
		    sd_ptr = &spec_data[j][0];
		    enhance_ptr = &enhance_array[0];
		    for (i = 0; i < get_components; i++, sd_ptr++){
			*sd_ptr *= (*enhance_ptr++);
		    }
		}
	    }
	    sib_ptr2 += spec_fft_points / spec_overlap;
	    j++;
	}
	
	if (j == 0) return;
	
	ffts_per_buffer = j;
	
	fd_ptr = &freq_data[0];
	if ((spectrum_on || waterfall_on) && !spec_average_on) {
	    scale /= (float)ffts_per_buffer;
	    for (i = 0; i < get_components; i++, fd_ptr++){
		*fd_ptr *= scale;
	    }
	}
	if (spectrum_on) {
	    calc_freq_data ();
	    plot_freq_data ();
	}
	if (waterfall_on) {
	    plot_waterfall ();
	}
	idum = ((long)sib_ptr1 - (long)sib_ptr2) / sizeof(SAMPLE); 
	
	sib_ptr1 = &spec_input_buffer[0];
	
	for (i = 0; i < idum; i++) {
	     *sib_ptr1++ = *sib_ptr2++;
	}
	if (spectrogram_on) {
    	    if (scrolling_on) plot_spec_data2 ();
	    else plot_spec_data ();
	} 
}

/***************************************************************************/

static	void	get_spectrum_nrt ()

{
	int		i, j, get_components;
	long		idum, current_gid;
	float		*sd_ptr;
	float		scale;
	static	SAMPLE	spec_input_buffer[50000], *sib_ptr1;
	static	int	get_components_old = 0;
	static	float	kHz_per_component_old = 0.0;
	float		*enhance_ptr;
	long		main_time_points, ffts_per_display;
	SAMPLE		*end_of_mt_array, *mt_ptr;
	float		*end_of_td_array;
	float		sum, sum1, sum2;
	
	reset_spectro = TRUE;
	single_scroll = TRUE;
	
	main_time_points = (long) ((float)input_sample_rate * MAX_DISPLAY_SECONDS);
	end_of_mt_array = &main_time_buffer[main_time_points-1];
	
	mt_ptr = main_time_ptr;
	mt_ptr -= (long)((last_input_end_time - time_axis_start) * (float)input_sample_rate);
	if (mt_ptr < &main_time_buffer[0]) {
		idum = (long)end_of_mt_array + sizeof (SAMPLE) - (long)&main_time_buffer[0]; 
		mt_ptr += idum/sizeof (SAMPLE);
	}
	spec_components = spec_fft_points / 2;

	get_components = (int)(1.2 * (float)spec_components 
		    * (float)spec_axis_end * 2000.0 / (float)input_sample_rate);

	if (get_components >= spec_components) get_components = spec_components;
	
	set_sdisplay_parameters ();
	
	get_components += 20;
		 
	if ((get_components != get_components_old) || (skHz_per_component != kHz_per_component_old)) {
	    get_components_old = get_components;
	    kHz_per_component_old = skHz_per_component;
	    enhance_ptr = &enhance_array[0];
	    for (i = 0; i < get_components; i++) {
		*enhance_ptr++ = pow(10.0, skHz_per_component * i * PRE_EMPHASIS / 10.0);
	    }	
	}
	sib_ptr1 = &spec_input_buffer[0];
	
	ffts_per_buffer = 1;
	ffts_per_display = (int)((float)spec_overlap * (float)input_sample_rate * (time_axis_end - time_axis_start) 
			    / (float)spec_fft_points);
	
	for (j = 0; j < ffts_per_display; j++) {
	    
	    if (window_type == RECTANGULAR) {
		for (i = 0; i < spec_fft_points; i++) {
		    sib_ptr1[i] = (*mt_ptr);	
		    if (++mt_ptr > end_of_mt_array) mt_ptr = &main_time_buffer[0];	 
		}
	    }
	    else {
		make_window (spec_fft_points);
		window_size_old = spec_fft_points;
		
		sum1 = 0.0;
		sum2 = 0.0;
		for (i = 0; i < spec_fft_points; i++) {
		    sum1 += (*mt_ptr) * (*mt_ptr);
		    sib_ptr1[i] = (*mt_ptr) * window_array[i];
		    sum2 += sib_ptr1[i] * sib_ptr1[i];	
		    if (++mt_ptr > end_of_mt_array) mt_ptr = &main_time_buffer[0];	 
		}
		if ((sum1 > 0.0) && (sum2 > 0.0)) {
		    scale = sqrt (sum1/sum2);
		    for (i = 0; i < spec_fft_points; i++) sib_ptr1[i] *= scale;
		}
	    } 
	    
	    window_type_old = window_type;
	    
	    mt_ptr += (spec_fft_points / spec_overlap) - spec_fft_points; 	
	    sib_ptr1 = &spec_input_buffer[0];
	    spectrum (sib_ptr1, spec_fft_points, &spec_data[0][0], get_components);
	    
	    sd_ptr = &spec_data[0][0];
	    
	    if (enhance_on) {
		enhance_ptr = &enhance_array[0];
		for (i = 0; i < get_components; i++, sd_ptr++){
		    *sd_ptr *= (*enhance_ptr++);
		}
		/* for(i = 0; i < 20; i++) *sd_ptr++ = 0.0; */
	    } 
	/*    else {
		sd_ptr += get_components;
		for(i = 0; i < 20; i++) *sd_ptr++ = 0.0;
	    } */
	    plot_spec_data2 (); 
	}
	single_scroll = FALSE;
	
	if ((current_gid = winget ()) != swin.gid) {
	    winset (swin.gid); draw_spec_axes (); winset (current_gid);
	}
	else draw_spec_axes ();
}

/***************************************************************************/

static	void	get_waterfall_nrt ()

{
    int		i;
    int		end, components_per_point;
    long	idum, main_time_points;
    float	time, time_increment;
    float	freq, freq_increment;
    FILE	*fptr;
    float	*td_ptr, *end_of_td_array, *pow_ptr;
    SAMPLE	*mt_ptr, *end_of_mt_array;

    float   time_start_old, time_end_old, increment;
    long    fft_points_old, x, y, current_gid;	
	
    if (!waterfall_on || !pause) return;
    
    current_gid = winget ();
    winset (wwin.gid);
    getorigin (&x, &y);
    if ((y + wdis.y_offset_bottom) <= 0) {
	winset (current_gid);
	return; 
    }
    
    winpop ();
    
/*    clear_screen ();   
    draw_water_axes ();
    return; */
   
    time_start_old = time_selected_start;
    time_end_old = time_selected_end;
    fft_points_old = spec_fft_points;
    
    waterfall_nrt = TRUE;
   
/*    increment = (time_axis_end - time_axis_start) / 40.0;
    spec_fft_points = (int)(increment * input_sample_rate); */
    
    increment = spec_fft_points/input_sample_rate;
    
    time_selected_start = time_axis_start;
    time_selected_end = time_axis_start + increment;
    
    set_fdisplay_parameters ();
    set_sdisplay_parameters ();
    
    
    for (;time_selected_end < time_axis_end; time_selected_start += increment, time_selected_end += increment) {
	get_spectrum ();
	plot_waterfall ();
    }
    
    waterfall_nrt = FALSE;
    time_selected_start = time_start_old;
    time_selected_end = time_end_old; 
    spec_fft_points = fft_points_old;
}

/***************************************************************************/

static	void	play_selection ()

{
	int		i, j, end;
	long		main_time_points, idum;
	SAMPLE		selection[MAX_DISPLAY_SECONDS*100000];
	SAMPLE		*end_of_mt_array, *mt_ptr, *sel_ptr;
	long		buffer_length = 2;
	long		PVbuffer[2] = {AL_OUTPUT_RATE, 0};

	if (time_selected == FALSE) return;
	
	open_audio_output ();

	/* 	set output sample rate = input sample rate	*/

	PVbuffer[1] = 1000*(long)(input_sample_rate/1000.0);
	ALsetparams (AL_DEFAULT_DEVICE, PVbuffer, buffer_length);
	
	main_time_points = (long) ((float)input_sample_rate * MAX_DISPLAY_SECONDS);
	end_of_mt_array = &main_time_buffer[main_time_points-1];
	
	mt_ptr = main_time_ptr;
	mt_ptr -= (long)((last_input_end_time - time_selected_start) * (float)input_sample_rate);
	if (mt_ptr < &main_time_buffer[0]) {
		idum = (long)end_of_mt_array + sizeof (SAMPLE) - (long)&main_time_buffer[0]; 
		mt_ptr += idum / sizeof (SAMPLE);
	}
	sel_ptr = &selection[0];
	
	end = (int)((time_selected_end - time_selected_start) * input_sample_rate); 
	if (end == 0) return;
	new_play_selection = FALSE;

	for (i = 0; i < end * 2; i+=2) {
	    sel_ptr[i] = (*mt_ptr);	
	    sel_ptr[i+1] = sel_ptr[i];
	    if (++mt_ptr > end_of_mt_array) mt_ptr = &main_time_buffer[0];	 
	}
	
	ALwritesamps (output_port, selection, end * 2);
	
}

/***************************************************************************/

static	void	open_audio_input (void)

{
	short		badopen = 0;

	if (!input_port) {

	    ALseterrorhandler(0);
	    input_config = ALnewconfig ();
	    ALsetwidth (input_config, AL_SAMPLE_16);
	    ALsetchannels (input_config, input_channels);
	    ALsetqueuesize (input_config, Q_SIZE);
 
	    badopen = ((input_port = ALopenport ("audio_in", "r", input_config)) == 0);
	    if (badopen) {
		system ("inform 'could not open the audio ports'");
		exit (1);
	    }
	}
}

/***************************************************************************/

static 	void	close_audio_input (void)

{
	if (input_port) { ALcloseport (input_port); input_port = 0; }
}

/***************************************************************************/

static	void	open_audio_output (void)

{
	short		badopen = 0;

	if (!output_port) {

	    ALseterrorhandler(0);
	    output_config = ALnewconfig ();
	    ALsetwidth (output_config, AL_SAMPLE_16);
	    ALsetchannels (output_config, input_channels);
	    ALsetqueuesize (output_config, Q_SIZE);
 
	    badopen = ((output_port = ALopenport ("audio_out", "w", output_config)) == 0);
	    if (badopen) {
		system ("inform 'could not open the audio ports'");
		exit (1);
	    }
	}
}

/***************************************************************************/

static 	void	close_audio_output (void)

{
	if (output_port) { ALcloseport (output_port); output_port = 0; }
}

/***************************************************************************/

static	void	set_sample_rate (void)

{
	int	i;
	long	buffer_length = 2;
	long	PVbuffer[2] = {AL_INPUT_RATE, 0};

/* 	set current sample rate		*/

	PVbuffer[1] = (long)(input_sample_rate);
	ALsetparams (AL_DEFAULT_DEVICE, PVbuffer, buffer_length);
	
	input_sample_rate = (float)PVbuffer[1] * SR_CORRECTION;
    
	if (time_menu != NULL) {
	    for (i = 1; i < 8; i++) setpup (time_submenu3, i, PUP_BOX);
	    if (input_sample_rate < 10000) setpup (time_submenu3, 7, PUP_CHECK);
	    else if (input_sample_rate < 12000) setpup (time_submenu3, 6, PUP_CHECK);
	    else if (input_sample_rate < 18000) setpup (time_submenu3, 5, PUP_CHECK); 
	    else if (input_sample_rate < 24000) setpup (time_submenu3, 4, PUP_CHECK);
	    else if (input_sample_rate < 34000) setpup (time_submenu3, 3, PUP_CHECK);
	    else if (input_sample_rate < 46000) setpup (time_submenu3, 2, PUP_CHECK);
	    else setpup (time_submenu3, 1, PUP_CHECK);
	}
	set_tdisplay_parameters ();
	set_fdisplay_parameters ();
	set_sdisplay_parameters ();    
}

/***************************************************************************/

static	void	set_attenuators (void)

{
	long	buffer_length = 4;
	long	PVbuffer[4] = {AL_LEFT_INPUT_ATTEN, 0, AL_RIGHT_INPUT_ATTEN, 0};

	if ((int)input_atten == 0) {
 		PVbuffer[1] = 0, PVbuffer[3] = 0;
		if (line_input) level_scale = LINE_CALIBRATION;
		else 	level_scale = MIC_CALIBRATION;
		amp_scale = (MAX_VOLTAGE_IN / 32768.0);
	}
	else if ((int)input_atten == 10) {
		PVbuffer[1] = 96, PVbuffer[3] = 96;
		if (line_input) level_scale = LINE_CALIBRATION + input_atten - 1.1;
		else level_scale = MIC_CALIBRATION + input_atten - 1.1;
		amp_scale = (MAX_VOLTAGE_IN / 32768.0) * pow (10.0,(input_atten - 1.1)/ 20.0);
	}
	else if ((int)input_atten == 20) {
		PVbuffer[1] = 208, PVbuffer[3] = 208;	
		if (line_input) level_scale = LINE_CALIBRATION + input_atten -0.7;
		else level_scale = MIC_CALIBRATION + input_atten -0.7;
		amp_scale = (MAX_VOLTAGE_IN / 32768.0) * pow (10.0,(input_atten -0.5)/ 20.0);
	}
	ALsetparams (AL_DEFAULT_DEVICE, PVbuffer, buffer_length);
}

/***************************************************************************/

static	void	set_input_source (void)

{
	long	buffer_length = 2;
	long	PVbuffer[2] = {AL_INPUT_SOURCE, 0};

	if (line_input) PVbuffer[1] = AL_INPUT_LINE;
	else PVbuffer[1] = AL_INPUT_MIC;

	ALsetparams (AL_DEFAULT_DEVICE, PVbuffer, buffer_length);
}

/***************************************************************************/

static	void	exit_program (void)

{
	close_audio_input ();
/*	close_audio_output ();	*/	
	gexit ();
	exit (0);
}

/***************************************************************************/

static	void	time_zoom_in (void)

{
	old_time_axis_start[tz_count] = time_axis_start;
	old_time_axis_end[tz_count++] = time_axis_end;
	if (time_selected) {
		time_axis_start = time_selected_start;
		time_axis_end = time_selected_end;
		time_selected = FALSE;
	}
	else {
		time_axis_end = time_axis_start + (time_axis_end - time_axis_start) / 2.0;		

	}
	if ((time_axis_end - time_axis_start) < 0.001 * (48000 / input_sample_rate))
		time_axis_end = time_axis_start + 0.001 * (48000 / input_sample_rate); 
	approx_buffer_size = (time_axis_end - time_axis_start) * (float)input_sample_rate / 4.0;
	if (approx_buffer_size > APPROX_BUFFER_SIZE) approx_buffer_size = APPROX_BUFFER_SIZE;
	set_tdisplay_parameters ();
	draw_display ();
	if (swin.gid) {
	    winset (swin.gid);
	    draw_display ();
	    winset (twin.gid);
	}
	if (pause) {
		process_buffer ();
		plot_time_data ();
		if (swin.gid && spectrogram_on) {   
		    get_spectrum_nrt ();
		}
		if (wwin.gid && waterfall_on) get_waterfall_nrt ();
	}
}

/***************************************************************************/

static	void	time_zoom_out (void)

{
	if (tz_count < 1) {
		tz_count = 0;
		time_axis_end *= 2.0;
		if (time_axis_end > (float)MAX_DISPLAY_SECONDS) 
			time_axis_end = (float)MAX_DISPLAY_SECONDS;
	}
	else {
		time_axis_start = old_time_axis_start[--tz_count];
		time_axis_end = old_time_axis_end[tz_count];
	}
	approx_buffer_size = (time_axis_end - time_axis_start) * (float)input_sample_rate;
	if (approx_buffer_size > APPROX_BUFFER_SIZE) approx_buffer_size = APPROX_BUFFER_SIZE;
	
	set_tdisplay_parameters ();
	draw_display ();
	if (swin.gid) {
	    winset (swin.gid);
	    draw_display ();
	    winset (twin.gid);
	}
	time_selected = FALSE;
	if (pause) {
		process_buffer ();
		plot_time_data ();
		if (swin.gid && spectrogram_on) {
		    get_spectrum_nrt ();
		}
		if (wwin.gid && waterfall_on) get_waterfall_nrt ();	
	}		
}

/***************************************************************************/

static	void	freq_zoom_in (void)

{
	old_freq_axis_start[fz_count] = freq_axis_start;
	old_freq_axis_end[fz_count++] = freq_axis_end;
	if (freq_selected) {
		freq_axis_start = freq_selected_start;
		freq_axis_end = freq_selected_end;
		freq_selected = FALSE;
	}
	else {
		freq_axis_end = freq_axis_start + (freq_axis_end - freq_axis_start) / 2.0;		

	}
	if ((freq_axis_end - freq_axis_start) < 0.001)
		freq_axis_end = freq_axis_start + 0.001;
	set_fdisplay_parameters ();
	draw_display ();
	if (fwin.gid) {calc_freq_data (); plot_freq_data ();}
	if (waterfall_on && pause) get_waterfall_nrt ();
	average_first_time = TRUE;
}

/***************************************************************************/

static	void	freq_zoom_out (void)

{

	if (fz_count < 1) {
		fz_count = 0;
		freq_axis_end *= 2.0;
		if (freq_axis_end > (float)MAX_DISPLAY_KHZ) 
			freq_axis_end = (float)MAX_DISPLAY_KHZ;
	}
	else {
		freq_axis_start = old_freq_axis_start[--fz_count];
		freq_axis_end = old_freq_axis_end[fz_count];
	}
	set_fdisplay_parameters ();
	draw_display ();
	freq_selected = FALSE;
	
	if (fwin.gid) {calc_freq_data (); plot_freq_data ();}
	if (waterfall_on && pause) get_waterfall_nrt ();
	average_first_time = TRUE;			
}

/***************************************************************************/

static	void	save_as_ascii (char* filename)

{
    int		i;
    int		end, components_per_point;
    long	idum, main_time_points;
    float	time, time_increment;
    float	freq, freq_increment;
    FILE	*fptr;
    float	*td_ptr, *end_of_td_array, *pow_ptr;
    SAMPLE	*mt_ptr, *end_of_mt_array;
    float	time_start_old, time_end_old, increment;
    long	fft_points_old, x, y, current_gid;	
    
    fptr = fopen (filename, "w");
    
    if (winget () == twin.gid) {
	if ((!rms_on) && (!envelope_on)) {
	    mt_ptr = main_time_ptr;
	    main_time_points = (long) ((float)input_sample_rate * MAX_DISPLAY_SECONDS);
	    end_of_mt_array = &main_time_buffer[main_time_points-1];
	    mt_ptr -= (long)((last_input_end_time - time_axis_start) * (float)input_sample_rate);
	    if (mt_ptr < &main_time_buffer[0]) {
		idum = (long)end_of_mt_array + sizeof (SAMPLE) - (long)&main_time_buffer[0]; 
		mt_ptr += idum/sizeof (SAMPLE);
	    }
	    samples_per_display = time_points_per_display * samples_per_time_point;
	    
	    time_increment = 1000.0 / (float)input_sample_rate;
	    time = 0.0;
	    for (i = 0; i < samples_per_display; i++) {
		fprintf (fptr, "%6.3f   %f\n", time, (float)*mt_ptr);
		if (++mt_ptr > end_of_mt_array) mt_ptr = &main_time_buffer[0];
		time += time_increment;
	    }
	}	   
	else {    
	
	    if (rms_on) {
		end_of_td_array = &time_display_buffer[time_points_per_display-1];
		td_ptr = time_display_ptr;
		
		time_increment = 1000.0 * (time_axis_end - time_axis_start) / time_points_per_display;
		
		for (i = 0; i < time_points_per_display; i++){
		    fprintf (fptr, "%6.3f   %6.3f\n", time, (float)*td_ptr);	
		    if (++td_ptr > end_of_td_array) td_ptr = &time_display_buffer[0];
		    time += time_increment;
		}
	    }
	    else {
		end_of_td_array = &time_max_buffer[time_points_per_display-1];
		td_ptr = time_max_ptr;
		
		time_increment = 1000.0 * (time_axis_end - time_axis_start) / time_points_per_display;
		
		for (i = 0; i < time_points_per_display; i++){
		    fprintf (fptr, "%6.3f   %f\n", time, (float)*td_ptr);	
		    if (++td_ptr > end_of_td_array) td_ptr = &time_max_buffer[0];
		    time += time_increment;
		}
		
		
		if (samples_per_time_point > 1) {
		
		    fprintf (fptr, "\n\n\n\n");
		    
		    time = 0.0;
		
		    end_of_td_array = &time_min_buffer[time_points_per_display-1];
		    td_ptr = time_min_ptr;
		
		    for (i = 0; i < time_points_per_display; i++){
			fprintf (fptr, "%6.3f   %f\n", time, (float)*td_ptr);	
			if (++td_ptr > end_of_td_array) td_ptr = &time_min_buffer[0];
			time += time_increment;
		    }
		}
	    }
	}

    }
    else if (winget () == fwin.gid) {
    
	idum = (long)(freq_axis_start / kHz_per_component); 
	
	pow_ptr = &power_spectrum[0];	

	components_per_point = (int)((float)components_per_display * 2.0 / ((float)fdis.x_length));
	if (components_per_point < 1) components_per_point = 1;
	pixels_per_freq_point = pixels_per_component * (float)components_per_point;
	kHz_per_freq_point = pixels_per_freq_point / PIXELS_PER_KHZ;

	end = components_per_display;
	if (end > freq_components - (idum + components_per_point*2))
		end = freq_components - (idum + components_per_point*2); 
	 
	freq = freq_axis_start * 1000.0;
	freq_increment = kHz_per_freq_point * 1000.0;
	
	end /= components_per_point;
	
	for (i = 0; i < end; i++, pow_ptr++){
		
	    fprintf (fptr, "%6.3f   %6.3f\n", freq, (float)*pow_ptr);
	    freq += freq_increment;	
	} 
   
    }

    else if (winget () == wwin.gid) {
	
	if (!waterfall_on || !pause) return;
    
	getorigin (&x, &y);
	if ((y + wdis.y_offset_bottom) <= 0) {
	    return; 
	}
    
	time_start_old = time_selected_start;
	time_end_old = time_selected_end;
	fft_points_old = spec_fft_points;
    
	waterfall_nrt = TRUE;
   
	increment = spec_fft_points/input_sample_rate;
    
	time_selected_start = time_axis_start;
	time_selected_end = time_axis_start + increment;
    
	set_fdisplay_parameters ();
	set_sdisplay_parameters ();
    
	for (;time_selected_end < time_axis_end; time_selected_start += increment, time_selected_end += increment) {
	    get_spectrum ();
	    
	    idum = (long)(freq_axis_start / kHz_per_component); 
	    pow_ptr = &freq_data[0];	

	    components_per_point = (int)((float)components_per_display * 2.0 / ((float)fdis.x_length));
	    if (components_per_point < 1) components_per_point = 1;
	    pixels_per_freq_point = pixels_per_component * (float)components_per_point;
	    kHz_per_freq_point = pixels_per_freq_point / PIXELS_PER_KHZ;

	    end = components_per_display;
	    if (end > freq_components - (idum + components_per_point*2))
		end = freq_components - (idum + components_per_point*2); 
	 
	    freq = freq_axis_start * 1000.0;
	    freq_increment = kHz_per_freq_point * 1000.0;
	
	    end /= components_per_point;
	    
	    for (i = 0; i < end; i++, pow_ptr++){	
		*pow_ptr = level_scale + 10 * log10 (*pow_ptr);
		fprintf (fptr, "%6.3f   %6.3f   %6.3f\n", (float)time_selected_start, freq, (float)*pow_ptr);
		freq += freq_increment;	
	    } 
	    
	}
    
	waterfall_nrt = FALSE;
	time_selected_start = time_start_old;
	time_selected_end = time_end_old; 
	spec_fft_points = fft_points_old;
    }
    fclose (fptr);
    
}

/***************************************************************************/

static	void	make_time_menu (void)

{
	int	i;
	
	freepup (time_menu);
	freepup (time_submenu1);
	freepup (time_submenu2);
	freepup (time_submenu3);
	freepup (time_submenu4);
	freepup (time_submenu5);

	time_submenu1 = defpup ("Waveform %x111|Spectrum %x112|Spectrogram %x113|Waterfall %x114");
	for (i = 1; i < 5; i++) setpup (time_submenu1, i, PUP_BOX);
	if (time_on) setpup (time_submenu1, 1, PUP_CHECK);
	if (spectrum_on) setpup (time_submenu1, 2, PUP_CHECK); 
	if (spectrogram_on) setpup (time_submenu1, 3, PUP_CHECK);
	if (waterfall_on) setpup (time_submenu1, 4, PUP_CHECK);
 
	time_submenu4 = defpup ("Volts%x41|Envelope %x42|RMS dB %x43");
	for (i = 1; i < 4; i++) setpup (time_submenu4, i, PUP_BOX);
	if (!rms_on) setpup (time_submenu4, 1, PUP_CHECK);
	else setpup (time_submenu4, 3, PUP_CHECK);
	if (envelope_on) setpup (time_submenu4, 2, PUP_CHECK);

        time_submenu2 = defpup ("Microphone %x21|Line %x22|Soundfile %x23");
	for (i = 1; i < 3; i++) setpup (time_submenu2, i, PUP_BOX);
	if (!line_input) setpup (time_submenu2, 1, PUP_CHECK);
	else setpup (time_submenu2, 2, PUP_CHECK);
	
	time_submenu5 = defpup ("Left %x51|Right %x52");
	for (i = 1; i < 3; i++) setpup (time_submenu5, i, PUP_BOX);
	if (input_channel == 'L') setpup (time_submenu5, 1, PUP_CHECK);
	else setpup (time_submenu5, 2, PUP_CHECK);

	time_submenu3 = defpup ("48 kHz%x31| 44.1 kHz%x32|32 kHz%x33|22.05 kHz%x34|16 kHz%x35|11.025 kHz%x36|8 kHz%x37");
	for (i = 1; i < 8; i++) setpup (time_submenu3, i, PUP_BOX);
	if (input_sample_rate < 10000) setpup (time_submenu3, 7, PUP_CHECK);
	else if (input_sample_rate < 12000) setpup (time_submenu3, 6, PUP_CHECK);
	else if (input_sample_rate < 18000) setpup (time_submenu3, 5, PUP_CHECK); 
	else if (input_sample_rate < 24000) setpup (time_submenu3, 4, PUP_CHECK);
	else if (input_sample_rate < 34000) setpup (time_submenu3, 3, PUP_CHECK);
	else if (input_sample_rate < 46000) setpup (time_submenu3, 2, PUP_CHECK);
	else setpup (time_submenu3, 1, PUP_CHECK);
        
	time_submenu6 = defpup ("Default%x61|Auto%x62|Keep%x63|Off%x64");
	for (i = 1; i < 5; i++) setpup (time_submenu6, i, PUP_BOX);	
	if (dc_correction == DC_DEFAULT) setpup (time_submenu6, 1, PUP_CHECK);
	else if (dc_correction == DC_AUTO) setpup (time_submenu6, 2, PUP_CHECK); 
	else if (dc_correction == DC_KEEP) setpup (time_submenu6, 3, PUP_CHECK);
	else if (dc_correction == DC_OFF) setpup (time_submenu6, 4, PUP_CHECK);
	                                
	time_menu = defpup ("Time Menu %t|Zoom In %x1|Zoom Out %x2|Play %x202|Spectrum %x3|Trigger %x4|Input Source %m|Input Channel %m|Ordinate Units%m|Sample Rate %m|DC Correction%m|Display%m|Save As ASCII %x5",
			time_submenu2, time_submenu5, time_submenu4, time_submenu3, time_submenu6, time_submenu1);	
	setpup (time_menu, 5, PUP_BOX); 
}

/***************************************************************************/

static	void	make_freq_menu (void)

{
	int	i;
	freepup (freq_menu);
	freepup (freq_submenu1);
	freepup (freq_submenu2);
	freepup (freq_submenu3);

	freq_submenu1 = defpup ("Spectral Envelope %x11|Spectrum Level %x12");
	for (i = 1; i < 3; i++) setpup (freq_submenu1, i, PUP_BOX);
	if (!spectrum_level) setpup (freq_submenu1, 1, PUP_CHECK);
	else setpup (freq_submenu1, 2, PUP_CHECK);

	freq_submenu2 = defpup ("64 %x222|128 %x223|256 %x224|512 %x225|1024 %x226|2048 %x227|4096 %x228");
	for (i = 1; i < 8; i++) setpup (freq_submenu2, i, PUP_BOX);
	if (spec_fft_points == 64) setpup (freq_submenu2, 1, PUP_CHECK);
	else if (spec_fft_points == 128) setpup (freq_submenu2, 2, PUP_CHECK);
	else if (spec_fft_points == 256) setpup (freq_submenu2, 3, PUP_CHECK);
	else if (spec_fft_points == 512) setpup (freq_submenu2, 4, PUP_CHECK);
	else if (spec_fft_points == 1024) setpup (freq_submenu2, 5, PUP_CHECK);
	else if (spec_fft_points == 2048) setpup (freq_submenu2, 6, PUP_CHECK);
	else if (spec_fft_points == 4096) setpup (freq_submenu2, 7, PUP_CHECK);

	freq_submenu3 = defpup ("1 %x331|2 %x332|4 %x333|8 %x334|16 %x335|32 %x336");
	for (i = 1; i < 7; i++) setpup (freq_submenu3, i, PUP_BOX);
	if (spec_overlap == 1) setpup (freq_submenu3, 1, PUP_CHECK);
	else if (spec_overlap == 2) setpup (freq_submenu3, 2, PUP_CHECK);
	else if (spec_overlap == 4) setpup (freq_submenu3, 3, PUP_CHECK);
	else if (spec_overlap == 8) setpup (freq_submenu3, 4, PUP_CHECK);
	else if (spec_overlap == 16) setpup (freq_submenu3, 5, PUP_CHECK);
	else if (spec_overlap == 32) setpup (freq_submenu3, 6, PUP_CHECK);
	
	freq_submenu4 = defpup ("Rectangular %x431|Hamming %x432|Hanning %x433|Bartlett %x434|Blackman %x435|Blackman-Harris %x436");
	for (i = 1; i < 7; i++) setpup (freq_submenu4, i, PUP_BOX);
	if (window_type == RECTANGULAR) setpup (freq_submenu4, 1, PUP_CHECK);
	else if (window_type == HAMMING) setpup (freq_submenu4, 2, PUP_CHECK);
	else if (window_type == HANNING) setpup (freq_submenu4, 3, PUP_CHECK);
	else if (window_type == BARTLETT) setpup (freq_submenu4, 4, PUP_CHECK);
	else if (window_type == BLACKMAN) setpup (freq_submenu4, 5, PUP_CHECK);
	else if (window_type == BLACKMAN_HARRIS) setpup (freq_submenu4, 6, PUP_CHECK);

	freq_menu = defpup ("Spectrum Menu%t|Zoom In %x1|Zoom Out %x2|Average %x3|Excitation %x4|Pre-emphasis %x5|Ordinate Units%m|FFT Size %m|FFT Overlap%m|Window Type%m|Display%m|Save As ASCII %x6", 
			    freq_submenu1, freq_submenu2, freq_submenu3, freq_submenu4, time_submenu1);	
	if (spec_average_on) setpup (freq_menu, 3, PUP_CHECK);
	else setpup (freq_menu, 3, PUP_BOX);
	if (excitation_pattern_on) setpup (freq_menu, 4, PUP_CHECK);
	else setpup (freq_menu, 4, PUP_BOX);
	if (enhance_on) setpup (freq_menu, 5, PUP_CHECK);
	else setpup (freq_menu, 5, PUP_BOX);
}

/***************************************************************************/

static	void	make_spec_menu (void)

{
	int	i;
	freepup (spec_menu);
	freepup (spec_submenu3);

	spec_submenu3 = defpup ("Scroll %x31|Roll %x32");
	for (i = 1; i < 3; i++) setpup (spec_submenu3, 1, PUP_BOX);
	if (scrolling_on) setpup (spec_submenu3, 1, PUP_CHECK);
	else setpup (spec_submenu3, 2, PUP_CHECK);
	
	spec_menu = defpup ("Spectrogram Menu%t|Boost %x1|Unboost %x2|Excitation %x202|Pre-emphasis %x3|Smooth %x303|Color %x4|Invert Colors %x5|Plot Mode%m|FFT Size %m|FFT Overlap%m|Window Type%m|Display%m",
			spec_submenu3, freq_submenu2, freq_submenu3, freq_submenu4, time_submenu1);
	setpup (spec_menu, 4, PUP_BOX);
	setpup (spec_menu, 6, PUP_BOX);
	if (smooth_spec == TRUE) setpup (spec_menu, 5, PUP_CHECK);
	else setpup (spec_menu, 5, PUP_BOX); 
	if (excitation_pattern_on) setpup (spec_menu, 3, PUP_CHECK);
	else setpup (spec_menu, 3, PUP_BOX);	
}

/***************************************************************************/

int	argc_clone;
char	***argv_clone;
int	ch_pid, file_ready = FALSE;
char	filename[100];


void	main (int argc, char **argv)

{
	int	i;
	long	dev, menval, idum, x, y;
	short	val, old_val, xpos = 0, ypos = 0, xpos_sel = 0, count = 0;
	short	done = FALSE, old_spec_on = FALSE, old_water_on = FALSE;
	short	ascii_save = FALSE;
	
	argc_clone = argc;
	argv_clone = &argv;
	
	initialise ();
	create_time_window ();
	make_time_menu ();
	make_freq_menu ();
	make_spec_menu ();
	open_audio_input ();
	set_input_source ();
	set_attenuators ();
	
	if (argc == 2) {
	    strcpy (filename, argv[1]);
	    process_filedata (filename, input_channel);
	    if (input_sum == 0) {
		if (input_channel == 'L') {
		    input_channel = 'R';
		    setpup (time_submenu5, 1, PUP_BOX);
		    setpup (time_submenu5, 2, PUP_CHECK);
		}
		else {
		    input_channel = 'L';
		    setpup (time_submenu5, 1, PUP_CHECK);
		    setpup (time_submenu5, 2, PUP_BOX);
    		}
	    }
	    process_filedata (filename, input_channel);
	    set_sample_rate ();
	    pause = TRUE;
	    file_data = TRUE;
	    plot_time_data ();
	}

	while (!done) {
	
		if (file_ready) {
		    if (ascii_save) {
			save_as_ascii (filename);
			ascii_save = FALSE;
		    }
		    else if (process_filedata (filename, input_channel)) {
			if (input_sum == 0) {
			    if (input_channel == 'L') {
				input_channel = 'R';
				setpup (time_submenu5, 1, PUP_BOX);
				setpup (time_submenu5, 2, PUP_CHECK);
			    }
			    else {
				input_channel = 'L';
				setpup (time_submenu5, 1, PUP_CHECK);
				setpup (time_submenu5, 2, PUP_BOX);
			    }
			}
			process_filedata (filename, input_channel);
			set_sample_rate ();
    	    		pause = TRUE;
			file_data = TRUE;
			plot_time_data ();
			if (spectrogram_on) get_spectrum_nrt ();
			if (wwin.gid && waterfall_on) get_waterfall_nrt ();
		    }
		    file_ready = FALSE;
		}
	
		if (!pause) {
			if (ALgetfilled(input_port) > input_buffer_size) { 
				ALreadsamps(input_port, (void *) input_buffer,
						input_buffer_size);
				if (time_on) {
				    process_input ();
				    plot_time_data ();
				}
				if (waterfall_on) {
				    old_val = winget ();  
				    if (old_val != wwin.gid) winset (wwin.gid);
				    getorigin (&x, &y);
				    if ((y + wdis.y_offset_bottom) <= 0) {
					waterfall_on = FALSE;
					old_water_on = TRUE;
					winset (old_val);
				    }
				    else winpop ();
				}
				if (spectrogram_on && waterfall_on) {
				    spectrogram_on = FALSE;
				    old_spec_on = TRUE;
				}
				if (spectrogram_on) {
				    old_val = winget ();
				    winset (swin.gid);
				    winpop ();
				    winset (old_val);
				}
				
				if (spectrogram_on || spectrum_on || waterfall_on) get_spectrum_rt ();
				
				if (old_spec_on) spectrogram_on = TRUE, old_spec_on = FALSE;  
				if (old_water_on) waterfall_on = TRUE, old_water_on = FALSE;
				
				if (triggered) {
					if (++count > (POST_TRIGGER_PROPORTION 
						* time_points_per_display / (float)time_points_per_buffer)){
						count = 0;
						pause = TRUE;
						close_audio_input ();
						triggered = FALSE;
					}
				}  
			}
		}
	/*	if (spectrogram_on && !pause) {
		    old_val = winget ();
		    winset (swin.gid);
		    winpop ();
		    winset (old_val);
		} */
		    	
		if (pause && mouse_within_display (xpos = getvaluator (MOUSEX),
				ypos = getvaluator (MOUSEY)) && getbutton (MIDDLEMOUSE)) {
			draw_cursor (xpos, ypos);	
		}
		if (qtest ()) {

			dev = qread (&val);

			switch (dev) {
			    case ESCKEY:
				exit_program ();
				break;
			    case WINQUIT:
				exit_program ();
				break;
			    case WINSHUT:
				winclose (val);
				if (val == fwin.gid) {
				    fwin.gid = 0;
				    spectrum_on = FALSE;
				    freq_selected = FALSE;
				    setpup (time_submenu1, 2, PUP_BOX);
				}
				if (val == swin.gid) {
				    swin.gid = 0;
				    spectrogram_on = FALSE;
				    setpup (time_submenu1, 3, PUP_BOX);
				}
				if (val == wwin.gid) {
				    wwin.gid = 0;
				    waterfall_on = FALSE;
				    setpup (time_submenu1, 4, PUP_BOX);
				    if (spectrum_on) {
					calc_freq_data ();
					plot_freq_data ();
				    } 
				}
				if (val == twin.gid) exit_program ();
				break;
			    case WINFREEZE:
				break;
			    case WINTHAW:
				break;
			    case LEFTMOUSE:
				if (!val) break;
				if (!process_button (getvaluator (MOUSEX), getvaluator (MOUSEY)))
					deselect ();
				if (pause) {
					while (getbutton (LEFTMOUSE)) {
						if (mouse_within_display 
							(xpos = getvaluator (MOUSEX),
							ypos = getvaluator (MOUSEY))) {
							if (xpos != xpos_sel) highlight_selected (xpos);
							if (winget () == twin.gid) {
							    new_fft_selection = TRUE;
							    new_play_selection = TRUE;									
							}
							xpos_sel = xpos;
						}
					}
				}		
				break;
			    case MIDDLEMOUSE:
				break;
			    case RIGHTMOUSE:
				if (val) {			
					if ((winget () == twin.gid) || ((mouse_within_display 
							(getvaluator (MOUSEX), getvaluator (MOUSEY)) == 2))) {
						if (!pause) close_audio_input ();
						menval = dopup (time_menu);
						switch (menval) {
					    	    case 1:
							time_zoom_in ();
							break;
					    	    case 2:
							time_zoom_out ();
							break;
						    case 202:
							play_selection ();
							break;
						    case 3:
							if (time_selected && pause) get_spectrum ();
							break;
						    case 4:
							if (trigger_on) {
								trigger_on = FALSE;
								setpup (time_menu, 5, PUP_BOX);
							}
							else {
								trigger_on = TRUE;
								setpup (time_menu, 5, PUP_CHECK);
							}
							break;
						    case 5:
							if (winget() != swin.gid){
							    if ((ch_pid = sproc (get_file, PR_SADDR, 0)) < 0) {
								printf ("could not fork\n");
							    }
							    ascii_save = TRUE;
							}
							break;
						    case 111:
							if (!time_on) {
							    time_on = TRUE;
							    setpup (time_submenu1, 1, PUP_CHECK);
							}
							else {
							    time_on = FALSE;
							    setpup (time_submenu1, 1, PUP_BOX);
							}
							break;
						    case 112:
							if (!spectrum_on) {
							    spectrum_on = TRUE;
							    setpup (time_submenu1, 2, PUP_CHECK);
							    if (fwin.gid == 0) {
								    idum = winget ();
								    create_freq_window ();
								    freq_axis_end = (float)input_sample_rate / 2000.0;
								    if (freq_axis_end > 20.0) freq_axis_end = 20.0;
								    set_fdisplay_parameters ();
								    winset (idum);
							    }
							}
							else {
							    spectrum_on = FALSE;
							    setpup (time_submenu1, 2, PUP_BOX);
							}
							break;
						    case 113:
							if (!spectrogram_on) {
							    spectrogram_on = TRUE;
							    setpup (time_submenu1, 3, PUP_CHECK);
							    if (swin.gid == 0) {
								    idum = winget ();
								    create_spec_window ();
								    /* spec_axis_end = (float)input_sample_rate / 2000.0;*/
								    spec_axis_end = 4.0;
								    if (spec_axis_end > 20.0) spec_axis_end = 20.0;
								    set_sdisplay_parameters ();
								    winset (idum);
							    }
							    else if (pause) get_spectrum_nrt ();
							}
							else {
							    spectrogram_on = FALSE;
							    setpup (time_submenu1, 3, PUP_BOX);
							}
							break;
						    case 114:
							if (!waterfall_on) {
							    waterfall_on = TRUE;
							    setpup (time_submenu1, 4, PUP_CHECK);
							    if (wwin.gid == 0) {
								    idum = winget ();
								    create_water_window ();
								    freq_axis_end = (float)input_sample_rate / 2000.0;
								    if (freq_axis_end > 20.0) freq_axis_end = 20.0;
								    winset (idum);
							    }
							    else if (pause) get_waterfall_nrt ();
							}
							else {
							    waterfall_on = FALSE;
							    setpup (time_submenu1, 4, PUP_BOX);
							}
							break;
						    case 21:
							line_input = FALSE;
							setpup (time_submenu2, 1, PUP_CHECK);
							setpup (time_submenu2, 2, PUP_BOX);		
							set_input_source ();
							if (!pause) set_attenuators ();
							break;
						    case 22:
							line_input = TRUE;
							setpup (time_submenu2, 1, PUP_BOX);
							setpup (time_submenu2, 2, PUP_CHECK);		
							set_input_source ();
							if (!pause) set_attenuators ();
							break;
						    case 23:
							if ((ch_pid = sproc (get_file, PR_SADDR, 0)) < 0) {
							    printf ("could not fork\n");
							}
							break;
						    case 51:
							input_channel = 'L';
							if (file_data) {
							    if (process_filedata (filename, input_channel)) {					
								plot_time_data ();
								if (spectrogram_on) get_spectrum_nrt ();
								if (waterfall_on) get_waterfall_nrt ();
							    }
							}
							setpup (time_submenu5, 1, PUP_CHECK);
							setpup (time_submenu5, 2, PUP_BOX);
							break;
						    case 52:
							input_channel = 'R';
							if (file_data) {
							    if (process_filedata (filename, input_channel)) {					
								plot_time_data ();
								if (spectrogram_on) get_spectrum_nrt ();
								if (waterfall_on) get_waterfall_nrt ();
							    }
							}
							setpup (time_submenu5, 1, PUP_BOX);
							setpup (time_submenu5, 2, PUP_CHECK);
							break;
						    case 31:
							input_sample_rate = 48000;
							sample_rate_old = input_sample_rate;
							if (!pause) set_sample_rate ();
							for (i = 1; i < 8; i++) setpup (time_submenu3, i, PUP_BOX);
							setpup (time_submenu3, 1, PUP_CHECK);
							break;
						    case 32:
							input_sample_rate = 44100;
							sample_rate_old = input_sample_rate;
							if (!pause) set_sample_rate ();
							for (i = 1; i < 8; i++) setpup (time_submenu3, i, PUP_BOX);
							setpup (time_submenu3, 2, PUP_CHECK);
							break;
						    case 33:
							input_sample_rate = 32000;
							sample_rate_old = input_sample_rate;
						/*	while (time_axis_end - time_axis_start < 0.0015)
							    time_zoom_out (); */
							if (!pause) set_sample_rate ();
							for (i = 1; i < 8; i++) setpup (time_submenu3, i, PUP_BOX);
							setpup (time_submenu3, 3, PUP_CHECK);
							break;
						    case 34:
							input_sample_rate = 22500;
							sample_rate_old = input_sample_rate;
							if (!pause) set_sample_rate ();
							for (i = 1; i < 8; i++) setpup (time_submenu3, i, PUP_BOX);
							setpup (time_submenu3, 4, PUP_CHECK);
							break;
						    case 35:
							input_sample_rate = 16000;
							sample_rate_old = input_sample_rate;
							if (!pause) set_sample_rate ();
							for (i = 1; i < 8; i++) setpup (time_submenu3, i, PUP_BOX);
							setpup (time_submenu3, 5, PUP_CHECK);
							break;
						    case 36:
							input_sample_rate = 11025;
							sample_rate_old = input_sample_rate;
							if (!pause) set_sample_rate ();
							for (i = 1; i < 8; i++) setpup (time_submenu3, i, PUP_BOX);
							setpup (time_submenu3, 6, PUP_CHECK);
							break;
						    case 37:
							input_sample_rate = 8000;
							sample_rate_old = input_sample_rate;
							if (!pause) set_sample_rate ();
							for (i = 1; i < 8; i++) setpup (time_submenu3, i, PUP_BOX);
							setpup (time_submenu3, 7, PUP_CHECK);
							break;
						    case 41:
							if (rms_on == TRUE) {
							    rms_on = FALSE;
							    setpup (time_submenu4, 1, PUP_CHECK);
							    setpup (time_submenu4, 3, PUP_BOX);
							    change_axis_scales ();
							    draw_display ();
							    process_buffer ();
							    plot_time_data ();
							    if (spectrogram_on && pause) get_spectrum_nrt ();
							    if (waterfall_on && pause) get_waterfall_nrt ();
							}
							break;
						    case 42:
							envelope_on = !envelope_on;
							if (envelope_on) setpup (time_submenu4, 2, PUP_CHECK);
							else setpup (time_submenu4, 2, PUP_BOX);
							process_buffer ();
							plot_time_data ();
							break;
						    case 43:
							if (rms_on == FALSE) {
							    rms_on = TRUE;
							    setpup (time_submenu4, 1, PUP_BOX);
							    setpup (time_submenu4, 3, PUP_CHECK);
							    change_axis_scales ();
							    draw_display ();
							    process_buffer ();
							    plot_time_data ();
							}
							break;
						    case 61:
							dc_correction = DC_DEFAULT;
							for (i = 1; i < 5; i++) setpup (time_submenu6, i, PUP_BOX);	
							setpup (time_submenu6, 1, PUP_CHECK);
							break;
						    case 62:
							dc_correction = DC_AUTO;
							for (i = 1; i < 5; i++) setpup (time_submenu6, i, PUP_BOX);	
							setpup (time_submenu6, 2, PUP_CHECK);
							break;
						    case 63:
							dc_correction = DC_KEEP;
							for (i = 1; i < 5; i++) setpup (time_submenu6, i, PUP_BOX);	
							setpup (time_submenu6, 3, PUP_CHECK);
							break;
						    case 64:
							dc_correction = DC_OFF;
							for (i = 1; i < 5; i++) setpup (time_submenu6, i, PUP_BOX);	
							setpup (time_submenu6, 4, PUP_CHECK);
							break;
						}
						if (!pause) open_audio_input ();
					}
					else if ((winget () == fwin.gid) || ((mouse_within_display 
							(getvaluator (MOUSEX), getvaluator (MOUSEY)) == 3))) {
						if (!pause) close_audio_input ();
						menval = dopup (freq_menu);
						switch (menval) {
					    	    case 1:
							freq_zoom_in ();
							break;
					    	    case 2:
							freq_zoom_out ();
							break;
						    case 3:
							spec_average_on = !spec_average_on;
							average_first_time = TRUE;
							if (spec_average_on) setpup (freq_menu, 3, PUP_CHECK);
							else setpup (freq_menu, 3, PUP_BOX);
							break;
						    case 4:
							excitation_pattern_on = !excitation_pattern_on;
							
							if (pause) {
							    if (fwin.gid) {calc_freq_data (); plot_freq_data ();}
							    if (spectrogram_on) get_spectrum_nrt ();
							    if (waterfall_on) get_waterfall_nrt ();
							}   
							
							if (excitation_pattern_on) setpup (spec_menu, 3, PUP_CHECK);
							else setpup (spec_menu, 3, PUP_BOX);	
							if (excitation_pattern_on) setpup (freq_menu, 4, PUP_CHECK);
							else setpup (freq_menu, 4, PUP_BOX);
							
							break;
						    case 5:
							enhance_on = !enhance_on;
							if (pause) {
							    if (fwin.gid) get_spectrum ();
							    if (spectrogram_on) get_spectrum_nrt ();
							    if (waterfall_on) get_waterfall_nrt ();
							}
							if (enhance_on) {
							    setpup (spec_menu, 4, PUP_CHECK);
							    setpup (freq_menu, 5, PUP_CHECK);
							}
							else {
							    setpup (spec_menu, 4, PUP_BOX);
							    setpup (freq_menu, 5, PUP_BOX);
							}
							break;
						    case 6:
							if ((winget() != swin.gid)){
							    if ((ch_pid = sproc (get_file, PR_SADDR, 0)) < 0) {
								printf ("could not fork\n");
							    }
							    ascii_save = TRUE;
							}
							break;
						    case 111:
							if (!time_on) {
							    time_on = TRUE;
							    setpup (time_submenu1, 1, PUP_CHECK);
							}
							else {
							    time_on = FALSE;
							    setpup (time_submenu1, 1, PUP_BOX);
							}
							break;
						    case 112:
							if (!spectrum_on) {
							    spectrum_on = TRUE;
							    setpup (time_submenu1, 2, PUP_CHECK);
							    if (fwin.gid == 0) {
								    idum = winget ();
								    create_freq_window ();
								    freq_axis_end = (float)input_sample_rate / 2000.0;
								    if (freq_axis_end > 20.0) freq_axis_end = 20.0;
								    set_fdisplay_parameters ();
								    winset (idum);
							    }
							}
							else {
							    spectrum_on = FALSE;
							    setpup (time_submenu1, 2, PUP_BOX);
							}
							break;
						    case 113:
							if (!spectrogram_on) {
							    spectrogram_on = TRUE;
							    setpup (time_submenu1, 3, PUP_CHECK);
							    if (swin.gid == 0) {
								    idum = winget ();
								    create_spec_window ();
								   /* spec_axis_end = (float)input_sample_rate / 2000.0;*/
								    spec_axis_end = 4.0;
								    if (spec_axis_end > 20.0) spec_axis_end = 20.0;
								    set_sdisplay_parameters ();
								    winset (idum);
							    }
							    else if (pause) get_spectrum_nrt ();
							}
							else {
							    spectrogram_on = FALSE;
							    setpup (time_submenu1, 3, PUP_BOX);
							}
							break;
						    case 114:
							if (!waterfall_on) {
							    waterfall_on = TRUE;
							    setpup (time_submenu1, 4, PUP_CHECK);
							    if (wwin.gid == 0) {
								    idum = winget ();
								    create_water_window ();
								    freq_axis_end = (float)input_sample_rate / 2000.0;
								    if (freq_axis_end > 20.0) freq_axis_end = 20.0;
								    winset (idum);
							    }
							    else if (pause) get_waterfall_nrt ();
							}
							else {
							    waterfall_on = FALSE;
							    setpup (time_submenu1, 4, PUP_BOX);
							}
							break;
						    case 11:
							spectrum_level = FALSE;	
							setpup (freq_submenu1, 1, PUP_CHECK);
							setpup (freq_submenu1, 2, PUP_BOX);
							calc_freq_data ();
							plot_freq_data ();	 
							break;
						    case 12:
							spectrum_level = TRUE;	
							setpup (freq_submenu1, 1, PUP_BOX);
							setpup (freq_submenu1, 2, PUP_CHECK);
							calc_freq_data ();
							plot_freq_data ();
							break;
						
						 /* case 21:
							spec_fft_points = 32;
							if (pause && spectrogram_on) get_spectrum_nrt ();
							break; */
						    case 222:
							for (i = 1; i < 9; i++) setpup (freq_submenu2, i, PUP_BOX);
							setpup (freq_submenu2, 1, PUP_CHECK);
							spec_fft_points = 64;
							if (pause && spectrogram_on) get_spectrum_nrt ();
							if (waterfall_on && pause) get_waterfall_nrt ();
							break;
						    case 223:
							for (i = 1; i < 9; i++) setpup (freq_submenu2, i, PUP_BOX);
							setpup (freq_submenu2, 2, PUP_CHECK);
							spec_fft_points = 128;
							if (pause && spectrogram_on) get_spectrum_nrt ();
							if (waterfall_on && pause) get_waterfall_nrt ();
							break;
						    case 224:
							for (i = 1; i < 9; i++) setpup (freq_submenu2, i, PUP_BOX);
							setpup (freq_submenu2, 3, PUP_CHECK);
							spec_fft_points = 256;
							if (pause && spectrogram_on) get_spectrum_nrt ();
							if (waterfall_on && pause) get_waterfall_nrt ();
							break;
						    case 225:
							for (i = 1; i < 9; i++) setpup (freq_submenu2, i, PUP_BOX);
							setpup (freq_submenu2, 4, PUP_CHECK);
							spec_fft_points = 512;
							if (pause && spectrogram_on) get_spectrum_nrt ();
							if (waterfall_on && pause) get_waterfall_nrt ();
							break;
						    case 226:
							for (i = 1; i < 9; i++) setpup (freq_submenu2, i, PUP_BOX);
							setpup (freq_submenu2, 5, PUP_CHECK);
							spec_fft_points = 1024;
							if (pause && spectrogram_on) get_spectrum_nrt ();
							if (waterfall_on && pause) get_waterfall_nrt ();
							break;
						    case 227:
							for (i = 1; i < 9; i++) setpup (freq_submenu2, i, PUP_BOX);
							setpup (freq_submenu2, 6, PUP_CHECK);
							spec_fft_points = 2048;
							if (pause && spectrogram_on) get_spectrum_nrt ();
							if (waterfall_on && pause) get_waterfall_nrt ();
							break;
						    case 228:
							for (i = 1; i < 9; i++) setpup (freq_submenu2, i, PUP_BOX);
							setpup (freq_submenu2, 7, PUP_CHECK);
							spec_fft_points = 4096;
							if (pause && spectrogram_on) get_spectrum_nrt ();
							if (waterfall_on && pause) get_waterfall_nrt ();
							break;
						    case 331:
							for (i = 1; i < 7; i++) setpup (freq_submenu3, i, PUP_BOX);
							setpup (freq_submenu3, 1, PUP_CHECK);
							spec_overlap = 1;
							if (pause && spectrogram_on) get_spectrum_nrt ();
							break;
						    case 332:
							for (i = 1; i < 7; i++) setpup (freq_submenu3, i, PUP_BOX);
							setpup (freq_submenu3, 2, PUP_CHECK);
							spec_overlap = 2;
							if (pause && spectrogram_on) get_spectrum_nrt ();
							break;
						    case 333:
							for (i = 1; i < 7; i++) setpup (freq_submenu3, i, PUP_BOX);
							setpup (freq_submenu3, 3, PUP_CHECK);
							spec_overlap = 4;
							if (pause && spectrogram_on) get_spectrum_nrt ();
							break;
						    case 334:
							for (i = 1; i < 7; i++) setpup (freq_submenu3, i, PUP_BOX);
							setpup (freq_submenu3, 4, PUP_CHECK);
							spec_overlap = 8;
							if (pause && spectrogram_on) get_spectrum_nrt ();
							break;
						    case 335:
							for (i = 1; i < 7; i++) setpup (freq_submenu3, i, PUP_BOX);
							setpup (freq_submenu3, 5, PUP_CHECK);
							spec_overlap = 16;
							if (pause && spectrogram_on) get_spectrum_nrt ();
							break;
						    case 336:
							for (i = 1; i < 7; i++) setpup (freq_submenu3, i, PUP_BOX);
							setpup (freq_submenu3, 6, PUP_CHECK);
							spec_overlap = 32;
							if (pause && spectrogram_on) get_spectrum_nrt ();
							break;
						    case 431:
							for (i = 1; i < 7; i++) setpup (freq_submenu4, i, PUP_BOX);
							setpup (freq_submenu4, 1, PUP_CHECK);
							window_type = RECTANGULAR;
							break;
						    case 432:
							for (i = 1; i < 7; i++) setpup (freq_submenu4, i, PUP_BOX);
							setpup (freq_submenu4, 2, PUP_CHECK);
							window_type = HAMMING;
							break;
						    case 433:
							for (i = 1; i < 7; i++) setpup (freq_submenu4, i, PUP_BOX);
							setpup (freq_submenu4, 3, PUP_CHECK);
							window_type = HANNING;
							break;
						    case 434:
							for (i = 1; i < 7; i++) setpup (freq_submenu4, i, PUP_BOX);
							setpup (freq_submenu4, 4, PUP_CHECK);
							window_type = BARTLETT;
							break;
						    case 435:
							for (i = 1; i < 7; i++) setpup (freq_submenu4, i, PUP_BOX);
							setpup (freq_submenu4, 5, PUP_CHECK);
							window_type = BLACKMAN;
							break;
						    case 436:
							for (i = 1; i < 7; i++) setpup (freq_submenu4, i, PUP_BOX);
							setpup (freq_submenu4, 6, PUP_CHECK);
							window_type = BLACKMAN_HARRIS;
							break;
						}
						if (pause && (menval > 430) && (menval < 437) && (window_type != window_type_old)) {
						    if (spectrogram_on) get_spectrum_nrt ();
						    if (!new_fft_selection) get_spectrum ();
						    if (waterfall_on && pause) get_waterfall_nrt ();    
						}
						if (!pause) open_audio_input ();
					}
					else if (winget () == swin.gid) {
						if (!pause) close_audio_input ();
						menval = dopup (spec_menu);
						switch (menval) {
						    case 1:
							spec_boost *= ROOT_TWO;
							if (pause && spectrogram_on) get_spectrum_nrt ();
							break;
						    case 2:
							spec_boost /= ROOT_TWO;
							if (pause && spectrogram_on) get_spectrum_nrt ();
							break;
						    case 202:
							excitation_pattern_on = !excitation_pattern_on;
							
							if (pause) {
							    if (fwin.gid) {calc_freq_data (); plot_freq_data ();}
							    if (spectrogram_on) get_spectrum_nrt ();
							    if (waterfall_on) get_waterfall_nrt ();
							}   
							
							if (excitation_pattern_on) setpup (spec_menu, 3, PUP_CHECK);
							else setpup (spec_menu, 3, PUP_BOX);	
							if (excitation_pattern_on) setpup (freq_menu, 4, PUP_CHECK);
							else setpup (freq_menu, 4, PUP_BOX);
							
							break;
						    case 3:
							if (enhance_on) {
							    enhance_on = FALSE;
							    setpup (spec_menu, 4, PUP_BOX);
							    setpup (freq_menu, 5, PUP_BOX);
							    if (pause) {
								if (fwin.gid) get_spectrum ();
								if (spectrogram_on) get_spectrum_nrt ();
								if (waterfall_on) get_waterfall_nrt ();
							    }
							}
							else {
							    enhance_on = TRUE;
							    setpup (spec_menu, 4, PUP_CHECK);
							    setpup (freq_menu, 5, PUP_CHECK);
							    if (pause) {
								if (fwin.gid) get_spectrum ();
								if (spectrogram_on) get_spectrum_nrt ();
								if (waterfall_on) get_waterfall_nrt ();
							    }
							}
							break;
						    case 303:
							smooth_spec = !smooth_spec;
							if (smooth_spec == TRUE) setpup (spec_menu, 5, PUP_CHECK);
							else setpup (spec_menu, 5, PUP_BOX);
							if (pause && spectrogram_on) get_spectrum_nrt ();
							break;
						    case 4:
							if (color_on) {
							    color_on = FALSE;
							    setpup (spec_menu, 6, PUP_BOX);
							    if (pause && spectrogram_on) get_spectrum_nrt ();
							}
							else {
							    color_on = TRUE;
							    setpup (spec_menu, 6, PUP_CHECK);
							    if (pause && spectrogram_on) get_spectrum_nrt ();
							}
							break;
						    case 5:
							invert_colors = !invert_colors;
							clear_display ();
							if (pause && spectrogram_on) get_spectrum_nrt ();
							break;
						    case 111:
							if (!time_on) {
							    time_on = TRUE;
							    setpup (time_submenu1, 1, PUP_CHECK);
							}
							else {
							    time_on = FALSE;
							    setpup (time_submenu1, 1, PUP_BOX);
							}
							break;
						    case 112:
							if (!spectrum_on) {
							    spectrum_on = TRUE;
							    setpup (time_submenu1, 2, PUP_CHECK);
							    if (fwin.gid == 0) {
								    idum = winget ();
								    create_freq_window ();
								    freq_axis_end = (float)input_sample_rate / 2000.0;
								    if (freq_axis_end > 20.0) freq_axis_end = 20.0;
								    set_fdisplay_parameters ();
								    winset (idum);
							    }
							}
							else {
							    spectrum_on = FALSE;
							    setpup (time_submenu1, 2, PUP_BOX);
							}
							break;
						    case 113:
							if (!spectrogram_on) {
							    spectrogram_on = TRUE;
							    setpup (time_submenu1, 3, PUP_CHECK);
							    if (swin.gid == 0) {
								    idum = winget ();
								    create_spec_window ();
								    /* spec_axis_end = (float)input_sample_rate / 2000.0; */
								    spec_axis_end = 4.0;
								    if (spec_axis_end > 20.0) spec_axis_end = 20.0;
								    set_sdisplay_parameters ();
								    winset (idum);
							    }
							    else if (pause) get_spectrum_nrt ();
							}
							else {
							    spectrogram_on = FALSE;
							    setpup (time_submenu1, 3, PUP_BOX);
							}
							break;
						    case 114:
							if (!waterfall_on) {
							    waterfall_on = TRUE;
							    setpup (time_submenu1, 4, PUP_CHECK);
							    if (wwin.gid == 0) {
								    idum = winget ();
								    create_water_window ();
								    freq_axis_end = (float)input_sample_rate / 2000.0;
								    if (freq_axis_end > 20.0) freq_axis_end = 20.0;
								    winset (idum);
							    }
							    else if (pause) get_waterfall_nrt ();
							}
							else {
							    waterfall_on = FALSE;
							    setpup (time_submenu1, 4, PUP_BOX);
							}
							break;
						   /* case 11:
							spec_fft_points = 32;
							if (pause && spectrogram_on) get_spectrum_nrt ();
							break; */
						    case 222:
							for (i = 1; i < 9; i++) setpup (freq_submenu2, i, PUP_BOX);
							setpup (freq_submenu2, 1, PUP_CHECK);
							spec_fft_points = 64;
							if (pause && spectrogram_on) get_spectrum_nrt ();
							if (waterfall_on && pause) get_waterfall_nrt ();
							break;
						    case 223:
							for (i = 1; i < 9; i++) setpup (freq_submenu2, i, PUP_BOX);
							setpup (freq_submenu2, 2, PUP_CHECK);
							spec_fft_points = 128;
							if (pause && spectrogram_on) get_spectrum_nrt ();
							if (waterfall_on && pause) get_waterfall_nrt ();
							break;
						    case 224:
							for (i = 1; i < 9; i++) setpup (freq_submenu2, i, PUP_BOX);
							setpup (freq_submenu2, 3, PUP_CHECK);
							spec_fft_points = 256;
							if (pause && spectrogram_on) get_spectrum_nrt ();
							if (waterfall_on && pause) get_waterfall_nrt ();
							break;
						    case 225:
							for (i = 1; i < 9; i++) setpup (freq_submenu2, i, PUP_BOX);
							setpup (freq_submenu2, 4, PUP_CHECK);
							spec_fft_points = 512;
							if (pause && spectrogram_on) get_spectrum_nrt ();
							if (waterfall_on && pause) get_waterfall_nrt ();
							break;
						    case 226:
							for (i = 1; i < 9; i++) setpup (freq_submenu2, i, PUP_BOX);
							setpup (freq_submenu2, 5, PUP_CHECK);
							spec_fft_points = 1024;
							if (pause && spectrogram_on) get_spectrum_nrt ();
							if (waterfall_on && pause) get_waterfall_nrt ();
							break;
						    case 227:
							for (i = 1; i < 9; i++) setpup (freq_submenu2, i, PUP_BOX);
							setpup (freq_submenu2, 6, PUP_CHECK);
							spec_fft_points = 2048;
							if (pause && spectrogram_on) get_spectrum_nrt ();
							if (waterfall_on && pause) get_waterfall_nrt ();
							break;
						    case 228:
							for (i = 1; i < 9; i++) setpup (freq_submenu2, i, PUP_BOX);
							setpup (freq_submenu2, 7, PUP_CHECK);
							spec_fft_points = 4096;
							if (pause && spectrogram_on) get_spectrum_nrt ();
							if (waterfall_on && pause) get_waterfall_nrt ();
							break;
						    case 331:
							for (i = 1; i < 7; i++) setpup (freq_submenu3, i, PUP_BOX);
							setpup (freq_submenu3, 1, PUP_CHECK);
							spec_overlap = 1;
							if (pause && spectrogram_on) get_spectrum_nrt ();
							break;
						    case 332:
							for (i = 1; i < 7; i++) setpup (freq_submenu3, i, PUP_BOX);
							setpup (freq_submenu3, 2, PUP_CHECK);
							spec_overlap = 2;
							if (pause && spectrogram_on) get_spectrum_nrt ();
							break;
						    case 333:
							for (i = 1; i < 7; i++) setpup (freq_submenu3, i, PUP_BOX);
							setpup (freq_submenu3, 3, PUP_CHECK);
							spec_overlap = 4;
							if (pause && spectrogram_on) get_spectrum_nrt ();
							break;
						    case 334:
							for (i = 1; i < 7; i++) setpup (freq_submenu3, i, PUP_BOX);
							setpup (freq_submenu3, 4, PUP_CHECK);
							spec_overlap = 8;
							if (pause && spectrogram_on) get_spectrum_nrt ();
							break;
						    case 335:
							for (i = 1; i < 7; i++) setpup (freq_submenu3, i, PUP_BOX);
							setpup (freq_submenu3, 5, PUP_CHECK);
							spec_overlap = 16;
							if (pause && spectrogram_on) get_spectrum_nrt ();
							break;
						    case 336:
							for (i = 1; i < 7; i++) setpup (freq_submenu3, i, PUP_BOX);
							setpup (freq_submenu3, 6, PUP_CHECK);
							spec_overlap = 32;
							if (pause && spectrogram_on) get_spectrum_nrt ();
							break;
						    case 31:
							scrolling_on = TRUE;
							setpup (spec_submenu3, 1, PUP_CHECK);
							setpup (spec_submenu3, 2, PUP_BOX);
							break;
						    case 32:
							scrolling_on = FALSE;
							setpup (spec_submenu3, 1, PUP_BOX);
							setpup (spec_submenu3, 2, PUP_CHECK);
							break;
						    case 431:
							for (i = 1; i < 7; i++) setpup (freq_submenu4, i, PUP_BOX);
							setpup (freq_submenu4, 1, PUP_CHECK);
							window_type = RECTANGULAR;
							break;
						    case 432:
							for (i = 1; i < 7; i++) setpup (freq_submenu4, i, PUP_BOX);
							setpup (freq_submenu4, 2, PUP_CHECK);
							window_type = HAMMING;
							break;
						    case 433:
							for (i = 1; i < 7; i++) setpup (freq_submenu4, i, PUP_BOX);
							setpup (freq_submenu4, 3, PUP_CHECK);
							window_type = HANNING;
							break;
						    case 434:
							for (i = 1; i < 7; i++) setpup (freq_submenu4, i, PUP_BOX);
							setpup (freq_submenu4, 4, PUP_CHECK);
							window_type = BARTLETT;
							break;
						    case 435:
							for (i = 1; i < 7; i++) setpup (freq_submenu4, i, PUP_BOX);
							setpup (freq_submenu4, 5, PUP_CHECK);
							window_type = BLACKMAN;
							break;
						    case 436:
							for (i = 1; i < 7; i++) setpup (freq_submenu4, i, PUP_BOX);
							setpup (freq_submenu4, 6, PUP_CHECK);
							window_type = BLACKMAN_HARRIS;
							break;
						}
						if (pause && (menval > 430) && (menval < 437) && (window_type != window_type_old)) {
						    if (spectrogram_on) get_spectrum_nrt ();
						    if (!new_fft_selection) get_spectrum ();  
						    if (waterfall_on) get_waterfall_nrt ();  
						}
						if (!pause) open_audio_input ();
					}
				}
				break;
			    case SPACEKEY:
			    	if (val) {
					pause = !pause;
					if (pause) {
					    close_audio_input ();
					    process_buffer ();
					    average_first_time = TRUE;
					}
					else {
					    open_audio_input ();
					    input_sample_rate = sample_rate_old;
					    set_sample_rate ();
					    set_attenuators ();
					    file_data = FALSE;
					}
				}
				break; 
			    case REDRAW:
				old_val = winget (); 
			    	winset (val); 
				draw_display ();
				winset (old_val); 
				if ((pause) && (val == twin.gid)) plot_time_data ();
				if ((pause) && (val == swin.gid) && spectrogram_on) get_spectrum_nrt ();
				if ((pause) && (val == wwin.gid) &&waterfall_on) get_waterfall_nrt ();
				if (val == fwin.gid) {
				    if (waterfall_on) {
					calc_freq_data ();
				    }
				    plot_freq_data ();
				}
				
				winset (old_val);
				break; 
			    case INPUTCHANGE:
				if (val) {
					winset (val);
				/*	winpop (); */ 
				}
				break;	
			}
		}
	}
}

/***************************************************************************/

#define	    MEDIUM_FONT			"-adobe-helvetica-bold-r-normal--18-0-0-0-p-0-iso8859-1"
#define	    MEDIUM_TAG			"medium_tag"
#define	    FILE_BOX_COLOR		14		
#define	    FILE_BOX_LIST_COLOR		17
#define	    TEXT_BACKGROUND_COLOR	17
#define	    TEXT_COLOR			BLACK
#define	    TEXT_HIGHLIGHT_COLOR	3

void	get_file ()

{
	int		n;
	Arg		args[20];
	char		*dir_text;
	Widget		file_selection_shell, file_selection_box, wid;
	XtAppContext    app_context;
	Display		*display;
	XmFontList	medium_font_list;
	XFontStruct	*font;
	XmFontListEntry font_list_entry;
	XtCallbackRec   ok_callback_list[]={
			{get_file_callback, 0}, 
			{NULL, NULL}
		    };
	XtCallbackRec   cancel_callback_list[]={
			{cancel_callback, 0}, 
			{NULL, NULL}
		    };
	static	int	first_time = TRUE;
    
	if (first_time) {
	    XtToolkitInitialize ();
	    first_time = FALSE;
	}
	
	app_context = XtCreateApplicationContext ();
    
	display = XtOpenDisplay (app_context, NULL, "cspect", "cspect", NULL, 0, &argc_clone, *argv_clone);
	file_selection_shell = XtAppCreateShell ("File Selection", "Cspect", 
		    applicationShellWidgetClass, display, NULL, 0);
	
	
	font = XLoadQueryFont (display, MEDIUM_FONT);
	font_list_entry = XmFontListEntryCreate (MEDIUM_TAG, XmFONT_IS_FONT, (XtPointer) font);
	medium_font_list = XmFontListAppendEntry (NULL, font_list_entry);
	XmFontListEntryFree (&font_list_entry);
	
	dir_text = XmStringCreate ("/usr/people/chrisp", MEDIUM_TAG);
	n = 0;
/*	XtSetArg (args[n], XmNdirectory, dir_text); n++;	    */
	XtSetArg (args[n], XmNokCallback, ok_callback_list); n++;
	XtSetArg (args[n], XmNcancelCallback, cancel_callback_list); n++;
	XtSetArg (args[n], XmNbuttonFontList, medium_font_list); n++;
	XtSetArg (args[n], XmNlabelFontList, medium_font_list); n++;
	XtSetArg (args[n], XmNtextFontList, medium_font_list); n++;
	XtSetArg (args[n], XmNbackground, FILE_BOX_COLOR); n++;
	XtSetArg (args[n], XmNwidth, 500); n++;
	XtSetArg (args[n], XmNheight, 500); n++;
	
	file_selection_box = XmCreateFileSelectionBox (file_selection_shell, "file box", args, n);
	wid = XmFileSelectionBoxGetChild (file_selection_box, XmDIALOG_DIR_LIST);
	n=0;
	XtSetArg (args[n], XmNbackground, FILE_BOX_LIST_COLOR); n++;
	XtSetArg (args[n], XmNforeground, TEXT_COLOR); n++;
	XtSetArg (args[n], XmNhighlightColor, TEXT_HIGHLIGHT_COLOR); n++;
	XtSetValues (wid, args, n);
	wid = XmFileSelectionBoxGetChild (file_selection_box, XmDIALOG_LIST);
	XtSetValues (wid, args, n);
	wid = XmFileSelectionBoxGetChild (file_selection_box, XmDIALOG_FILTER_TEXT);
	XtSetValues (wid, args, n);
	wid = XmFileSelectionBoxGetChild (file_selection_box, XmDIALOG_TEXT);
	XtSetValues (wid, args, n);
	wid = XmFileSelectionBoxGetChild (file_selection_box, XmDIALOG_HELP_BUTTON);
	XtDestroyWidget (wid);
	XtManageChild (file_selection_box);
	XtRealizeWidget (file_selection_shell);
	XtAppMainLoop (app_context); 

}

/***************************************************************************/

void	get_file_callback (Widget wid, caddr_t client_data, 
				    XmFileSelectionBoxCallbackStruct* call_data)

{  
    strcpy (filename, XmCvtXmStringToCT(call_data->value));
    file_ready = TRUE;
    exit (ch_pid);   
}

/***************************************************************************/

void	cancel_callback (Widget wid, caddr_t client_data, caddr_t call_data)

{
    exit (ch_pid);   
}

/***************************************************************************/
